// Repetier-Server Touchscreen Compile Settings
// --------------------------------------------
// This is the target directory for the created files.
// Point the shadow directory to this directory for quick testing.
// You can not change it for resulting app/register - these paths are
// hardcoded in angular.json outputPath properties (4 times).
const shadow = './shadow'

// --------- Do not change anything below this line -------------

const target = shadow + '/modules/front2'
const cssSource = './projects/style'

const {series, parallel, watch, src, dest} = require('gulp')
const cleanCSS = require('gulp-clean-css')
sass_compiler = require('node-sass')
const sass = require('gulp-sass')(sass_compiler)
const shell = require('gulp-shell')

function compileCss() {
  return src('./projects/style/style/**/*.scss')
    .pipe(sass().on('error', sass.logError))
    .pipe(dest('./projects/style/css'))
    .pipe(cleanCSS({compatibility: '*'}))
    .pipe(dest('./projects/style/cssmin'))
}

function copycssProd1 () {
  return src([cssSource + '/fonts'])
    .pipe(dest(target + '/www'))
}

function copycssProd2 () {
  return src([cssSource + '/cssmin/**/*'])
    .pipe(dest(target + '/www/css/'))
}

function compileBinding () {
  return shell.task('ng build binding')()
}

function compileFrontend () {
  return shell.task('ng build front-ui')()
}

function compilePages () {
  return shell.task('ng build pages')()
}

 function compileAppDev () {
  return shell.task('ng build app')()
}

 function compileAppProd () {
  return  shell.task('ng build app --configuration production')()
}

 function compileRegisterDev () {
  return  shell.task('ng build register')()
}

 function compileRegisterProd () {
  return  shell.task('ng build register --configuration production')()
}

const copyCssProdFunc = parallel(copycssProd1, copycssProd2)
const compileLibs = series(compileBinding, compileFrontend, compilePages)
const compileDev = series(compileAppDev, compileRegisterDev)
const compileProd = series(compileAppProd, compileRegisterProd)
const all = series(compileLibs, compileAppProd, compileRegisterProd, compileCss, copyCssProdFunc)
const allDev = series(compileLibs, compileAppDev, compileRegisterDev, compileCss, copyCssProdFunc)
exports['watch-css'] = function () {
  watch('./projects/style/style/**/*.scss', series(compileCss, copyCssProdFunc))
}
const defaultAction = series([compileCss, copyCssProdFunc])

exports.default = defaultAction
exports.css = defaultAction
exports.all = all
exports['all-dev'] = allDev
exports.libs = compileLibs
exports["app-dev"] = compileDev
exports['app-prod'] = compileProd
