/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

export interface GroupState {
  id: string; // internal name
  name: string; // Shown name
}

export interface LastPrintFile {
  name: string;
  time: number;
  size: number;
}

export interface GCodeFileState {
  group: string;
  analysed: number;
  created: number;
  error: boolean;
  extruderUsage: number[];
  filamentTotal: number;
  id: number;
  repeat: number;
  layer: number;
  length: number;
  lines: number;
  printed: number;
  slicer: string;
  processing: boolean;
  printTime: number;
  lastPrintTime: number;
  printTimeBest: number;
  name: string;
  list: string;
  state: string;
  volumeTotal: number;
  volumeUsage: number[];
  volumetric: boolean;
  type: number;
  ok: boolean;
  imgL: string;
  imgM: string;
  imgT: string;

  currency: string;
  weightTotal: number;
  total: number;
  perExtruder: any[];

  slug: string;
  xMin: number;
  xMax: number;
  yMin: number;
  yMax: number;
  zMin: number;
  zMax: number;
  xMinMove: number;
  xMaxMove: number;
  yMinMove: number;
  yMaxMove: number;
  xMinView: number;
  xMaxView: number;
  yMinView: number;
  yMaxView: number;
  radius: number;
  fits: boolean;
  version: number;
}


export const INITIAL_GCODE_FILE_STATE: GCodeFileState = {
  group: "*",
  analysed: 0,
  created: 0,
  error: false,
  extruderUsage: [],
  filamentTotal: 0,
  id: -1,
  repeat: 1,
  layer: 0,
  length: 0,
  lines: 0,
  printed: 0,
  slicer: "",
  processing: false,
  printTime: 0,
  lastPrintTime: 0,
  printTimeBest: 0,
  name: "",
  list: "",
  state: "",
  volumeTotal: 0,
  volumeUsage: [],
  volumetric: false,
  type: 0,
  ok: true,
  imgL: "",
  imgM: "",
  imgT: "",

  currency: "",
  weightTotal: 0,
  total: 0,
  perExtruder: [],

  slug: "",
  xMin: 0,
  xMax: 0,
  yMin: 0,
  yMax: 0,
  zMin: 0,
  zMax: 0,
  xMinMove: 0,
  xMaxMove: 0,
  yMinMove: 0,
  yMaxMove: 0,
  xMinView: 0,
  xMaxView: 0,
  yMinView: 0,
  yMaxView: 0,
  radius: 0,
  fits: true,
  version: 0
};

export interface TemperatureDataState {
  time: number;
  temp: number;
  tempSet: number;
  output: number;
}

export interface ToolState {
  id: number;
  currentTemp: number;
  tempSet: number;
  output: number;
  error: number;
  tempHistory: TemperatureDataState[];
}

export const INITAL_TOOL_STATE: ToolState = {
  id: -1,
  currentTemp: 0,
  tempSet: 0,
  output: 0,
  error: 0,
  tempHistory: []
};

export interface LogState {
  logACK: boolean;
  logCommands: boolean;
  logM105: boolean;
  paused: boolean;
  autoscroll: boolean;
  lastLogStart: number;
  maxLogLines: number;
  lastText: string;
  repeat: number;
  history: string[];
}

export const INITIAL_LOG_STATE: LogState = {
  logACK: false,
  logCommands: false,
  logM105: true,
  paused: false,
  autoscroll: true,
  lastLogStart: 0,
  maxLogLines: 500,
  lastText: "",
  repeat: 0,
  history: []
};

export interface QuickCommandConfig {
  command: string
  name:string
  icon:string
  editPermission:string
  seePermission:string
  visibleWhenPrinting:boolean
  condition:string
}

export interface WizardCommandConfig {
  command: string
  name:string
  icon:string
  author:string
  description:string
  editPermission:string
  seePermission:string
  visibleWhenPrinting:boolean
  condition:string
  uuid:string
  source:number
  version:number
}

export interface FanState {
  on: boolean;
  voltage: number;
  percent: number;
}

export const INITIAL_FAN_STATE: FanState = {
  on: false,
  voltage: 0,
  percent: 0
};
export interface FanConfig {
  alias: string
  fanType: number
  firmwareId: number
  firmwareName: string
  max: number
  min: number
  pos: number
  readOnly: boolean
}
export interface ServerDialogInput {
  caption: string
  dataType: string
  defaultValue:string
  dropdownData: Array<any>
  inputType: number
  labelStyles: number
  marginBottom: string
  marginTop: string
  maxValue: number
  minValue: number
  name: string
  readonly: boolean
}
export interface ServerDialog {
  id: number;
  title: string;
  message: string;
  choices: Array<string>;
  printer: string;
  dialogType: number;
  icon: string;
  input: Array<ServerDialogInput>
}

export interface EEPROMValue {
  pos: string;
  text: string;
  value: string;
  valueOrig: string;
  textNU: string;
  unit: string;
  type: string;
  class: string;
}
export interface HeaterConfig {
  temperatures: Array<{name:string, temp:number}>
alias: string
  firmwareName: string
  lastTemp: number
  maxTemp: number
  offset: number
  cooldownPerSecond: number
  heatupPerSecond: number
  heaterType: number
  supportTemperature: boolean
  sensorType: number
  unit: string
}
export interface PrinterConfig {
  general: {
    active: boolean,
    defaultVolumetric: boolean,
    deleteJobAfterManualStop: boolean,
    doorHandling: number,
    eepromType: string,
    enableQueue: boolean,
    firmwareName: string,
    firmwareStyle: string,
    heatedBed: boolean,
    logHistory: boolean,
    manufacturer: string,
    model: string,
    name: string,
    numFans: number,
    pauseHandling: number,
    pauseSeconds: number,
    printerHomepage: string,
    printerManual: string,
    printerVariant: string,
    sdcard: boolean,
    slug: string,
    softwareLight: boolean,
    softwarePower: boolean,
    tempUpdateEvery: number,
    useModelFromSlug: string,
    useOwnModelRepository: boolean
  };
  extruders: Array<any>;
  buttonCommands: Array<any>;
  connection: any;
  fanPresets: Array<any>;
  fans: Array<FanConfig>;
  flowPresets: Array<any>;
  gcodeReplacements: Array<any>;
  heatedBeds: Array<HeaterConfig>;
  heatedChambers: Array<HeaterConfig>;
  isZBelt: boolean;
  movement: any;
  properties: any;
  quickCommands: Array<QuickCommandConfig>;
  recover: any;
  responseEvents: Array<any>;
  shape: any;
  speedPresets: Array<any>;
  webcams: Array<any>;
  wizardCommands: Array<WizardCommandConfig>;
  wizardTemplates: Array<any>;
}

export const INITIAL_PRINTER_CONFIG: PrinterConfig = {
  general: {
    active: true,
    defaultVolumetric: false,
    deleteJobAfterManualStop: false,
    doorHandling: 1,
    eepromType: "none",
    enableQueue: true,
    firmwareName: "",
    firmwareStyle: "",
    heatedBed: true,
    logHistory: true,
    manufacturer: "",
    model: "",
    name: "",
    numFans: 0,
    pauseHandling: 0,
    pauseSeconds: 120,
    printerHomepage: "",
    printerManual: "",
    printerVariant: "cartesian",
    sdcard: false,
    slug: "",
    softwareLight: false,
    softwarePower: false,
    tempUpdateEvery: 1,
    useModelFromSlug: "",
    useOwnModelRepository: true
  },
  extruders: [],
  buttonCommands: [],
  connection: {},
  fanPresets: [],
  fans: [],
  flowPresets: [],
  gcodeReplacements: [],
  heatedBeds: [],
  heatedChambers: [],
  isZBelt: false,
  movement: {},
  properties: {},
  quickCommands: [],
  recover: {},
  responseEvents: [],
  shape: {},
  speedPresets: [],
  webcams: [],
  wizardCommands: [],
  wizardTemplates: [],
};

export interface PrinterState {
  online: number;
  name: string;
  active: boolean;
  slug: string; // short printer identifier
  printing: boolean;
  paused: boolean;
  pauseState: number;
  idle: boolean; // true if moves etc. are allowed
  printJob: string; // name of running printjob
  printDone: number; // percentage of print finished
  jobid: number;
  jobstate: string;
  ofLayer: number;
  printedTimeComp: number;
  printedTime: number;
  stateTime: number;
  startTime: number;
  config: PrinterConfig;
  scripts: { [key: string]: string };
  exists: boolean;
  posX: number;
  posY: number;
  posZ: number;
  posE: number;
  posDE: number;
  f: number;
  fans: FanState[];
  /* fanPercent: number;
  fan: number;
  fanOn: boolean;
  fan2Percent: number;
  fan2: number;
  fan2On: boolean; */
  debugLevel: number;
  hasXHome: boolean;
  hasYHome: boolean;
  hasZHome: boolean;
  powerOn: boolean;
  sdcardMounted: boolean;
  speedMultiply: number;
  flowMultiply: number;
  firmware: string;
  firmwareURL: string;
  layer: number;
  doorOpen: boolean;
  shutdownAfterPrint: boolean;
  numExtruder: number;
  extruders: ToolState[];
  activeExtruderId: number;
  heatedBeds: ToolState[];
  heatedChambers: ToolState[];
  rec: boolean; // webcam recording
  lightsOn: boolean;
  volumetric: boolean;
  queue: GCodeFileState[];
  // models: {[key: number]: GCodeFileState};
  modelsFiltered: GCodeFileState[];
  allModels: GCodeFileState[];
  imageUrls: { [key: string]: string };
  sortBy: number; // Sort criteria for gcodes
  modelFilter: string; // filter for visible models
  activeGroupId: string; // activated group name
  groups: { [key: string]: GroupState };
  groupsSorted: GroupState[]; // Sorted list of keys
  groupsSortedNoAll: GroupState[];
  xyzStepSize: number; // move amount for arrow buttons
  eStepSize: number; // move amount for arrow buttons
  retractSpeed: number;
  extrudeSpeed: number;
  gcodeViewPage: number; // Can be used to page through gcode list
  hasFirmwareSettings: boolean;
  firmwareSettings: any;
  log: LogState;
  eeprom: EEPROMValue[];
  eepromErrors: boolean;
  recover: any;
  dialogs: ServerDialog[];
  lastPrints: LastPrintFile[];
  firmwareStyle: string;
  condition: number;
  conditionReason: string;
  perm: { [key: string]: string };
  gperm: { [key: string]: string };
  global: { [key: string]: string };
  sglobal: { [key: string]: string };
  visibleWizards: Record<string,boolean>;
  visibleQuickCommands: Record<string,boolean>;
}

export const INITIAL_PRINTER_STATE: PrinterState = {
  online: 0,
  name: "",
  active: false,
  slug: "",
  printing: false,
  paused: false,
  pauseState: 0,
  idle: true,
  printJob: "",
  printDone: 0,
  jobid: 0,
  jobstate: "",
  ofLayer: 0,
  printedTimeComp: 0,
  printedTime: 0,
  stateTime: 0,
  startTime: 0,
  config: Object.assign({}, INITIAL_PRINTER_CONFIG),
  scripts: {},
  exists: false,
  posX: 0,
  posY: 0,
  posZ: 0,
  posE: 0,
  posDE: 0,
  f: 60,
  activeExtruderId: 0,
  fans: [],
  /* fanPercent: 0,
  fan: 0,
  fanOn: false,
  fan2Percent: 0,
  fan2: 0,
  fan2On: false, */
  debugLevel: 0,
  hasXHome: true,
  hasYHome: true,
  hasZHome: true,
  powerOn: false,
  sdcardMounted: false,
  speedMultiply: 100,
  flowMultiply: 100,
  firmware: "",
  firmwareURL: "",
  layer: 0,
  doorOpen: false,
  shutdownAfterPrint: false,
  numExtruder: 0,
  extruders: [],
  heatedBeds: [],
  heatedChambers: [],
  rec: false,
  lightsOn: false,
  volumetric: false,
  queue: [],
  modelsFiltered: [],
  allModels: [],
  imageUrls: {},
  sortBy: 2,
  modelFilter: "",
  groups: {},
  activeGroupId: "#",
  groupsSorted: [],
  groupsSortedNoAll: [],
  xyzStepSize: 10, // move amount for arrow buttons
  eStepSize: 10, // move amount for arrow buttons
  retractSpeed: 20,
  extrudeSpeed: 2,
  gcodeViewPage: 0, // Can be used to page through gcode list
  hasFirmwareSettings: false,
  firmwareSettings: {},
  eeprom: [],
  eepromErrors: false,
  recover: {state: 0},
  log: INITIAL_LOG_STATE,
  dialogs: [],
  lastPrints: [],
  firmwareStyle: "marlin",
  condition: 5,
  conditionReason: "",
  perm: {},
  gperm: {},
  global: {},
  sglobal: {},
  visibleWizards: {},
  visibleQuickCommands: {},
};
