/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, OnDestroy} from "@angular/core";
import {PrinterAction, ServerAction} from "binding";
import type {RepetierState, ServerState} from "binding";
import {RSModal} from "./modals/rsModal.component";
import type {MessageState, PrintersState, PrinterState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {RSUIHelper} from "./services/ui-helper.service";
import {Router} from "@angular/router";
import {SubscriptionLike as ISubscription} from "rxjs/internal/types";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-headline",
  template: `
    <h1 style="font-weight: 400;display: flex;flex-wrap: nowrap;justify-content: left;">
    <span  *ngIf="wideScreen && printer && printer.online===1" style="font-size:90%;margin-left:10px">
        <span style="padding-right:10px;flex-grow: 0" *ngIf="printer.extruders.length>0 && printer.config.extruders[printer.activeExtruderId].supportTemperature">
              <rs-button class="h1button" (hit)="ui.setExtruderTemp(printer.activeExtruderId)">
                <i [class]="printerAction.extruderIcon(printer.slug,printer.activeExtruderId)" class="fw text-primary" style="display: inline-block;position: relative;top: -7px;"></i>
                  <span style="font-size:15px"><span style="display:inline-block;line-height:17px">{{printer.extruders[printer.activeExtruderId].currentTemp | fixed:1}}°C<br>{{printer.extruders[printer.activeExtruderId].tempSet | tempSet:0}}</span></span>
              </rs-button>
        </span>
        <span class="" *ngIf="printer.heatedBeds.length>0" style="flex-grow: 0">
              <rs-button class="h1button" (hit)="ui.setBedTemp(0)">
                <i class="rs rs-heated-bed fw text-primary" style="display: inline-block;position: relative;top: -7px;"></i>
                  <span style="font-size:15px"><span style="display:inline-block;line-height:17px">{{printer.heatedBeds[0].currentTemp | fixed:1}}°C<br>{{printer.heatedBeds[0].tempSet | tempSet:0}}</span></span>
              </rs-button>
        </span>
        <span class="" *ngIf="printer.heatedChambers.length>0" style="padding-left:8px;flex-grow: 0">
              <rs-button class="h1button" (hit)="ui.setChamberTemp(0)">
                <i [class]="printerAction.chamberIcon(printer.slug,0)" class="fw text-primary" style="display: inline-block;position: relative;top: -7px;"></i>
                  <span style="font-size:15px"><span [ngStyle]="{'line-height': printer.config.heatedChambers[0].supportTemperature ? '17px' : '34px','vertical-align': printer.config.heatedChambers[0].supportTemperature ? 'baseline' : 'bottom'}" style="display:inline-block;">{{printer.heatedChambers[0].currentTemp | fixed:1}} {{ printer.config.heatedChambers[0].unit ?? "°C" }}
                    <span *ngIf="printer.config.heatedChambers[0].supportTemperature"><br>{{printer.heatedChambers[0].tempSet | tempSet:0}}</span></span></span>
              </rs-button>
        </span>
     </span>
     <span *ngIf="(messages$|async).length" style="flex-grow: 0;margin-left: 10px">
      <rs-button class="h1button" (hit)="showMessages()" style="font-size:90%"><i class="fa fa-envelope text-primary"><span class="counter">{{(messages$|async).length}}</span></i></rs-button>
    </span>
      <i *ngIf="statusIcon" [class]="statusIcon" (rsClick)="showStatusInfo()"></i>
    <span (rsClick)="goPrinterList()" style="flex-grow: 1"><ng-content *ngIf="showFunction"></ng-content>
    <span *ngIf="!showFunction" style="flex-grow: 1"><i class="rs rs-printer"></i> {{printers.printers[printers.active].name}}</span>
    </span>
    </h1>`
})
export class RSHeadlineComponent implements OnDestroy {
  messages$: Observable<MessageState[]>;
  server$: Observable<ServerState>;
  printer: PrinterState;
  printers: PrintersState;
  printers$: Observable<PrintersState>;
  wideScreen: Boolean = false;
  showFunction: Boolean = true;
  private printersSubscription: SubscriptionLike;
  private serverSubscription: ISubscription;
  serverState: ServerState = undefined;
  intervalStop: any;
  conditionDialog: any = null;
  lastCondition = -1;
  statusIcon = '';
  @Input() nameSwitchable = true;

  constructor(public ui: RSUIHelper,
              public printerAction: PrinterAction,
              public modal: RSModal,
              public router: Router,
              private serverAction: ServerAction,
              private redux: Store<RepetierState>) {
    this.messages$ = redux.select("messages");
    this.server$ = redux.select("server");
    this.printers$ = redux.select("printers");
    this.serverAction.initSetting("gui", {"autopopupMessages": 1, "emergencyIcon": 0, "noAnimation": 1, "printerNamePos": 1});
    this.wideScreen = window.innerWidth >= 800;
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      this.printers = p;
      if (p.active) {
        this.printer = p.printers[p.active];
        this.updateStatusIcon();
      } else {
        this.printer = undefined;
      }
    });
    this.serverSubscription = this.server$.subscribe((s: ServerState) => {
      this.serverState = s;
      if (this.serverState.settings["gui"] && this.serverState.settings["gui"].printerNamePos !== 1) {
        this.showFunction = true;
      }
    });
    this.intervalStop = setInterval(() => {
      if (this.serverState.settings["gui"] && this.serverState.settings["gui"].printerNamePos === 1 && this.printers.sortedPrinters.length > 1 && this.nameSwitchable && this.printer) {
        this.showFunction = !this.showFunction;
      }
    }, 2000);
  }

  showStatusInfo() {
    if (this.conditionDialog) {
      this.modal.hide(this.conditionDialog);
      this.conditionDialog = null;
    }
    switch (this.printer.condition) {
      case 2: // shutdown
      {
        const question = "<p><?php _('Your printer reported, that it went into killed state. This is normally caused by a serious problem detected inside the printer.') ?></p>\n<p><?php _('To restart the printer must be reset. For most printers with 32 bit processor we can not reset remotely. If the printer has no reset button, you need to power off the printer and eventually even unplug usb cable.') ?></p>\n<p><?php _('Message from printer:') ?></p>\n<p>" +
          this.printer.conditionReason + "</p>";
        this.conditionDialog = this.modal.confirm("<?php _('Printer Problems') ?>", question, {
          no: "<?php _('Close') ?>",
          yes: "<?php _('Try to recover') ?>"
        });
        this.conditionDialog.then(
          () => this.printerAction.recoverBadCondition(this.printer.slug), () => {
          }
        );
      }
        break;
      case 3: // killed
      {
        const question = "<p><?php _('Your printer reported, that it went into killed state. This is normally caused by a serious problem detected inside the printer.') ?></p>\n<p><?php _('To restart the printer must be reset. For most printers with 32 bit processor we can not reset remotely. If the printer has no reset button, you need to power off the printer and eventually even unplug usb cable.') ?></p>\n<p><?php _('Message from printer:') ?></p>\n<p>" +
          this.printer.conditionReason + "</p>";
        this.conditionDialog = this.modal.confirm("<?php _('Printer Problems') ?>", question, {
          no: "<?php _('Close') ?>",
          yes: "<?php _('Try to recover') ?>"
        });
        this.conditionDialog.then(
          () => this.printerAction.recoverBadCondition(this.printer.slug), () => {
          }
        );
      }
        break;
      case 4: // ignoring
      {
        const question = "<p><?php _('Your printer reported an internal printer problem and disabled several commands. This is normally caused by a problem detected inside the printer like temperature runaway.') ?></p>\n<p><?php _('Fix the problem and try to recover. If the source is not fixed, it might go into error state again.') ?></p>\n<p><?php _('Message from printer:') ?></p>\n<p>" +
          this.printer.conditionReason + "</p>";
        this.conditionDialog = this.modal.confirm("<?php _('Printer Problems') ?>", question, {
          no: "<?php _('Close') ?>",
          yes: "<?php _('Try to recover') ?>"
        });
        this.conditionDialog.then(
          () => this.printerAction.recoverBadCondition(this.printer.slug), () => {
          }
        );
      }
        break;
    }
  }

  updateStatusIcon(): string {
    if (!this.printer || !this.printer.active || this.printer.online !== 1) {
      if (this.conditionDialog) {
        this.modal.hide(this.conditionDialog);
        this.conditionDialog = null;
      }
      this.statusIcon = '';
      return;
    }
    if (this.lastCondition !== this.printer.condition) {
      if (this.conditionDialog) {
        this.modal.hide(this.conditionDialog);
        this.conditionDialog = null;
      }
      this.lastCondition = this.printer.condition;
    }
    switch (this.printer.condition) {
      case 0: // unknown
        break;
      case 1: // ready
        break;
      case 2: // shutdown
        this.statusIcon =  "fa fa-warning";
        return;
      case 3: // killed
        this.statusIcon =  "fa fa-exclamation";
        return;
      case 4: // ignoring
        this.statusIcon =  "fa fa-warning";
        return;
      case 5: // offline
        break;
    }
    this.statusIcon = '';
    return "";
  }

  goPrinterList(): void {
    if (this.printers.sortedPrinters.length > 1 && this.nameSwitchable) {
      this.ui.selectPrinter();
      // this.router.navigateByUrl("/printer-list");
    }
  }

  showMessages(): void {
    this.router.navigateByUrl("/messages");
  }

  ngOnDestroy(): void {
    clearInterval(this.intervalStop);
    this.printersSubscription.unsubscribe();
    this.serverSubscription.unsubscribe();
  }
}
