import {
  AfterViewInit,
  Component,
  ComponentRef, ElementRef,
  Input, OnDestroy, OnInit, ViewChild,
  ViewContainerRef
} from "@angular/core";
import {RSModalDialog} from "./rsModalDialog.component";
import {Deferred, PrinterAction, PrintersState, PrinterState, RepetierState, ServerDialog} from "binding";
import {RSIntKeyboard} from "./int-keyboard.service";
import {RSModalViewComponent} from "./modal-view.component";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {RSFloatKeyboard} from "./float-keyboard.service";
import {RSStringKeyboard} from "./string-keyboard.service";
import {RSSelect} from "./select.service";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-modal-view",
  template: `
<div #main [@visibleState]="visState" style="opacity: 0" [ngClass]="'modal modal-dialog '+ modalClass" tabindex="1">
    <div class="box" [@visibleState]="visState" (mousedown)="$event.stopPropagation()"><div><div #box></div></div></div>
</div>
`,
  animations: [
    trigger("visibleState", [
      state("hidden", style({
        opacity: 0
      })),
      state("visible", style({
        opacity: 1
      })),
      transition("*=>hidden", animate("200ms")),
      transition("*=>visible", animate("200ms"))
    ])
  ]
})
export class RSModalViewDialogComponent implements AfterViewInit, OnInit {
  component: ComponentRef<{}> = undefined;
  @ViewChild("box", {read: ViewContainerRef, static: true}) view;
  @ViewChild("main", {static: true}) main: ElementRef;
  deferred: Deferred = undefined;
  data: any;
  name: string;
  modalClass = "";
  componentType: any;
  componentModule: any;
  visState = "hidden";
  starter: RSModalDialog = undefined;


  constructor() {
    RSModalViewComponent.count++;
    this.name = "rs-modal-" + RSModalViewComponent.count;
  }

  cancelIfAllowed = () => {
    if (this.data.closeable) {
      this.cancel();
    }
  }
  cancel() {
    if (this.component) {
      this.visState = "hidden";
      setTimeout(() => {
        if (this.component) {
          this.component.destroy();
        }
        this.component = undefined;
      }, 300);
    } else {
      this.component = undefined;
    }
    if (this.deferred) {
      this.deferred.reject();
    }
  }


  close(result: any) {
    if (this.component) {
      this.visState = "hidden";
      setTimeout(() => {
        if (this.component) {
          this.component.destroy();
        }
        this.component = undefined;
      }, 300);
    } else {
      this.component = undefined;
    }
    if (this.deferred) {
      this.deferred.resolve(result);
    }
  }

  ngOnInit() {
    // this.view.createComponent(this.resolver.resolveComponentFactory(this.componentType));
    this.view.createComponent(this.componentType);
    setTimeout(() => {
      this.visState = "visible";
    }, 10);
  }

  ngAfterViewInit() {
    this.main.nativeElement.focus();
  }
}

enum DialogComponentTypes {
  INPUT = 0,
  DROPDOWN = 1,
  CHECKBOX = 2,
  LABEL = 3
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-server-dialog-modal",
  template: `
    <div class="modal-group-container" *ngIf="dialog">
    <rs-modal-head *ngIf="modal.data.dialog.title" style="display: flex">
      <rs-icon *ngIf="modal.data.dialog.icon" [icon]="modal.data.dialog.icon" style="margin-right: 8px;"></rs-icon>
      <rs-icon *ngIf="!modal.data.dialog.icon" icon="rs rs-printer-commands" style="margin-right: 8px;"></rs-icon>
      <div [innerHTML]="modal.data.dialog.title"></div>
    </rs-modal-head>
    <rs-modal-content>
      <div [innerHTML]="dialog?.message"></div>
      <div *ngIf="dialog.dialogType === 0" style="margin-top:10px;font-size:70%">Note: When printer is blocking
        and communication buffers are full answers might not be executed!
      </div>
      <div style="display: inline-table; justify-content: center; width: 100%;"
           *ngFor="let inputElement of dialog.input ;let indexOfElement=index" [ngStyle]="{'margin-top': inputElement.marginTop , 'margin-bottom': inputElement.marginBottom}">
        <div *ngIf="inputElement.inputType === DialogComponentTypes.INPUT">
          <div
            style="margin-bottom: 0; justify-content: center; align-items:center; display: flex;">
            <div style="margin: auto auto auto 0;">
              <p style="margin: 0 auto 0 0">{{dialog.input[indexOfElement]?.caption}}: </p>
              <strong style="color: red; font-size: small" *ngIf="inputErrors[inputElement.name]?.visible">{{inputErrors[inputElement.name]?.error}}</strong>
            </div>
            <rs-button (hit)="openKeyboard(inputElement)"
                       class="secondary big">{{getPreviewValue(inputElement)}} </rs-button>
          </div>

        </div>
        <div *ngIf="inputElement.inputType === DialogComponentTypes.DROPDOWN">
          <div
            style="margin-bottom: 0; justify-content: center; align-items:center; display: flex;">
            <p style="margin-left: 0; margin-right: auto">{{inputElement.caption}}: </p>
            <rs-button (hit)="selectValues(inputElement)"
                       class="secondary big">{{getPreviewName(inputElement)}}</rs-button>
          </div>
        </div>
        <div *ngIf="inputElement.inputType === DialogComponentTypes.CHECKBOX">
          <div
            style="margin-bottom: 10px; margin-bottom: 0px; justify-content: center; align-items:center; display: flex;">
            <p style="margin-left: 0; margin-right: auto">{{inputElement.caption}}: </p>
            <rs-switch style="margin-right: 0;" [checked]="getPreviewBoolValue(inputElement)"
                       (checkedChange)="inputChanged(inputElement.name, !getPreviewBoolValue(inputElement), inputElement)" class="switch"></rs-switch>
          </div>
        </div>
        <label *ngIf="inputElement.inputType === DialogComponentTypes.LABEL" class="margin-top" [ngStyle]="getLabelStyles(inputElement.labelStyles)">{{inputElement.caption}}</label>
      </div>

    </rs-modal-content>
    <rs-modal-footer *ngIf="dialog.choices.length>0">
      <rs-button *ngFor="let c of modal.data.dialog.choices;let indexOfElement=index;" (hit)="choice(indexOfElement)"
                 class="secondary big">{{c}}</rs-button>
    </rs-modal-footer>
    </div>
  `
})
export class RSServerDialogModalComponent implements OnDestroy {
  // @Input() dialog = {};
  icon = "";
  variant = "";
  inputEntries: Record<string, string> = {};
  inputDropdownSelected: object = {};
  inputErrors:Record<string, { error: string, visible: boolean }> = {} ;
  DialogComponentTypes = DialogComponentTypes;
  printer: PrinterState;
  printers$: Observable<PrintersState>;
  private printersSubscription: SubscriptionLike;
  dialog: ServerDialog

  constructor(public modal: RSModalViewDialogComponent,
              private action: PrinterAction,
              private intKbd: RSIntKeyboard,
              public store: Store<RepetierState>,
              private floatKbd: RSFloatKeyboard,
              private stringKbd: RSStringKeyboard,
              private rsSelect: RSSelect) {
    // this.dialog = modal.data.dialog
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.printer = p.printers[p.active];
        this.dialog = undefined
        const diaList = this.printer.dialogs.filter((x) => x.id === modal.data.dialog.id)
        if(diaList.length) {
          this.dialog = diaList[0]
        }
      } else {
        this.printer = undefined;
      }
    });

    for (const input of Object.keys(modal.data.dialog.input)) {
      const inputElement = modal.data.dialog.input[input];
      this.inputEntries[inputElement.name] = modal.data.dialog.input[input].defaultValue.toString();
      if (inputElement.inputType === DialogComponentTypes.INPUT) {
        this.checkForErrors(inputElement, inputElement.defaultValue.toString());
      }
    }

    switch (this.modal.data.dialog.dialogType) {
      case 1: // info
        break;
      case 2: // warning
        break;
      case 3: // error
        break;
      default:
        break;
    }
  }
  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
  }
  getLabelStyles = (styleNumber: number) => {

    let fontStyle = "normal";
    let fontWeight = "normal";
    let fontsize = "inherit";

    // eslint-disable-next-line no-bitwise
    if (styleNumber & 32) {
      fontWeight = "bold";
      fontsize = "xx-large";
    }
    // eslint-disable-next-line no-bitwise
    if (styleNumber & 16) {
      fontWeight = "bold";
      fontsize = "x-large";
    }
    // eslint-disable-next-line no-bitwise
    if (styleNumber & 8) {
      fontStyle = "italic";
    }
    // eslint-disable-next-line no-bitwise
    if (styleNumber & 4) {
      fontWeight = "bold";
    }
    // eslint-disable-next-line no-bitwise
    if (styleNumber & 2) {
      fontsize = "inherit";
    }
    // eslint-disable-next-line no-bitwise
    if (styleNumber & 1) {
      fontsize = "small";
    }
    return {"font-style": fontStyle, "font-weight": fontWeight, "font-size": fontsize}
  }

  selectValues(inputElement: any) {
    if (!inputElement.readonly) {
      const eList = [];
      for (let i = 0; i < inputElement.dropdownData.length; i++) {
        eList.push({
          id: i,
          name: inputElement.dropdownData[i].name,
          value: inputElement.dropdownData[i].value
        });
      }
      this.rsSelect.select(eList, "name", "<i class=\"rs\"></i> " + inputElement.caption).then(
        (ex) => {
          this.inputChanged(inputElement.name, ex.value);
          this.inputDropdownSelected[inputElement.caption] = ex.name;
        }, () => {
        }
      );
    }
  }
  openKeyboard(inputElement: any) {
    if (!inputElement.readonly) {
      if (inputElement.dataType === "Int") {
        this.intKbd.getInt(Number(inputElement.defaultValue), inputElement.caption, {
          min: Number(inputElement.minValue),
          max: Number(inputElement.maxValue)
        }).then((value) => {
          this.inputChanged(inputElement.name, value, inputElement);
        }, () => {
        });
      }
      if (inputElement.dataType === "Double") {
        this.floatKbd.getFloat(Number(inputElement.defaultValue), inputElement.caption, {
          min: Number(inputElement.minValue),
          max: Number(inputElement.maxValue)
        }).then((value) => {
          this.inputChanged(inputElement.name, value, inputElement);
        }, () => {
        });
      }
      if (inputElement.dataType === "String") {
        this.stringKbd.getString(inputElement.defaultValue, inputElement.caption, {
          minLength: Number(inputElement.minValue),
          maxLength: Number(inputElement.maxValue)
        }).then((value) => {
          this.inputChanged(inputElement.name, value, inputElement);
        }, () => {
        });
      }
    }
  }
  choice(idx: number) {
    let errorFound = false;
    for (const inputName of Object.keys(this.inputErrors)) {
      this.inputErrors[inputName].visible = this.inputErrors[inputName].error !== "";
      errorFound = errorFound || this.inputErrors[inputName].error !== "";
    }
    this.action.selectDialogChoice(idx, this.modal.data.dialog, this.inputEntries).then((answer) => {
      if (typeof(answer) === "object") {
        if (answer.ok) {
          this.modal.close(true);
        }
      }
    });
  }
  inputChanged(name: string, val: any, inputElement?: any) {

    let readonly = false;
    if (val !== undefined) {
      if (inputElement !== undefined) {
        inputElement = {value: val, ...inputElement};
        readonly = inputElement.readonly;
      }
      if (!readonly) {
        this.checkForErrors(inputElement, val);
        if (typeof (val) === "boolean") {
          const val2: string = val ? "1" : "0";
          this.inputEntries[name.toString()] = val2;
        } else {
          this.inputEntries[name.toString()] = val.toString();
        }
      }
    }

  }

  checkForErrors(dialogInput: any, value: string): boolean {
    let error = "";
    if (dialogInput !== undefined) {
      if (dialogInput.inputType === DialogComponentTypes.INPUT) { // Just validate input fields
        if (dialogInput.dataType === "String") {
          if (dialogInput.minValue > value.length) {
            error = "Minimum text length is: " + dialogInput.minValue;
          } else if (dialogInput.maxValue < value.length) {
            error = "Maximum text length is: " + dialogInput.maxValue;
          }
        } else {
          if (isNaN(Number(value))) {
            error = "Entered text is not a number";
          } else if ((dialogInput.dataType !== "String") && (dialogInput.minValue > value)) {
            error = "Minimum value is: " + dialogInput.minValue;
          } else if (dialogInput.maxValue < value) {
            error = "Maximum value is: " + dialogInput.maxValue;
          }
        }
      }
      this.inputErrors[dialogInput.name] = {error: error, visible: false};
    }
    return (error === "");
  }

  getPreviewBoolValue(c: any) {
    let cValue: string = (c.value !== undefined) ? c.value : c.defaultValue;
    cValue = (this.inputEntries[c.name] !== undefined) ? this.inputEntries[c.name] : cValue;
    return  cValue === '1';
  }

  getPreviewValue(c: any) {
    let cValue: string = (c.value !== undefined) ? c.value : c.defaultValue;
    cValue = (this.inputEntries[c.name] !== undefined) ? this.inputEntries[c.name] : cValue;
    return  (cValue !== "") ? cValue : "empty";
  }
  getPreviewName(c: any) {
    if (this.inputDropdownSelected[c.caption] === undefined) {
      this.inputDropdownSelected[c.caption] = "N/A";
      c.dropdownData.forEach((x) => {
        // double equal is correct here - want to compare string against number as well
        // eslint-disable-next-line eqeqeq
        if (c.defaultValue == x.value) {
          this.inputDropdownSelected[c.caption] = x.name.toString();
        }
      });
    }
    return this.inputDropdownSelected[c.caption];
  }
}
