/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, EventEmitter, Output, HostBinding, OnDestroy} from "@angular/core";
import {Router} from "@angular/router";
import {PrinterAction, ServerAction} from "binding";
import type {PrintersState, PrinterState, RepetierState, ServerState} from "binding";
import {RSModal} from "./modals/rsModal.component";
import {Observable, SubscriptionLike} from "rxjs";
import {SubscriptionLike as ISubscription} from "rxjs/internal/types";
import {RSUIHelper} from "./services/ui-helper.service";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-page",
  template: `
    <div class="sidemenu">
      <div style="text-align: center">
      <div  (rsClick)="goPrinterList()" *ngIf="showPrinter" class="sidebar-printername">{{printer.name}}</div>
      <span style="font-size:90%">{{time | date: 'H:mm'}}</span></div>
      <ng-content select="rs-side"></ng-content>
      <div class="emergency fa-stack" *rsIfConnected="true" (rsClick)="emergencyStop()">
        <div *ngIf="(server$|async).settings.gui.emergencyIcon===0">
          <i class="rs rs-emergency-1 fa-stack-1x"></i>
          <i class="rs rs-emergency-2 fa-stack-1x"></i>
          <i class="rs rs-emergency-3 fa-stack-1x"></i>
          <i class="rs rs-emergency-4 fa-stack-1x"></i>
        </div>
        <div *ngIf="(server$|async).settings?.gui.emergencyIcon===1" style="background: #cf0000;height:100%;vertical-align: middle;text-align: center">
          <i class="fa fa-hand-paper-o fa-stack-1x" style="color:#ffffff;line-height:1em;font-size:80%;padding-top:10%"></i>
        </div>
      </div>
    </div>
    <div class="main"><ng-content select="main"></ng-content></div>`
})
export class RSPageComponent implements OnDestroy {
  @HostBinding("class") hostClass = "fullpage";
  time = new Date();
  timer: ReturnType<typeof setInterval> = undefined;
  highScreen: Boolean = false;
  server$: Observable<ServerState>;
  printers$: Observable<PrintersState>;
  printers: PrintersState;
  printer: PrinterState;
  serverState: ServerState = undefined;
  private printersSubscription: SubscriptionLike;
  private serverSubscription: ISubscription;
  @Input() nameSwitchable = true;
  showPrinter: Boolean = false;

  constructor(public ui: RSUIHelper,
              public printerAction: PrinterAction,
              private serverAction: ServerAction,
              public modal: RSModal,
              private store: Store<RepetierState>) {
    this.server$ = store.select('server');
    this.printers$ = store.select('printers');
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      this.printers = p;
      if (p.active) {
        this.printer = p.printers[p.active];
      } else {
        this.printer = undefined;
      }
    });
    this.serverSubscription = this.server$.subscribe((s: ServerState) => {
      this.serverState = s;
      this.showPrinter = (this.highScreen && this.serverState.settings["gui"] && this.serverState.settings["gui"].printerNamePos === 2 && this.printers && this.printers.sortedPrinters && this.printers.sortedPrinters.length > 1 && this.nameSwitchable && this.printer ? true : false);
    });

    this.timer = setInterval(() => {
      this.time = new Date();
      this.highScreen = window.innerHeight > 320;
      this.showPrinter = (this.highScreen && this.serverState.settings["gui"] && this.serverState.settings["gui"].printerNamePos === 2 && this.printers && this.printers.sortedPrinters && this.printers.sortedPrinters.length > 1 && this.nameSwitchable && this.printer ? true : false);
    }, 1000);
  }

  goPrinterList(): void {
    if (this.printers.sortedPrinters.length > 1) {
      this.ui.selectPrinter();
      // this.router.navigateByUrl("/printer-list");
    }
  }

  ngOnDestroy() {
    clearInterval(this.timer);
    this.printersSubscription.unsubscribe();
    this.serverSubscription.unsubscribe();
  }

  emergencyStop() {
    this.modal.confirm("<?php _('Confirmation required') ?>", "<?php _('Execute Emergency Stop?') ?>", {}).then(() => {
      this.printerAction.emergencyStop(this.printers.active);
    }, () => {
    });
  }
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-side",
  template: "<ng-content></ng-content>"
})
export class RSSideComponent {
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-sideicon",
  template: `<a *ngIf="link" (rsClick)="doLink()" href="javascript:void(0)"><i class="{{icon}}"></i><h3>{{title}}</h3></a><a *ngIf="!link" href="javascript:void(0)" (rsClick)="doHit()"><i class="{{icon}}"></i><h3>{{title}}</h3></a>`
  /* host: {
      'class': 'icon'
  } */
})
export class RSSideIconComponent {
  @Input() title = "";
  @Input() icon = "";
  @Input() link = "";
  @Output() hit: EventEmitter<any> = new EventEmitter();
  @HostBinding("class") hostClass = "icon";

  constructor(public router: Router) {
  }

  doLink() {
    this.router.navigateByUrl(this.link);
  }

  doHit() {
    this.hit.emit(true);
  }
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-empty-sideicon",
  template: `<a (rsClick)="doHit()" href="javascript:void(0)"><i class="icon">&nbsp;</i><h3>&nbsp;</h3></a>`
  /* host: {
      'class': 'icon'
  } */
})
export class RSEmptySideIconComponent {
  @Output() hit: EventEmitter<any> = new EventEmitter();
  @HostBinding("class") hostClass = "icon";

  constructor(public router: Router) {
  }

  doHit() {
    this.hit.emit(true);
  }
}
