/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
var core_1 = require('@angular/core');
var RSBinding_1 = require("../../RSBinding");
var int_keyboard_service_1 = require("../modals/int-keyboard.service");
var select_service_1 = require("../modals/select.service");
var modal_service_1 = require("../modals/modal.service");
var RSUIHelper = (function () {
    function RSUIHelper(server, intKbd, select, modal) {
        this.server = server;
        this.intKbd = intKbd;
        this.select = select;
        this.modal = modal;
    }
    RSUIHelper.prototype.setExtruderTemp = function (eid) {
        var _this = this;
        this.intKbd.getInt(this.server.active.extruders[eid].tempSet, '<i class="rs rs-extruder"></i> ' + this.server.active.extruderName(eid), {
            min: 0,
            max: this.server.active.config.extruders[eid].maxTemp
        }).then(function (v) {
            _this.server.active.setExtruderTemperature(eid, v);
        }, function () {
        });
    };
    RSUIHelper.prototype.setBedTemp = function () {
        var _this = this;
        this.intKbd.getInt(this.server.active.heatedBed.tempSet, '<i class="rs rs-heated-bed"></i> ' + "<?php _('Bed') ?>", {
            min: 0,
            max: this.server.active.config.heatedBed.maxTemp
        }).then(function (v) {
            _this.server.active.setBedTemperature(v);
        }, function () {
        });
    };
    RSUIHelper.prototype.setFan = function () {
        var _this = this;
        this.intKbd.getInt(this.server.active.fanPercent, '<i class="rs rs-fan"></i> ' + "<?php _('Fan') ?>", {
            min: 0,
            max: 100
        }).then(function (v) {
            _this.server.active.setFanPercent(v);
        }, function () {
        });
    };
    RSUIHelper.prototype.setSpeed = function () {
        var _this = this;
        this.intKbd.getInt(this.server.active.speedMultiply, '<i class="rs rs-speed"></i> ' + "<?php _('Speed') ?>", {
            min: 25,
            max: 400
        }).then(function (v) {
            _this.server.active.speedMultiply = v;
        }, function () {
        });
    };
    RSUIHelper.prototype.setFlow = function () {
        var _this = this;
        this.intKbd.getInt(this.server.active.flowMultiply, '<i class="rs rs-flow"></i> ' + "<?php _('Flow') ?>", {
            min: 0,
            max: 300
        }).then(function (v) {
            _this.server.active.flowMultiply = v;
        }, function () {
        });
    };
    RSUIHelper.prototype.selectExtruder = function () {
        var _this = this;
        var eList = [];
        for (var i = 0; i < this.server.active.extruders.length; i++) {
            eList.push({ id: i, name: this.server.active.extruderName(i) });
        }
        this.select.select(eList, "name", '<i class="rs rs-extruder"></i> ' + "<?php _('Select Extruder') ?>").then(function (ex) {
            _this.server.active.selectExtruder(ex.id);
        }, function () { });
    };
    RSUIHelper.prototype.stopPrint = function () {
        var _this = this;
        this.modal.confirm("<?php _('Confirmation required') ?>", "<?php _('Really stop current print?') ?>").then(function () {
            _this.server.active.stopPrint();
        }, function () { });
    };
    RSUIHelper = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [RSBinding_1.RepetierServer, int_keyboard_service_1.RSIntKeyboard, select_service_1.RSSelect, modal_service_1.RSModal])
    ], RSUIHelper);
    return RSUIHelper;
})();
exports.RSUIHelper = RSUIHelper;
//# sourceMappingURL=ui-helper.service.js.map