/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from "@angular/core";
import {Router} from "@angular/router";
import {RSUIHelper, RSIntKeyboard} from "front-ui";
import {PrinterAction, PrintersAction, PrinterState, PrintersState, RSUtils, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-change-filament-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" (rsClick)="cancel()"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfPrinting="true" title="<?php _('Print') ?>" icon="rs rs-print" link="/print-status"></rs-sideicon>
        <rs-sideicon *rsIfPrinting="false" icon="rs rs-move-xy" title="<?php _('Move') ?>" link="/move-tool"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfConnected="true" icon="rs rs-params" title="<?php _('Status') ?>" link="/params"></rs-sideicon>
        <rs-empty-sideicon *rsIfConnected="false"></rs-empty-sideicon>
      </rs-side>
      <main>
        <rs-headline><i [class]="printerAction.extruderChangeIcon(printer.slug,printer.activeExtruderId)"></i> {{ title }}</rs-headline>
        <div *rsIfConnected="true">
          <div class="content-h1" *ngIf="page===0 && printer?.extruders && targetExtruder>-1" rsDrag>
            <div>
              <div class="margin-leftright" #positions>
                <rs-button class="secondary full" (hit)="ui.selectExtruder()">
                  <i [class]="printerAction.extruderIcon(printer.slug,printer.activeExtruderId)"></i>
                  <h3>{{ printerAction.extruderName(printer.slug, printer.activeExtruderId) }}
                    <span *ngIf="printerAction.toolSupportsTemperatures(printer.slug,printer.activeExtruderId)">&nbsp;{{ printer.extruders[printer.activeExtruderId]|tempFull:1 }}</span>
                  </h3>
                </rs-button>
              </div>
              <div class="iconlist">
                <div class="button-icon sep-icon full" *ngIf="printerAction.toolSupportsTemperatures(printer.slug,printer.activeExtruderId)">
                  <rs-button class="secondary outline full" (hit)="setTargetTemp()">
                    <i class="rs rs-extruder fw"></i>
                    <h3><?php _('Change Temp.') ?><br class="hide-max-320">
                      {{ targetTemperature|fixed:1 }} °C</h3>
                  </rs-button>
                </div>
                <div class="button-icon sep-icon full">
                  <rs-button class="secondary outline full" (hit)="setFastDistance()">
                    <i class="rs rs-distance fw"></i>
                    <h3><?php _('Fast Distance') ?><br class="hide-max-320">
                      {{ fastDistance|fixed:1 }} mm</h3>
                  </rs-button>
                </div>
                <div class="button-icon sep-icon full">
                  <rs-button class="secondary outline full" (hit)="setSlowDistance()">
                    <i class="rs rs-distance fw"></i>
                    <h3><?php _('Slow Distance') ?><br class="hide-max-320">
                      {{ slowDistance|fixed:1 }} mm</h3></rs-button>
                </div>
                <div class="button-icon autosize">
                  <rs-button class="secondary full" [disabled]="!printer?.idle" (hit)="removeFilament()">
                    <i [class]="printerAction.extruderChangeUpIcon(printer.slug,printer.activeExtruderId)" style="font-size:1.6rem"></i>
                    <span class="hide-max-480">{{ printerAction.extruderChangeRemove(printer.slug, printer.activeExtruderId) }}</span>
                  </rs-button>
                </div>
                <div class="button-icon autosize">
                  <rs-button class="secondary full" [disabled]="!printer?.idle" (hit)="insertFilament()">
                    <i [class]="printerAction.extruderChangeDownIcon(printer.slug,printer.activeExtruderId)" style="font-size:1.6rem"></i>
                    <span class="hide-max-480">{{ printerAction.extruderChangeInsert(printer.slug, printer.activeExtruderId) }}</span>
                  </rs-button>
                </div>
              </div>
            </div>
          </div>
          <div class="content-h1" *ngIf="page===1" rsDrag>
            <div>
              <div class="alert info margin  margin-bottom-minh-320" [innerHtml]="status"></div>

              <div *ngIf="showStart" class="iconlist margin-bottom-minh-320">
                <div class="button-icon sep-icon">
                  <rs-button class="secondary full outline" (hit)="startPressed()" style="font-size:1.5rem">
                    <h3><?php _('Start') ?></h3></rs-button>
                </div>
              </div>
              <div *ngIf="!showStart && !waitForTargetTemp && !printerAction.isSyringe(printer.slug,printer.activeExtruderId)" class="iconlist margin-bottom-minh-320">
                <div class="button-icon sep-icon autosize">
                  <rs-button class="secondary full outline" [disabled]="!printer?.idle" (hit)="removeMore(10)">
                    <i class="rs rs-arrow-h-up"></i>
                    <h3><span class="hide-max-320"><?php _('Retract') ?>&nbsp;<br></span>10 mm</h3></rs-button>
                </div>
                <div class="button-icon sep-icon autosize">
                  <rs-button class="secondary full outline" [disabled]="!printer?.idle" (hit)="insertMore(10)">
                    <i class="rs rs-arrow-h-down"></i>
                    <h3><span class="hide-max-320"><?php _('Extrude') ?>&nbsp;<br></span>10 mm</h3></rs-button>
                </div>
              </div>
              <div *ngIf="!showStart && !waitForTargetTemp && printerAction.isSyringe(printer.slug,printer.activeExtruderId)" class="iconlist">
                <div class="button-icon sep-icon autosize">
                  <rs-button class="secondary full outline" [disabled]="!printer?.idle" (hit)="removeMore(1)">
                    <i class="rs rs-arrow-h-up"></i>
                    <h3><span class="hide-max-320"><?php _('Retract') ?>&nbsp;<br></span>1 mm</h3></rs-button>
                </div>
                <div class="button-icon sep-icon autosize">
                  <rs-button class="secondary full outline" [disabled]="!printer?.idle" (hit)="insertMore(1)">
                    <i class="rs rs-arrow-h-down"></i>
                    <h3><span class="hide-max-320"><?php _('Extrude') ?>&nbsp;<br></span>1 mm</h3></rs-button>
                </div>
              </div>
              <div *ngIf="!showStart && !waitForTargetTemp && printerAction.isSyringe(printer.slug,printer.activeExtruderId)" class="iconlist margin-bottom-minh-320">
                <div class="button-icon sep-icon autosize">
                  <rs-button class="secondary full outline" [disabled]="!printer?.idle" (hit)="removeMore(0.1)">
                    <i class="rs rs-arrow-h-up"></i>
                    <h3><span class="hide-max-320"><?php _('Retract') ?>&nbsp;<br></span>0.1 mm</h3></rs-button>
                </div>
                <div class="button-icon sep-icon autosize">
                  <rs-button class="secondary full outline" [disabled]="!printer?.idle" (hit)="insertMore(0.1)">
                    <i class="rs rs-arrow-h-down"></i>
                    <h3><span class="hide-max-320"><?php _('Extrude') ?>&nbsp;<br></span>0.1 mm</h3></rs-button>
                </div>
              </div>

              <div class="iconlist margin-bottom-minh-320" style="padding-top: 0">
                <div class="button-icon" *ngIf="remove && !waitForTargetTemp">
                  <rs-button class="secondary full" [disabled]="!printer?.idle" (hit)="insertFilament()">
                    <i [class]="printerAction.extruderChangeDownIcon(printer.slug,printer.activeExtruderId)" style="font-size:1.6rem"></i>
                    <h3><?php _('Insert') ?></h3></rs-button>
                </div>
                <div class="button-icon" *ngIf="!remove && !waitForTargetTemp">
                  <rs-button class="secondary full" [disabled]="!printer?.idle" (hit)="removeFilament()">
                    <i [class]="printerAction.extruderChangeUpIcon(printer.slug,printer.activeExtruderId)" style="font-size:1.6rem"></i>
                    <h3><?php _('Remove') ?></h3></rs-button>
                </div>
                <div class="button-icon">
                  <rs-button class="secondary full" (hit)="cancel()">
                    <i class="fa fa-close" style="font-size:1.6rem"></i>
                    <h3><?php _('Finished') ?></h3></rs-button>
                </div>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>`
})
export class ChangeFilamentComponent implements OnDestroy {
  page = 0;
  remove = false;
  startTemperature = 0;
  targetTemperature = 0;
  targetExtruder = -1;
  fastDistance = 0;
  slowDistance = 0;
  wasInserted = false;
  showStart = false;
  waitForTargetTemp = false;
  status = "";
  title = "";
  printer: PrinterState = undefined;
  printers$: Observable<PrintersState>;

  private printersSubscription: SubscriptionLike;

  constructor(public printerAction: PrinterAction,
              private printersAction: PrintersAction,
              private intKbd: RSIntKeyboard,
              public router: Router,
              public ui: RSUIHelper,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.printer = p.printers[p.active];
        if (!this.printer || this.printer.config.extruders === undefined || this.printer.extruders.length === 0) {
          return;
        }
        if (!this.title) {
          this.title = printerAction.extruderChangeName(this.printer.slug, this.printer.activeExtruderId);
        }
        // console.log("select printer",this.printer);
        if (this.targetExtruder !== this.printer.activeExtruderId) {
          this.targetExtruder = this.printer.activeExtruderId;
          this.fastDistance = this.printer.config.extruders[this.targetExtruder].changeFastDistance;
          this.slowDistance = this.printer.config.extruders[this.targetExtruder].changeSlowDistance;
          this.targetTemperature = this.printer.config.extruders[this.targetExtruder].lastTemp;
        }
        if (this.waitForTargetTemp) {
          if (this.printer.extruders[this.targetExtruder].currentTemp >= this.targetTemperature - 0.7 || this.printer.extruders[this.targetExtruder].currentTemp < -274) {
            this.targetTempReached();
          } else {
            this.status = `<?php _('Heating') ?>...${this.printer.extruders[this.targetExtruder].currentTemp.toFixed(1)}/${this.targetTemperature}°C`;
          }
        }
      }
    });
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
    if (this.page === 1) {
      if (this.wasInserted) {
        this.printerAction.runScript(this.printer.slug, "insert_filament_finished");
        this.wasInserted = false;
      }
      this.printerAction.setExtruderTemperature(this.printer.slug, this.targetExtruder, this.startTemperature);
    }
  }

  setTargetTemp(): void {
    this.intKbd.getInt(this.targetTemperature, "<i class=\"" + this.printerAction.extruderIcon(this.printer.slug, this.targetExtruder) + "\"></i> " + "<?php _('Change Temp.') ?>", {
      min: 0,
      max: this.printer.config.extruders[this.targetExtruder].maxTemp,
      unit: "°C",
      presets: this.ui.removeDuplicates([0].concat(this.ui.tempList(this.printer.config.extruders[this.targetExtruder].temperatures)).concat([190, 200, 250]))
        .splice(0, 4).sort(function (a, b) {
          return a > b ? 1 : 0;
        })
    }).then(
      (v) => {
        this.targetTemperature = v;
      }, () => {
      }
    );
  }

  setFastDistance(): void {
    this.intKbd.getInt(this.fastDistance, "<i class=\"rs rs-distance\"></i> " + "<?php _('Fast Distance') ?>", {
      min: 0,
      max: 10000,
      unit: "mm"
    }).then(
      (v) => {
        this.fastDistance = v;
        const newConfig = RSUtils.copyObject(this.printer.config);
        newConfig.extruders[this.targetExtruder].changeFastDistance = this.fastDistance;
        this.printerAction.storeNewConfig(this.printer.slug, newConfig);
      }, () => {
      }
    );
  }

  setSlowDistance(): void {
    this.intKbd.getInt(this.slowDistance, "<i class=\"rs rs-distance\"></i> " + "<?php _('Slow Distance') ?>", {
      min: 0,
      max: 10000,
      unit: "mm"
    }).then(
      (v) => {
        this.slowDistance = v;
        const newConfig = RSUtils.copyObject(this.printer.config);
        newConfig.extruders[this.targetExtruder].changeSlowDistance = this.slowDistance;
        this.printerAction.storeNewConfig(this.printer.slug, newConfig);
      }, () => {
      }
    );
  }

  targetTempReached(): void {
    this.waitForTargetTemp = false;
    let time = 0;
    if (this.remove) {
      this.printerAction.runScript(this.printer.slug, "remove_filament");
      this.wasInserted = false;
      time = (this.slowDistance + this.fastDistance) / this.printer.config.extruders[this.targetExtruder].retractSpeed;
      this.status = "<?php _('Retracting ...') ?>";
      this.printerAction.eMoveRel(this.printer.slug, -(this.fastDistance + this.slowDistance), this.printer.config.extruders[this.targetExtruder].retractSpeed);
      setTimeout(() => {
        this.status = "<?php _('Use extrude or retract if necessary.') ?>";
      }, time * 1000);
    } else {
      this.status = "<?php _('Put filament in extruder and press Start to insert.') ?>";
      this.showStart = true;
    }
  }

  startPressed(): void {
    this.showStart = false;
    this.printerAction.runScript(this.printer.slug, "insert_filament");
    this.wasInserted = true;
    const time = this.slowDistance / this.printer.config.extruders[this.targetExtruder].extrudeSpeed + this.fastDistance / this.printer.config.extruders[this.targetExtruder].retractSpeed;
    this.status = "<?php _('Inserting ...') ?>";
    this.printerAction.eMoveRel(this.printer.slug, this.fastDistance, this.printer.config.extruders[this.targetExtruder].retractSpeed);
    this.printerAction.eMoveRel(this.printer.slug, this.slowDistance, this.printer.config.extruders[this.targetExtruder].extrudeSpeed);
    setTimeout(() => {
      this.status = "<?php _('Use extrude or retract if necessary.') ?>";
    }, time * 1000);
  }

  storeChangesSettings(): void {
    if (this.fastDistance !== this.printer.config.extruders[this.targetExtruder].changeFastDistance ||
      this.slowDistance !== this.printer.config.extruders[this.targetExtruder].changeSlowDistance) {
      const newConfig = RSUtils.copyObject(this.printer.config);
      newConfig.extruders[this.targetExtruder].changeFastDistance = this.fastDistance;
      newConfig.extruders[this.targetExtruder].changeSlowDistance = this.slowDistance;
      this.printerAction.storeNewConfig(this.printer.slug, newConfig);
    }
  }

  preheatAndWait(): void {
    if (this.printerAction.toolSupportsTemperatures(this.printer.slug, this.printer.activeExtruderId)) {
      this.startTemperature = this.printer.extruders[this.targetExtruder].tempSet;
      this.printerAction.setExtruderTemperature(this.printer.slug, this.targetExtruder, this.targetTemperature);
      this.waitForTargetTemp = true;
      this.printerAction.setExtruderTemperature(this.printer.slug, this.targetExtruder, this.targetTemperature);
      this.status = `<?php _('Heating') ?>...${this.printer.extruders[this.targetExtruder].currentTemp.toFixed(1)}/${this.targetTemperature}°C`;
    } else {
      this.targetTempReached();
    }
  }

  removeFilament(): void {
    this.remove = true;
    this.title = this.printerAction.extruderChangeRemove(this.printer.slug, this.printer.activeExtruderId); // "<?php _('Remove Filament') ?>";
    this.page = 1;
    this.showStart = false;
    this.storeChangesSettings();
    this.preheatAndWait();
  }

  insertFilament(): void {
    this.remove = false;
    this.title = this.printerAction.extruderChangeInsert(this.printer.slug, this.printer.activeExtruderId); // "<?php _('Insert Filament') ?>";
    this.page = 1;
    this.showStart = false;
    this.storeChangesSettings();
    this.preheatAndWait();
  }

  cancel(): void {
    if (this.page === 1) {
      if (this.wasInserted) {
        this.printerAction.runScript(this.printer.slug, "insert_filament_finished");
        this.wasInserted = false;
      }
      this.printerAction.setExtruderTemperature(this.printer.slug, this.targetExtruder, this.startTemperature);
    }
    this.router.navigateByUrl("/");
  }

  insertMore(dist): void {
    this.printerAction.eMoveRel(this.printer.slug, dist, this.printer.extrudeSpeed);
  }

  removeMore(dist): void {
    this.printerAction.eMoveRel(this.printer.slug, -dist, this.printer.retractSpeed);
  }
}
