/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {ChangeDetectorRef, Component, OnDestroy, AfterViewInit} from "@angular/core";
import {DomSanitizer, SafeResourceUrl} from "@angular/platform-browser";
import {
  ServerState,
  ExternalLink, RepetierState
} from "binding";
import {ActivatedRoute} from "@angular/router";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-external-link-component",
  template: `
<rs-page>
<rs-side>
    <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
</rs-side>
<rs-side><rs-empty-sideicon></rs-empty-sideicon></rs-side>
<rs-side><rs-empty-sideicon></rs-empty-sideicon></rs-side>
<main>
  <div>
    <rs-headline><i class="fa fa-external-link"></i> {{link?.title}}</rs-headline>
    <div class="content-h1">
      <div id="externalLink">
        <iframe  *ngIf="link" [src]="secureUrl" id="externalLinkFrame" style="border:0;width:100%;"></iframe>
      </div>
    </div>
  </div>
</main>
</rs-page>
    `
})
export class ExternalLinkComponent implements OnDestroy, AfterViewInit {
  server: ServerState = undefined;
  link: ExternalLink;
  id: number;
  lastUrl = '';
  closed = false;
  secureUrl: SafeResourceUrl;
  iframeHeight = "50px";

  server$: Observable<ServerState>;

  private serverSubscription: SubscriptionLike;
  private paramsSubscription: SubscriptionLike;

  constructor(private route: ActivatedRoute,
              public ref: ChangeDetectorRef,
              public sanitizer: DomSanitizer,
              private store: Store<RepetierState>) {
    this.server$ = store.select('server');
    this.paramsSubscription = this.route.params.subscribe( params => this.id = parseInt(params.id, 10) );
    this.serverSubscription = this.server$.subscribe((s: ServerState) => {
      this.server = s;
      this.link = this.server.externalLinks[this.id];
      if (this.link && this.lastUrl !== this.link.href) {
        this.lastUrl = this.link.href;
        const ip = window.location.host.match(/(.*?)(:\d+)?$/)[1];
        const insecureLink = this.link.href.replace("0.0.0.0", ip);
        this.secureUrl = sanitizer.bypassSecurityTrustResourceUrl(insecureLink);
      }
    });
    this.updateHeight();
    // setTimeout(this.updateHeight, 1000);
  }

  updateHeight = () => {
    const elem: any = document.getElementById('externalLinkFrame');
    if (elem) {
      const rect = elem.getBoundingClientRect();
      // console.log(rect.top, rect.right, rect.bottom, rect.left);
      const iframeHeight = (window.innerHeight - rect.top);
      elem.height = iframeHeight;
      // this.ref.markForCheck();
    } else if (!this.closed) {
      setTimeout(this.updateHeight, 50);
    }
  }

  ngAfterViewInit() {
    this.updateHeight();
  }

  ngOnDestroy(): void {
    this.closed = true;
    this.serverSubscription.unsubscribe();
    this.paramsSubscription.unsubscribe();
  }

}
