/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Output, Input, EventEmitter, ChangeDetectorRef, OnDestroy} from "@angular/core";
import {Router} from "@angular/router";
import {RSModal, RSSelect} from "front-ui";
import {
  RepetierServer,
  RepetierState,
  PrinterState,
  GroupState,
  GCodeFileState,
  PrinterAction,
  PrintersState,
  RSUtils,
  VisualAids, PrinterSortByAction, PrinterSortModelsAction
} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {ConnectionHandler, ServerEvent, getState} from "binding";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-gcode-info",
  template: `
    <h1>{{ file.name }}</h1>
    <div class="content-h1 " rsDrag>
      <div>
        <div class="margin-leftright15">
          <div class="margin-bottom" *ngIf="file.id !== printer.jobid">
            <rs-button *ngIf="!printer.printing && printer.online===1" style="margin-right:10px" [disabled]="printer.printing" [ngClass]="{primary: file.fits, error: !file.fits, 'small-icon':true}" (hit)="print()">
              <i class="rs rs-print"></i>
              <h3><?php _('Print') ?></h3>
            </rs-button>
            <rs-button *ngIf="file.type === 2 && printer.config.general.enableQueue" [ngClass]="{primary: file.fits, error: !file.fits, 'small-icon':true}" (hit)="queue()">
              <i class="rs rs-queue"></i>
              <h3><?php _('Queue File') ?></h3>
            </rs-button>
            <rs-button class="error outline small-icon" style="float:right" (hit)="deleteFile()">
              <i class="fa fa-trash-o"></i>
              <!-- <h3><?php _('Delete') ?></h3> -->
            </rs-button>
            <div style="clear: both"></div>
          </div>
          <div *ngIf="file.id === printer.jobid && printer?.jobstate === 'running'" style="display: flex; align-items: center;">
            <i class="fa fa-spinner fa-spin" style="height: 100%"></i>
            <h3 class="margin-leftright"><?php_('Currently printing!')?></h3>
            <div style="clear: both"></div>
          </div>
          <div *ngIf="file.id === printer.jobid && printer?.jobstate === 'killing'" style="display: flex; align-items: center;">
            <i class="fa fa-spinner fa-spin" style="height: 100%"></i>
            <h3 class="margin-leftright"><?php_('Aborting print ... Please be patient')?></h3>
            <div style="clear: both"></div>
          </div>

          <!-- <img [src]="printerAction.gcodeImageUrl(file.slug,file.list, file.id,'l',file.version)"> -->
          <img [src]="printer.imageUrls[file.slug + '_' + file.list + file.id + 'l']">
          <h3><?php _('Analysis') ?></h3>
          <div class="value-list">
            <div>{{ tPrintingTime }}</div>
            <div class="text-primary-bg">{{ file.printTime | dhms }}</div>
          </div>
          <div class="value-list" *ngIf="file.lastPrintTime">
            <div><?php _('Last Real Printing Time') ?></div>
            <div class="text-primary-bg">{{ file.lastPrintTime | dhms }}</div>
          </div>
          <div class="value-list">
            <div><?php _('Total Price') ?></div>
            <div class="text-primary-bg">{{ file.total | fixed:2 }} {{ file.currency }}</div>
          </div>
          <div class="value-list">
            <div><?php _('Filament') ?></div>
            <div class="text-primary-bg">{{ file.filamentTotal | fixed:1 }} mm</div>
          </div>
          <div class="value-list">
            <div><?php _('Volume') ?></div>
            <div class="text-primary-bg">{{ (file.volumeTotal / 1000) | fixed:1 }} cm<sup>3</sup></div>
          </div>
          <div class="value-list">
            <div><?php _('Weight') ?></div>
            <div class="text-primary-bg">{{ file.weightTotal | fixed:1 }} g</div>
          </div>
          <div class="value-list">
            <div><?php _('Lines') ?></div>
            <div class="text-primary-bg">{{ file.lines }}</div>
          </div>
          <div class="value-list">
            <div><?php _('Layer') ?></div>
            <div class="text-primary-bg">{{ file.layer }}</div>
          </div>
          <div class="value-list">
            <div><?php _('Printed') ?></div>
            <div class="text-primary-bg">{{ file.printed }} <?php _('times') ?></div>
          </div>
          <div class="value-list">
            <div><?php _('Uploaded') ?></div>
            <div class="text-primary-bg">{{ file.created | dateTimeShort }}</div>
          </div>
          <div class="value-list">
            <div>{{ tFileLength }}</div>
            <div class="text-primary-bg">{{ file.length | byte }}</div>
          </div>
          <div class="value-list">
            <div><?php _('Slicer') ?></div>
            <div class="text-primary-bg">{{ file.slicer }}</div>
          </div>
          <div class="value-list">
            <div><?php _('Volumetric Extrusion') ?></div>
            <div class="text-primary-bg">{{ file.volumetric | yesno }}</div>
          </div>

          <h3 style="margin-top: 10px"><?php _('Per Extruder Values') ?></h3>

          <div class="margin-bottom" *ngFor="let e of file?.perExtruder; let i = index">
            <h4><?php _('Extruder') ?> {{ e.extruder }}</h4>
            <div class="value-list">
              <div><?php _('Filament') ?></div>
              <div class="text-primary-bg">
                <rs-button class="primary" (rsClick)="selectFilament(i)">
                  <div>{{ e.filament.n }} <span class="caret"></span></div>
                </rs-button>
              </div>
            </div>
            <div class="value-list">
              <div><?php _('Length') ?></div>
              <div class="text-primary-bg">{{ e.mm | fixed:0 }}</div>
            </div>
            <div class="value-list">
              <div><?php _('Volume') ?></div>
              <div class="text-primary-bg">{{ e.volume | fixed:1 }} cm<sup>3</sup></div>
            </div>
            <div class="value-list">
              <div><?php _('Weight') ?></div>
              <div class="text-primary-bg">{{ e.weight | fixed:1 }} g</div>
            </div>
            <div class="value-list">
              <div><?php _('Material Costs') ?></div>
              <div class="text-primary-bg">{{ e.price | fixed:2 }} {{ file.currency }}</div>
            </div>
          </div>
        </div>
      </div>
    </div>
  `
})
export class GCodeInfoComponent implements OnDestroy {
  printer: PrinterState;
  waitPrintStart = "";
  splitscreen = false;
  @Input() file: GCodeFileState;
  // eslint-disable-next-line @angular-eslint/no-output-native
  @Output() close = new EventEmitter<boolean>();
  printers$: Observable<PrintersState>;
  tPrintingTime = "<?php _('Printing Time') ?>";
  tFileLength = "<?php _('File Length') ?>";

  private printersSubscription: SubscriptionLike;
  private eventSubscription: SubscriptionLike;

  constructor(public server: RepetierServer,
              public rsselect: RSSelect,
              public router: Router,
              private modal: RSModal,
              public printerAction: PrinterAction,
              private visualAid: VisualAids,
              private connectionHandler: ConnectionHandler,
              private store: Store<RepetierState>,
              private ref: ChangeDetectorRef) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe(printers => {
      if (this.printer !== printers.printers[printers.active]) {
        this.printer = printers.printers[printers.active];
        if (this.waitPrintStart.length > 0 && this.printer.jobid > 0 && this.printer.printing === true) {
          this.router.navigateByUrl("/print-status");
        }
        ref.markForCheck();
      }
    });
    this.eventSubscription = this.connectionHandler.serverEvents.subscribe(this.eventListener);
    window.addEventListener("resize", this.updateSize);
  }

  ngOnDestroy() {
    window.removeEventListener("resize", this.updateSize);
    this.printersSubscription.unsubscribe();
    this.eventSubscription.unsubscribe();
    if (this.waitPrintStart.length > 0) {
      this.visualAid.hideBusy(this.waitPrintStart);
    }
  }

  eventListener = (evt: ServerEvent) => {
    if ((evt.event === "jobDeactivated") && evt.slug === this.printer.slug) {
      if (this.waitPrintStart.length > 0) {
        this.visualAid.hideBusy(this.waitPrintStart);
        this.waitPrintStart = "";
      }
    }
  };

  updateSize = (): void => {
    this.splitscreen = document.body.clientWidth >= 800;
  };

  selectFilament(idx: number) {
    this.rsselect.select(getState(this.store).server.settings["pricing"].filaments, "n", "<?php _('Select Filament') ?>").then((f) => {
      let newFile = RSUtils.copyObject(this.file);
      // console.log('selected', f);
      newFile.perExtruder[idx].filament = f;
      newFile = this.printerAction.gcodeUpdatePricing(newFile);
      this.printerAction.updateGCode(newFile);
      this.file = newFile;
      this.ref.markForCheck();
    }, () => {
    });
  }

  print() {
    if (this.file.fits) {
      this.printNoQuestion();
    } else {
      this.modal.confirm("<?php _('Confirmation required') ?>", "<?php _('According to your defined printer limits, this print will not fit inside your printer. Print anyway?') ?>").then(() => {
        this.printNoQuestion();
      }, () => {
      });

    }
  }

  printNoQuestion() {

    if (this.file.type === 1) {
      if (this.printer.online === 1 && !this.printer.printing) {
        this.waitPrintStart = this.visualAid.showBusy("<?php _('Preparing print ...') ?>");
        this.printerAction.gcodePrint(this.file);
      }
    } else {
      const numWaiting = this.printer.queue.length;
      if (numWaiting === 0 && this.printer.online === 1 && !this.printer.printing) {
        this.waitPrintStart = this.visualAid.showBusy("<?php _('Preparing print ...') ?>");
        this.printerAction.gcodePrint(this.file);
      } else {
        this.printerAction.gcodePrint(this.file);
        this.router.navigateByUrl("/print-queue");
      }
    }
  }

  queue() {
    this.printerAction.gcodeQueue(this.file);
    this.router.navigateByUrl("/print-queue");
  }

  deleteFile() {
    this.modal.confirm("<?php _('Confirmation required') ?>", "<?php _('Do you really want to delete $1?') ?>".replace("$1", this.file.name)).then(() => {
      this.printerAction.gcodeDelete(this.file);
      this.close.emit(true);
    }, () => {
    });
  }
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-gcodes",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-icon *ngIf="(printer?.queue?.length>0) && printer?.config.general?.enableQueue" [counter]="printer?.queue?.length" icon="rs rs-queue" title="{{tPrintQueue}}" link="/print-queue"></rs-icon>
        <rs-empty-sideicon *ngIf="!((printer?.queue?.length>0) && printer?.config.general?.enableQueue)"></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon title="<?php _('Back') ?>" icon="fa fa-reply" (rsClick)="page=0" *ngIf="page>0 && !splitscreen"></rs-sideicon>
        <rs-sideicon *ngIf="splitscreen && printer?.lastPrints.length>0" icon="rs rs-last-prints-g" title="{{tLastPrints}}" link="/last-prints"></rs-sideicon>
      </rs-side>
      <main *ngIf="printer">
        <div *ngIf="page===0 && !splitscreen">
          <h1><i class="rs rs-file-gcode"></i> <?php _('G-Codes') ?></h1>
          <div class="content-h1" rsDrag>
            <div>
              <div class="margin-leftright margin-bottom">
                <rs-button class="secondary full" style="display:block" (hit)="selectGroup()">
                  <div>{{ group?.name }} <span class="caret"></span></div>
                </rs-button>
                <rs-button class="secondary full" style="display:block" (hit)="selectSortOrder()">
                  <?php _('Sort') ?>: {{ sortName() }} <span class="caret"></span></rs-button>
              </div>
              <div class="entry-list">
                <div class="entry left pointer" *ngFor="let f of printer?.modelsFiltered" (rsClick)="selectGCode(f)">
                  <div class="left">
                    <!-- <img [src]="printerAction.gcodeImageUrl(f.slug,f.list, f.id,'s',f.version)"> -->
                    <img [src]="printer.imageUrls[f.slug + '_' + f.list + f.id + 's']">
                  </div>
                  <h3>{{ f.name }}</h3>
                  <p>{{ f.printTimeBest | dhms }}</p>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="splitscreen" style="display: flex;">
          <div style="flex: 1;max-width:50%">
            <h1><i class="rs rs-file-gcode"></i> <?php _('G-Codes') ?></h1>
            <div class="content-h1" rsDrag>
              <div>
                <div class="margin-leftright margin-bottom">
                  <rs-button class="secondary full" style="display:block" (hit)="selectGroup()">
                    <div>{{ group?.name }} <span class="caret"></span></div>
                  </rs-button>
                  <rs-button class="secondary full" style="display:block" (hit)="selectSortOrder()">
                    <?php _('Sort') ?>: {{ sortName() }} <span class="caret"></span></rs-button>
                </div>
                <div class="entry-list">
                  <div class="entry left pointer" *ngFor="let f of printer?.modelsFiltered" (rsClick)="selectGCode(f)">
                    <div class="left">
                      <!-- <img [src]="printerAction.gcodeImageUrl(f.slug,f.list, f.id,'s',f.version)"> -->
                      <img [src]="printer.imageUrls[f.slug + '_' + f.list + f.id + 's']">
                    </div>
                    <h3>{{ f.name }}</h3>
                    <p>{{ f.printTimeBest | dhms }}</p>
                  </div>
                </div>
              </div>
            </div>
          </div>
          <div style="flex: 1;max-width:50%">
            <rs-gcode-info *ngIf="activeGCode" [file]="activeGCode"></rs-gcode-info>
          </div>
        </div>
        <rs-gcode-info *ngIf="page===1 && !splitscreen" (close)="page=0" [file]="activeGCode"></rs-gcode-info>
      </main>
    </rs-page>
  `
})
export class GCodesComponent implements OnDestroy {
  tPrintQueue = "<?php _('Print Queue') ?>";
  tLastPrints = "<?php _('Last Prints') ?>";
  group: GroupState = undefined;
  printer: PrinterState = undefined;
  activeGCode: GCodeFileState = undefined;
  page = 0;
  splitscreen = false;
  printers$: Observable<PrintersState>;
  sortNames = [
    {id: 2, name: "<?php _('Created Desc.') ?>"},
    {id: 3, name: "<?php _('Created Asc.') ?>"},
    {id: 1, name: "<?php _('Name Asc.') ?>"},
    {id: 0, name: "<?php _('Name Desc.') ?>"},
    {id: 5, name: "<?php _('Print Time Asc.') ?>"},
    {id: 4, name: "<?php _('Print Time Desc.') ?>"},
    {id: 11, name: "<?php _('Filament Usage Asc.') ?>"},
    {id: 10, name: "<?php _('Filament Usage Desc.') ?>"}
  ];
  private printersSubscription: SubscriptionLike;

  constructor(public modal: RSModal,
              public server: RepetierServer,
              public rsselect: RSSelect,
              public printerAction: PrinterAction,
              private ref: ChangeDetectorRef,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe(printers => {
      if (!printers.active) {
        return;
      } // on load it can be undefined!
      if (this.printer !== printers.printers[printers.active]) {
        this.printer = printers.printers[printers.active];
        if (this.printer.groupsSorted && this.printer.groupsSorted.length > 0) {
          this.group = this.printer.groups[this.printer.activeGroupId];
          this.activeGCode = this.printerAction.gcodeGetLatestVersion(this.activeGCode);
        }
        ref.markForCheck();
      }
    });
    window.addEventListener("resize", this.updateSize);
    this.updateSize();
  }

  ngOnDestroy() {
    window.removeEventListener("resize", this.updateSize);
    this.printersSubscription.unsubscribe();
  }

  sortName() {
    return this.sortNames.filter(x => x.id === this.printer.sortBy)[0].name;
  }

  updateSize = (): void => {
    this.splitscreen = document.body.clientWidth >= 800;
  };

  selectGroup() {
    this.rsselect.select(this.printer.groupsSorted, "name", "<?php _('Select Group') ?>").then((grp) => {
      this.printerAction.setActiveGroupId(this.printer.slug, grp.id);
    }, () => {
    });
  }

  selectSortOrder() {
    this.rsselect.select(this.sortNames, "name", "<?php _('Sorting Order') ?>").then((grp) => {
      this.printerAction.dispatch(this.printer.slug, PrinterSortByAction, grp.id);
      this.printerAction.dispatch(this.printer.slug, PrinterSortModelsAction, {});
    }, () => {
    });
  }

  selectGCode(gc: any) {
    this.activeGCode = gc;
    this.page = 1;
  }

  protected readonly undefined = undefined;
}

