/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */


import {Component, OnDestroy} from "@angular/core";
import {
  ServerFolderState, RSBrowseFolder, GroupState, PrintersState,
  ServerState, PrinterState, PrinterAction, INITIAL_SERVER_FOLDER_STATE,
  ConnectionHandler, RepetierState
} from "binding";
import {RSSelect} from "front-ui";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-import-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="page===1" title="<?php _('Refresh') ?>" icon="fa fa-refresh" (rsClick)="refresh()"></rs-sideicon>
        <rs-empty-sideicon *ngIf="page!==1"></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="server.serverFolders.length > 1 && page===1" title="<?php _('Back') ?>" icon="fa fa-reply" (rsClick)="page=0"></rs-sideicon>
        <rs-empty-sideicon *ngIf="!(server.serverFolders.length > 1 && page===1)"></rs-empty-sideicon>
      </rs-side>
      <main>
        <div *ngIf="page===0 && active">
          <rs-headline><i class="rs rs-usbstick"></i> <?php _('Import') ?></rs-headline>
          <div class="content-h1" rsDrag>
            <div>
              <div class="margin-leftright">
                <div *ngFor="let f of server.serverFolders" class="margin-bottom">
                  <rs-button (hit)="selectFolder(f)" class="secondary full"><h3>{{ f.name }}</h3></rs-button>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===1">
          <rs-headline><i class="rs rs-usbstick"></i> <?php _('Import')?></rs-headline>
          <div class="content-h1" rsDrag>
            <div>
              <div class="margin-leftright">
                <div>
                  <?php _('Folder') ?>: {{ bFolder.activeFolderPath }}
                </div>
                <div class="entry-list">
                  <div class="entry left buttons" [class.single2]="p.isDir" *ngFor="let p of bFolder?.filesInPath">
                    <rs-button class="primary left small-icon" (hit)="bFolder.toggleSelectFile(p)">
                      <i class="fa" [class.fa-download]="!p.isDir" [class.fa-arrow-circle-right]="p.isDir"></i>
                    </rs-button>
                    <h3><i [class]="p.icon"></i> {{ p.name }}</h3>
                    <p *ngIf="!p.isDir">{{ p.size | byte }}</p>
                  </div>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===2">
          <rs-headline><?php _('Importing ...')?></rs-headline>
          <div class="content-h1" rsDrag>
            <div>
              <div class="margin-leftright">
                <p>{{ transImport() }}</p>
                <rs-button class="secondary margin-leftright margin-bottom" style="display:block" (hit)="selectGroup()">
                  <h3>{{ group?.name }}</h3></rs-button>
                <rs-button class="secondary margin-leftright margin-bottom" style="display:block" (hit)="importModel()">
                  <h3>{{ transImportTo(group?.name) }}</h3></rs-button>
                <br>
                <rs-button *ngIf="active.queue.length===0" class="secondary margin-leftright margin-bottom" style="display:block" (hit)="importJob()">
                  <h3><?php _('Print File') ?></h3></rs-button>
                <rs-button *ngIf="active.queue.length!==0" class="secondary margin-leftright margin-bottom" style="display:block" (hit)="importJob()">
                  <h3>{{ tAddPrintQueue }}</h3></rs-button>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})
export class ImportComponent implements OnDestroy {
  private selected: ServerFolderState;

  page = 0; // 0 = folder select, 1 = file select
  group: GroupState;
  folder: ServerFolderState = Object.assign({}, INITIAL_SERVER_FOLDER_STATE);
  bFolder: RSBrowseFolder;
  importMsg: string;
  printers: PrintersState;
  active: PrinterState;
  server: ServerState;

  printers$: Observable<PrintersState>;
  server$: Observable<ServerState>;

  private printersSubscription: SubscriptionLike;
  private serverSubscription: SubscriptionLike;
  tAddPrintQueue = "<?php _('Add to Print Queue') ?>";

  constructor(private connectionHandler: ConnectionHandler,
              public rsselect: RSSelect,
              private printerAction: PrinterAction,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.server$ = store.select("server");
    this.printersSubscription = this.printers$.subscribe(p => {
      this.printers = p;
      if (p.active) {
        this.active = this.printers.printers[p.active];
        this.group = this.active.groups[this.active.activeGroupId];
      }
    });
    this.serverSubscription = this.server$.subscribe((s: ServerState) => {
      this.server = s;
      if (this.server.serverFolders.length === 1 && this.page === 0) {
        this.selectFolder(this.server.serverFolders[0]);
      }
    });
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
    this.serverSubscription.unsubscribe();
  }

  selectGroup(): void {
    this.rsselect.select(this.active.groupsSortedNoAll, "name", "<?php _('Select Group') ?>").then((grp: GroupState) => {
      this.printerAction.setActiveGroupId(this.active.slug, grp.id);
    }, () => {
    });
  }


  transImportTo(g: string): string {
    return "<?php _('Import to $') ?>".replace("$", g);
  }

  transImport(): string {
    return "<?php _('Where to import $?') ?>".replace("$", this.selected.name);
  }

  importModel(): void {
    this.printerAction.importFiles(this.active.slug, this.bFolder, [this.selected], false);
    this.page = 1;
  }

  importJob() {
    this.printerAction.importFiles(this.active.slug, this.bFolder, [this.selected], true);
    this.page = 1;
  }

  refresh() {
    if (this.bFolder) {
      this.bFolder.updateFiles();
    }
  }

  selectFolder(f: ServerFolderState) {
    this.bFolder = new RSBrowseFolder(this.connectionHandler, f);
    this.page = 1;
    this.bFolder.importFileCallback = (imp: ServerFolderState) => {
      this.selected = imp;
      this.page = 2;
    };
  }
}
