/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from "@angular/core";
import {
  PrintersState, LicenseState, LicenseAction,
  ServerState, PrinterState, PrinterAction, INITIAL_SERVER_FOLDER_STATE,
  ConnectionHandler, RepetierState, WifiState, ServerEvent
} from "binding";
import {RSSelect} from "front-ui";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-info-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon icon="fa fa-gear" title="<?php _('Settings') ?>" link="/settings"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="(wifi$ | async).manageable && (server$|async).features['config-network']" title="<?php _('Network') ?>" icon="fa fa-sitemap" link="/wlan"></rs-sideicon>
        <rs-empty-sideicon *ngIf="!((wifi$ | async).manageable && (server$|async).features['config-network'])"></rs-empty-sideicon>
      </rs-side>
      <main>
        <div *ngIf="page===0">
          <rs-headline><i class="fa fa-info-circle"></i> <?php _('Info') ?></rs-headline>
          <div class="content-h1" rsDrag>
            <div>
              <div class="margin-leftright">
                <table class="w100">
                  <tr>
                    <td colspan="2"><h2><?php _('PROJECT_NAME') ?></h2></td>
                    <td>
                    <td></td>
                  </tr>
                  <tr>
                    <td><?php _('Installed Version') ?>:</td>
                    <td>{{ (license$|async).currentVersion }}</td>
                  </tr>
                  <tr>
                    <td><?php _('Latest Version') ?>:</td>
                    <td>{{ (license$|async).versionName }}</td>
                  </tr>
                  <tr>
                    <td><?php _('Update') ?>:</td>
                    <td>
          <span *ngIf="(license$|async).updateAvailable && (server$|async).features['update-server']">
            <rs-button *ngIf="!(license$|async).isUpdating" class="button primary" (hit)="startUpdate()">
              <h3><?php _('Autoupdate to Latest Version') ?></h3>
            </rs-button>
             <span *ngIf="(license$|async).isUpdating" style="font-size:150%">
               <i class="fa fa-spinner fa-spin"></i>
               <?php _('Updating ...') ?></span>
          </span>
                      <span *ngIf="!(license$|async).updateAvailable">
            <i class="fa fa-thumbs-o-up"></i>
                        <?php _('You are already using the latest version.') ?>
          </span>
                    </td>
                  </tr>
                </table>
                <h2 class="margin-top"><i class="fa fa-sitemap"></i> <?php _('Network Interfaces') ?></h2>
                <div style="display:flex;flex-wrap: wrap;justify-content: space-between">

                  <div *ngIf="server.settings.externalAccess?.domain" style="padding: 10px; margin:0 auto;">
                    <h2><?php _('Network') ?></h2>
                    <div style="text-align: center;margin:8px">
                      <p>
                        {{ connectionHandler.httpType }}://{{ server.settings.externalAccess.domain }}
                        :{{ server.settings.externalAccess.port }}<br>
                        &nbsp;
                      </p>
                      <rs-qrcode [ip]="server.settings.externalAccess.domain" [port]="server.settings.externalAccess.port" margin="2" modSize="5"></rs-qrcode>
                    </div>
                  </div>
                  <div *ngFor="let n of net.ipv4" style="padding: 10px; margin:0 auto;">
                    <h2>{{ n.name }}</h2>
                    <div style="text-align: center;margin:8px">
                      <p>
                        {{ connectionHandler.httpType }}://{{ n.ip }}:{{ net.port }}<br>
                        MAC: {{ n.mac }}
                      </p>
                      <rs-qrcode [ip]="n.ip" [port]="net.port" margin="2" modSize="5"></rs-qrcode>
                    </div>
                  </div>
                  <div *ngFor="let n of net.ipv6" style="padding: 10px; margin:0 auto;">
                    <h2>{{ n.name }}</h2>
                    <div style="text-align: center;margin:8px">
                      <p>
                        {{ connectionHandler.httpType }}://[{{ n.ip }}]:{{ net.port }}<br>
                        MAC: {{ n.mac }}
                      </p>
                      <rs-qrcode [ip]="'['+n.ip+']'" [port]="net.port" margin="2" modSize="5"></rs-qrcode>
                    </div>
                  </div>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===1">
        </div>
      </main>
    </rs-page>
  `
})
export class InfoComponent implements OnDestroy {
  page = 0; // 0 = folder select, 1 = file select
  printers: PrintersState;
  active: PrinterState;
  server: ServerState;

  license$: Observable<LicenseState>;
  printers$: Observable<PrintersState>;
  server$: Observable<ServerState>;

  private printersSubscription: SubscriptionLike;
  private serverSubscription: SubscriptionLike;

  wifi$: Observable<WifiState>;
  net: any = {ipv4: []};
  private connectionSubscription: SubscriptionLike;

  constructor(public connectionHandler: ConnectionHandler,
              public rsselect: RSSelect,
              private printerAction: PrinterAction,
              public licenseAction: LicenseAction,
              private store: Store<RepetierState>) {
    this.server$ = store.select("server");
    this.printers$ = store.select("printers");
    this.license$ = store.select("license");
    this.printersSubscription = this.printers$.subscribe(p => {
      this.printers = p;
      if (p.active) {
        this.active = this.printers.printers[p.active];
      }
    });
    this.serverSubscription = this.server$.subscribe((s: ServerState) => {
      this.server = s;
    });
    this.wifi$ = store.select("wifi");
    this.connectionSubscription = connectionHandler.connectionState.subscribe(on => {
      if (on) {
        this.updateInterfaces();
      }
    });
    connectionHandler.send("checkForUpdates", {}, "");
  }

  updateInterfaces() {
    this.connectionHandler.networkInterfaces().then((data) => {
      this.net = data;
    }, () => {
    });
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
    this.serverSubscription.unsubscribe();
    this.connectionSubscription.unsubscribe();
  }

  startUpdate(): void {
    this.licenseAction.autoupdate();
  }

  protected readonly undefined = undefined;
}
