/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {AfterViewInit, Component, ElementRef, OnDestroy, OnInit, ViewChild} from "@angular/core";
import {RSUIHelper} from "front-ui";
import {PrinterAction, PrintersAction, PrinterState, PrintersState, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";
import {Router} from "@angular/router";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-move-xy-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="!showZ && printer?.online===1" title="<?php _('Z Move') ?>" icon="rs rs-move-z" link="/move-z-tool"></rs-sideicon>
        <rs-sideicon *ngIf="showZ && printer?.online===1" icon="rs rs-params" title="<?php _('Status') ?>" link="/params"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="printer" title="<?php _('Extrude') ?>" [icon]="printerAction.extruderIcon(printer?.slug,printer.activeExtruderId)" link="/move-e-tool"></rs-sideicon>
        <rs-empty-sideicon *ngIf="!printer"></rs-empty-sideicon>
      </rs-side>
      <main>
        <div #head style="margin:0;padding:0">
          <rs-headline><i class="rs rs-move-xy"></i> <?php _('Move Tool') ?></rs-headline>
        </div>
        <div *rsIfConnected="true">
          <div class="content-h1 centered-grid stacked" *ngIf="page===0 && printer">
            <!-- coordinates -->
            <div class="centered-grid" style="width:100%" #positions>
              <div class="col-3">
                X:<span class="coordinate" [innerHTML]="printer?.posX|coord:2"></span>
              </div>
              <div class="col-3">
                Y:<span class="coordinate" [innerHTML]="printer?.posY|coord:2"></span>
              </div>
              <div class="col-3">
                Z:<span class="coordinate" [innerHTML]="printer?.posZ|coord:2"></span>
              </div>
            </div>
            <div class="centered-grid" style="margin-left: 10px;width:100%" #gridpar>
              <!-- 3 x 3 move grid -->
              <div #gridxy class="regular-grid stacked h100" [style.width]="(percentPerRow*3)+'%'">
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'yUpXLeft')">
                      <i class="rs rs-arrow-h-up-left"></i></rs-icon-button>
                  </div>
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'yUp')">
                      <i class="rs rs-arrow-h-up"></i></rs-icon-button>
                  </div>
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'yUpXRight')">
                      <i class="rs rs-arrow-h-up-right"></i></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'xLeft')">
                      <i class="rs rs-arrow-h-left"></i></rs-icon-button>
                  </div>
                  <div class="col-3">
                    <rs-icon-button class="accent center" style="font-size:14px">{{ printer?.xyzStepSize|fixed:2 }}<br>mm</rs-icon-button>
                  </div>
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'xRight')">
                      <i class="rs rs-arrow-h-right"></i></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'yDownXLeft')">
                      <i class="rs rs-arrow-h-down-left"></i></rs-icon-button>
                  </div>
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'yDown')">
                      <i class="rs rs-arrow-h-down"></i></rs-icon-button>
                  </div>
                  <div class="col-3">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'yDownXRight')">
                      <i class="rs rs-arrow-h-down-right"></i></rs-icon-button>
                  </div>
                </div>
              </div>
              <div *ngIf="showZ" [style.width]="percentPerRow+'%'" class="regular-grid stacked h100">
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'zUp')">
                      <i class="rs rs-arrow-h-up"></i></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="accent center" style="font-size:14px">Z<br>{{ printer?.xyzStepSize|fixed:2 }} mm</rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'zDown')">
                      <i class="rs rs-arrow-h-down"></i></rs-icon-button>
                  </div>
                </div>
              </div>
              <!-- side functions -->
              <div [style.width]="percentPerRow+'%'" class="regular-grid stacked h100">
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="uiHelper. selectHome()">
                      <i class="fa fa-home"></i>
                      <h3><?php _('Home') ?></h3></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy" *ngIf="!big">
                  <div class="col-1">
                    <rs-icon-button class="primary center" (hit)="setPage(1)"><i class="rs rs-distance"></i>
                      <h3><?php _('Distance') ?></h3>
                    </rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy" *ngIf="printer?.config?.movement?.autolevel && big">
                  <div class="col-1">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'autolevel')">
                      <i class="rs rs-autolevel" style="font-size: 20px"></i>
                      <h3><?php _('Auto Level') ?></h3></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'motorsOff')">
                      <i class="rs rs-motor-off" style="font-size: 20px"></i>
                      <h3><?php _('Motors Off') ?></h3></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy" *ngIf="big">
                  <div class="col-1">
                    <rs-icon-button class="primary center" (hit)="goBabystep()"><i class="rs rs-move-z"></i>
                      <h3><?php _('Babystep') ?></h3>
                    </rs-icon-button>
                  </div>
                </div>
              </div>
            </div>
            <div *ngIf="big" class="centered-grid" style="width:100%" #distances>
              <div class="iconlist small" style="padding: 0;width: 100%;margin-left: 40px;">
                <div class="button-icon col6">
                  <rs-button class="secondary full" [class.outline]="printer?.xyzStepSize!==0.01" style="min-width: 5em;" (hit)="setXYStep(0.01)">
                    <h3 class="text single center">0.01</h3></rs-button>
                </div>
                <div class="button-icon col6">
                  <rs-button class="secondary full" [class.outline]="printer?.xyzStepSize!==0.1" style="min-width: 5em;" (hit)="setXYStep(0.1)">
                    <h3 class="text single center">0.1</h3></rs-button>
                </div>
                <div class="button-icon col6">
                  <rs-button class="secondary full" [class.outline]="printer?.xyzStepSize!==1" style="min-width: 5em;" (hit)="setXYStep(1)">
                    <h3 class="text single center">1</h3></rs-button>
                </div>
                <div class="button-icon col6">
                  <rs-button class="secondary full" [class.outline]="printer?.xyzStepSize!==10" style="min-width: 5em;" (hit)="setXYStep(10)">
                    <h3 class="text single center">10</h3></rs-button>
                </div>
                <div class="button-icon col6">
                  <rs-button class="secondary full" [class.outline]="printer?.xyzStepSize!==25" style="min-width: 5em;" (hit)="setXYStep(25)">
                    <h3 class="text single center">25</h3></rs-button>
                </div>
                <div class="button-icon col6">
                  <rs-button class="secondary full" [class.outline]="printer?.xyzStepSize!==50" style="min-width: 5em;" (hit)="setXYStep(50)">
                    <h3 class="text single center">50</h3></rs-button>
                </div>
              </div>
            </div>
          </div>

          <div class="content-h1 " *ngIf="page===1" rsDrag>
            <h2>{{ tSelectStepDistance }}</h2>
            <div class="iconlist">
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(0.01)"><h3 class="text single center">0.01 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(0.1)"><h3 class="text single center">0.1 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(1)"><h3 class="text single center">1 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(10)"><h3 class="text single center">10 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(25)"><h3 class="text single center">25 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(50)"><h3 class="text single center">50 mm</h3></rs-button>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>`
})
export class MoveComponent implements OnInit, OnDestroy, AfterViewInit {
  page = 0;
  printer: PrinterState = undefined;
  showZ = false;
  big = false;
  rows: number;
  percentPerRow: number;
  tSelectStepDistance = "<?php _('Select Step Distance') ?>";
  @ViewChild("gridxy") gridxy: ElementRef;
  @ViewChild("gridpar") gridpar: ElementRef;
  @ViewChild("head", {static: true}) head: ElementRef;
  @ViewChild("positions") positions: ElementRef;
  printers$: Observable<PrintersState>;

  private printersSubscription: SubscriptionLike;

  constructor(public printerAction: PrinterAction,
              public printersAction: PrintersAction,
              public uiHelper: RSUIHelper,
              public router: Router,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      const lastPrinter = this.printer;
      if (p.active) {
        this.printer = p.printers[p.active];
      }
      if (this.printer && lastPrinter && (lastPrinter !== this.printer || lastPrinter.online !== this.printer.online)) {
        this.updateHeight();
      }
    });
    this.updateHeight();
    document.addEventListener("resize", this.updateHeight, false);
  }

  updateHeight() {
    window.setTimeout(() => {
      this.showZ = document.body.clientWidth >= 640 && document.body.clientHeight >= 400;
      this.big = document.body.clientWidth >= 700 && document.body.clientHeight >= 479;
      this.rows = 4 + (this.showZ ? 1 : 0);
      this.percentPerRow = 100.0 / this.rows;
      if (!this.gridxy) {
        if (this.page === 0) {
          this.updateHeight();
        }
        return;
      }
      let cw = this.gridxy.nativeElement.clientWidth;
      let maxh = document.body.clientHeight - this.head.nativeElement.clientHeight - this.positions.nativeElement.clientHeight - 10;
      if (this.big) {
        maxh -= 74;
      }
      if (cw > maxh) {
        cw = maxh;
      }
      const h = cw + "px";
      if (h !== this.gridpar.nativeElement.style.height) {
        this.gridpar.nativeElement.style.height = cw + "px";
      }
    }, 100);
  }

  setPage(pg: number): void {
    this.page = pg;
    this.updateHeight();
  }

  ngOnInit() {
    this.printersAction.sendMoves();
  }

  ngOnDestroy() {
    this.printersSubscription.unsubscribe();
    this.printersAction.hideMoves();
    document.removeEventListener("resize", this.updateHeight);
  }

  ngAfterViewInit() {
    this.updateHeight();
  }

  setXYStep(v: number) {
    this.printerAction.updateState(this.printer.slug, {xyzStepSize: v});
    this.page = 0;
    this.updateHeight();
  }

  goBabystep(): void {
    this.router.navigateByUrl("/babystep");
  }

  protected readonly undefined = undefined;
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-move-z-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon title="<?php _('XY Move') ?>" icon="rs rs-move-xy" link="/move-tool"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="printer" title="<?php _('Extrude') ?>" [icon]="printerAction.extruderIcon(printer.slug,printer.activeExtruderId)" link="/move-e-tool"></rs-sideicon>
        <rs-empty-sideicon *ngIf="!printer"></rs-empty-sideicon>
      </rs-side>
      <main>
        <div #head style="margin:0;padding:0">
          <rs-headline><i class="rs rs-move-z"></i> <?php _('Z Move') ?></rs-headline>
        </div>
        <div *rsIfConnected="true">
          <div class="content-h1 centered-grid stacked" *ngIf="page===0">
            <div class="centered-grid" style="width:100%" #positions>
              <div class="col-3">
                X:<span class="coordinate" [innerHTML]="printer?.posX|coord:2"></span>
              </div>
              <div class="col-3">
                Y:<span class="coordinate" [innerHTML]="printer?.posY|coord:2"></span>
              </div>
              <div class="col-3">
                Z:<span class="coordinate" [innerHTML]="printer?.posZ|coord:2"></span>
              </div>
            </div>
            <div class="centered-grid w100" #gridpar style="flex-wrap: nowrap;">
              <div class="regular-grid stacked h100 col-2">
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'zUp')">
                      <i class="rs rs-arrow-h-up"></i></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="accent" style="font-size:14px">{{ printer?.xyzStepSize|fixed:2 }}<br>mm</rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'zDown')">
                      <i class="rs rs-arrow-h-down"></i></rs-icon-button>
                  </div>
                </div>
              </div>
              <div class="regular-grid stacked h100 col-2">
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="uiHelper.selectHome()"><i class="fa fa-home"></i>
                      <h3><?php _('Home') ?></h3></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary" (hit)="page=1"><i class="rs rs-distance"></i>
                      <h3><?php _('Distance') ?></h3>
                    </rs-icon-button>
                  </div>
                </div>
              </div>
              <div class="regular-grid stacked h100 col-2">
                <div class="row-3 regular-grid w100 center-content-xy" *ngIf="printer?.config?.movement?.autolevel">
                  <div class="col-1">
                    <rs-icon-button class="primary center" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'autolevel')">
                      <i class="rs rs-autolevel" style="font-size: 20px"></i>
                      <h3><?php _('Auto Level') ?></h3></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'motorsOff')">
                      <i class="rs rs-motor-off" style="font-size: 20px"></i>
                      <h3><?php _('Motors Off') ?></h3></rs-icon-button>
                  </div>
                </div>
                <div class="row-3 regular-grid w100 center-content-xy">
                  <div class="col-1">
                    <rs-icon-button class="primary center" (hit)="goBabystep()"><i class="rs rs-move-z"></i>
                      <h3><?php _('Babystep') ?></h3>
                    </rs-icon-button>
                  </div>
                </div>

              </div>
            </div>
          </div>
          <div class="content-h1" *ngIf="page===1" rsDrag>
            <h2>{{ tSelectStepDistance }}</h2>
            <div class="iconlist">
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(0.01)"><h3 class="text single center">0.01 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(0.1)"><h3 class="text single center">0.1 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(1)"><h3 class="text single center">1 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(10)"><h3 class="text single center">10 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(25)"><h3 class="text single center">25 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setXYStep(50)"><h3 class="text single center">50 mm</h3></rs-button>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>`
})
export class MoveZComponent implements OnInit, OnDestroy, AfterViewInit {
  page = 0;
  printer: PrinterState = undefined;
  tSelectStepDistance = "<?php _('Select Step Distance') ?>";
  @ViewChild("gridpar") gridpar: ElementRef;
  @ViewChild("head", {static: true}) head: ElementRef;
  @ViewChild("positions") positions: ElementRef;
  printers$: Observable<PrintersState>;

  private printersSubscription: SubscriptionLike;

  constructor(public printerAction: PrinterAction,
              public printersAction: PrintersAction,
              public router: Router,
              public uiHelper: RSUIHelper,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      const lastPrinter = this.printer;
      if (p.active) {
        this.printer = p.printers[p.active];
      }
      if (this.printer && lastPrinter && (lastPrinter !== this.printer || lastPrinter.online !== this.printer.online)) {
        this.updateHeight();
      }
    });
  }

  updateHeight() {
    window.setTimeout(() => {
      if (!this.gridpar) {
        if (this.page === 0) {
          this.updateHeight();
        }
        return;
      }
      const maxh = document.body.clientHeight - this.head.nativeElement.clientHeight - this.positions.nativeElement.clientHeight - 10;
      const h = maxh + "px";
      if (h !== this.gridpar.nativeElement.style.height) {
        this.gridpar.nativeElement.style.height = h;
      }
    }, 300);
  }

  ngOnInit() {
    this.printersAction.sendMoves();
  }

  ngOnDestroy() {
    this.printersAction.hideMoves();
    this.printersSubscription.unsubscribe();
  }

  ngAfterViewInit() {
    this.updateHeight();
  }

  setXYStep(v: number) {
    this.printerAction.updateState(this.printer.slug, {xyzStepSize: v});
    this.page = 0;
    this.updateHeight();
  }

  goBabystep(): void {
    this.router.navigateByUrl("/babystep");
  }

  protected readonly undefined = undefined;
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-move-e-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon title="<?php _('XY Move') ?>" icon="rs rs-move-xy" link="/move-tool"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="!showZ" title="<?php _('Z Move') ?>" icon="rs rs-move-z" link="/move-z-tool"></rs-sideicon>
        <rs-sideicon *ngIf="showZ" icon="rs rs-params" title="<?php _('Status') ?>" link="/params"></rs-sideicon>
      </rs-side>
      <main>
        <div #head style="margin:0;padding:0">
          <rs-headline>
            <i *ngIf="printer" [class]="printerAction.extruderIcon(printer.slug,printer.activeExtruderId)"></i> <?php _('Extrude and Retract') ?>
          </rs-headline>
        </div>
        <div *rsIfConnected="true">
          <div class="content-h1" *ngIf="page===0 && printer?.extruders" rsDrag>
            <div>
            <div class="margin-leftright centered-grid stacked">
              <div class="w100" #positions>
                <rs-button class="secondary full" (hit)="ui.selectExtruder()">
                  <i [class]="printerAction.extruderIcon(printer.slug,printer.activeExtruderId)"></i> {{ printerAction.extruderName(printer.slug, printer.activeExtruderId) }}
                  <span *ngIf="printerAction.toolSupportsTemperatures(printer.slug, printer.activeExtruderId)">&nbsp;{{ printer.extruders[printer.activeExtruderId]|tempFull:1 }}</span>

                </rs-button>
              </div>
              <div class="centered-grid w100" #gridpar>
                <!-- 3 x 3 move grid -->
                <div class="centered-grid h100">
                  <div class="col-2 center-content-xy">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'eRetract')">
                      <i class="rs rs-arrow-h-up"></i></rs-icon-button>
                  </div>
                  <div class="col-2 center-content-xy">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'eRetractFast')">
                      <i class="rs rs-arrow-h-double-up"></i></rs-icon-button>
                  </div>
                  <div class="col-1 center-content-xy">
                    <rs-icon-button class="accent" style="font-size:14px">{{ printer?.eStepSize|fixed:2 }}<br>mm
                    </rs-icon-button>
                  </div>
                  <div class="col-2 center-content-xy">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'eExtrude')">
                      <i class="rs rs-arrow-h-down"></i></rs-icon-button>
                  </div>
                  <div class="col-2 center-content-xy">
                    <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'eExtrudeFast')">
                      <i class="rs rs-arrow-h-double-down"></i></rs-icon-button>
                  </div>
                </div>
                <div class="regular-grid stacked h100">
                  <div class="row-3 regular-grid w100 center-content-xy" *ngIf="printerAction.toolSupportsTemperatures(printer.slug, printer.activeExtruderId)">
                    <div class="col-1">
                      <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="ui.setExtruderTemp(printer?.activeExtruderId)">
                        <i class="rs rs-temperature"></i>
                        <h3><?php _('Set Temp.') ?></h3></rs-icon-button>
                    </div>
                  </div>
                  <div *ngIf="!big" class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="primary" (hit)="page=1"><i class="rs rs-distance"></i>
                        <h3><?php _('Distance') ?></h3>
                      </rs-icon-button>
                    </div>
                  </div>
                  <div class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="primary" [disabled]="!printer?.idle" (hit)="printerAction.executeNamedFunction(printer.slug,'motorsOff')">
                        <i class="rs rs-motor-off" style="font-size: 20px"></i>
                        <h3><?php _('Motors Off') ?></h3></rs-icon-button>
                    </div>
                  </div>
                </div>
              </div>
              <div *ngIf="big" class="centered-grid" style="width:100%" #distances>
                <div class="iconlist small" style="padding: 0;width: 100%;margin-left: 40px;">
                  <div class="button-icon col6">
                    <rs-button class="secondary full" [class.outline]="printer?.eStepSize!==0.01" style="min-width: 5em;" (hit)="setEStep(0.01)">
                      <h3 class="text single center">0.01</h3></rs-button>
                  </div>
                  <div class="button-icon col6">
                    <rs-button class="secondary full" [class.outline]="printer?.eStepSize!==0.1" style="min-width: 5em;" (hit)="setEStep(0.1)">
                      <h3 class="text single center">0.1</h3></rs-button>
                  </div>
                  <div class="button-icon col6">
                    <rs-button class="secondary full" [class.outline]="printer?.eStepSize!==1" style="min-width: 5em;" (hit)="setEStep(1)">
                      <h3 class="text single center">1</h3></rs-button>
                  </div>
                  <div class="button-icon col6">
                    <rs-button class="secondary full" [class.outline]="printer?.eStepSize!==10" style="min-width: 5em;" (hit)="setEStep(10)">
                      <h3 class="text single center">10</h3></rs-button>
                  </div>
                  <div class="button-icon col6">
                    <rs-button class="secondary full" [class.outline]="printer?.eStepSize!==25" style="min-width: 5em;" (hit)="setEStep(25)">
                      <h3 class="text single center">25</h3></rs-button>
                  </div>
                  <div class="button-icon col6">
                    <rs-button class="secondary full" [class.outline]="printer?.eStepSize!==50" style="min-width: 5em;" (hit)="setEStep(50)">
                      <h3 class="text single center">50</h3></rs-button>
                  </div>
                </div>
              </div>
            </div>
            </div>
          </div>
          <div class="content-h1" *ngIf="page===1">
            <h2>{{ tSelectStepDistance }}</h2>
            <div class="iconlist">
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setEStep(0.01)"><h3 class="text single center">0.01 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setEStep(0.1)"><h3 class="text single center">0.1 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setEStep(1)"><h3 class="text single center">1 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setEStep(10)"><h3 class="text single center">10 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setEStep(25)"><h3 class="text single center">25 mm</h3></rs-button>
              </div>
              <div class="button-icon col2">
                <rs-button class="secondary outline full" (hit)="setEStep(50)"><h3 class="text single center">50 mm</h3></rs-button>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>`
})
export class MoveEComponent implements OnDestroy, AfterViewInit {
  page = 0;
  showZ = false;
  big = false;
  printer: PrinterState = undefined;
  tSelectStepDistance = "<?php _('Select Step Distance') ?>";
  @ViewChild("gridpar") gridpar: ElementRef;
  @ViewChild("head", {static: true}) head: ElementRef;
  @ViewChild("positions") positions: ElementRef;
  printers$: Observable<PrintersState>;

  private printersSubscription: SubscriptionLike;

  constructor(public printerAction: PrinterAction,
              public printersAction: PrintersAction,
              public ui: RSUIHelper,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.showZ = document.body.clientWidth >= 640 && document.body.clientHeight >= 400;
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      const lastPrinter = this.printer;
      if (p.active) {
        this.printer = p.printers[p.active];
      }
      if (this.printer && lastPrinter && (lastPrinter !== this.printer || lastPrinter.online !== this.printer.online)) {
        this.updateHeight();
      }
    });
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
  }

  updateHeight() {
    window.setTimeout(() => {
      if (!this.gridpar || !this.head) {
        if (this.page === 0) {
          this.updateHeight();
        }
        return;
      }
      this.showZ = document.body.clientWidth >= 640 && document.body.clientHeight >= 400;
      this.big = document.body.clientWidth >= 700 && document.body.clientHeight >= 479;
      let maxh = document.body.clientHeight - this.head.nativeElement.clientHeight - this.positions.nativeElement.clientHeight - 10;
      if (this.big) {
        maxh -= 74;
      }
      const h = maxh + "px";
      if (h !== this.gridpar.nativeElement.style.height) {
        this.gridpar.nativeElement.style.height = h;
      }
    }, 100);
  }

  ngAfterViewInit() {
    this.updateHeight();
  }

  setEStep(v: number) {
    this.printerAction.updateState(this.printer.slug, {eStepSize: v});
    this.page = 0;
    this.updateHeight();
  }
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-move-z-babystep-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfPrinting="true" title="<?php _('Print') ?>" icon="rs rs-print" link="/print-status"></rs-sideicon>
        <rs-empty-sideicon *rsIfPrinting="false"></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfConnected="true" icon="rs rs-params" title="<?php _('Status') ?>" link="/params"></rs-sideicon>
        <rs-empty-sideicon *rsIfPrinting="false"></rs-empty-sideicon>
      </rs-side>
      <main>
        <div #head style="margin:0;padding:0">
          <rs-headline><i class="rs rs-move-z"></i> <?php _('Babystep') ?></rs-headline>
        </div>
        <div *rsIfConnected="true">
          <div class="content-h1" rsDrag>
            <div class="centered-grid stacked">
              <div class="centered-grid" style="width:100%" #positions>
                <div class="col-3">
                  X:<span class="coordinate" [innerHTML]="printer?.posX|coord:2"></span>
                </div>
                <div class="col-3">
                  Y:<span class="coordinate" [innerHTML]="printer?.posY|coord:2"></span>
                </div>
                <div class="col-3">
                  Z:<span class="coordinate" [innerHTML]="printer?.posZ|coord:2"></span>
                </div>
              </div>
              <div class="centered-grid w100" #gridpar>
                <div class="regular-grid stacked h100 col-4">
                  <div class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="primary" (hit)="babystepZ(0.01)"><i class="rs rs-arrow-h-up"></i></rs-icon-button>
                    </div>
                  </div>
                  <div class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="accent" style="font-size:14px">{{ 0.01|fixed:2 }}<br>mm</rs-icon-button>
                    </div>
                  </div>
                  <div class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="primary" (hit)="babystepZ(-0.01)"><i class="rs rs-arrow-h-down"></i></rs-icon-button>
                    </div>
                  </div>
                </div>
                <div class="regular-grid stacked h100 col-4">
                  <div class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="primary" (hit)="babystepZ(0.1)"><i class="rs rs-arrow-h-up"></i></rs-icon-button>
                    </div>
                  </div>
                  <div class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="accent" style="font-size:14px">{{ 0.1|fixed:2 }}<br>mm</rs-icon-button>
                    </div>
                  </div>
                  <div class="row-3 regular-grid w100 center-content-xy">
                    <div class="col-1">
                      <rs-icon-button class="primary" (hit)="babystepZ(-0.1)"><i class="rs rs-arrow-h-down"></i></rs-icon-button>
                    </div>
                  </div>
                </div>
                <div class="h100 col-2" style="padding-top: 20px">
                  Moved Distance: {{ babydistance|fixed:2 }} mm<br><br>
                  <div class="alert info margin">
                    {{ tBabystepDelayed }}
                  </div>
                </div>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>`
})
export class MoveZBabystepComponent implements OnInit, OnDestroy, AfterViewInit {
  printer: PrinterState = undefined;
  babydistance = 0;
  @ViewChild("gridpar") gridpar: ElementRef;
  @ViewChild("head", {static: true}) head: ElementRef;
  @ViewChild("positions") positions: ElementRef;
  printers$: Observable<PrintersState>;

  private printersSubscription: SubscriptionLike;
  tBabystepDelayed = "<?php _('Babystepping changes z position without changing the official z position. Execution will be delayed by buffered moves. Please be patient when adding babysteps!') ?>";

  constructor(private printerAction: PrinterAction,
              private printersAction: PrintersAction,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      const lastPrinter = this.printer;
      if (p.active) {
        this.printer = p.printers[p.active];
      }
      if (this.printer && lastPrinter && (lastPrinter !== this.printer || lastPrinter.online !== this.printer.online)) {
        this.updateHeight();
      }
    });
  }

  updateHeight() {
    window.setTimeout(() => {
      // console.log("updh");
      if (!this.gridpar) {
        this.updateHeight();
        return;
      }
      const maxh = document.body.clientHeight - this.head.nativeElement.clientHeight - this.positions.nativeElement.clientHeight - 10;
      const h = maxh + "px";
      // console.log("new h"+h);
      if (h !== this.gridpar.nativeElement.style.height) {
        this.gridpar.nativeElement.style.height = h;
      }
    }, 300);
  }

  ngOnInit() {
    this.printersAction.sendMoves();
  }

  ngOnDestroy() {
    this.printersAction.hideMoves();
    this.printersSubscription.unsubscribe();
  }

  ngAfterViewInit() {
    this.updateHeight();
  }

  babystepZ(z: number): void {
    this.babydistance += z * (this.printer.config.movement.invertZ ? -1 : 1);
    this.printerAction.babystepZ(this.printer.slug, z * (this.printer.config.movement.invertZ ? -1 : 1));
  }

  protected readonly undefined = undefined;
}
