/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from "@angular/core";
import {RSUIHelper} from "front-ui";
import {PrinterState, PrintersState, PrinterAction, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {Router} from "@angular/router";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-params-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side *rsIfConnected="true">
        <rs-sideicon *ngIf="printer?.printing" title="<?php _('Print') ?>" icon="rs rs-print" link="/print-status"></rs-sideicon>
        <rs-sideicon *rsIfPrinting="false" icon="rs rs-move-xy" title="<?php _('Move') ?>" link="/move-tool"></rs-sideicon>
      </rs-side>
      <rs-side *rsIfConnected="false">
        <rs-empty-sideicon></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon title="<?php _('Temp.') ?>" icon="rs rs-chart" link="/temperatures"></rs-sideicon>
      </rs-side>
      <main>
        <rs-headline><i class="rs rs-params"></i> {{ tPrinterStatus }}</rs-headline>
        <div class="content-h1" style="padding: 0;" *ngIf="printer && printer.online === 1" rsDrag>
          <div class="iconlist">
            <ng-container *ngFor="let p of printer.extruders;let idx = index">
              <div class="button-icon sep-icon" *ngIf="printerAction.toolSupportsTemperatures(printer.slug, idx)">
                <rs-button class="secondary outline full" (hit)="ui.setExtruderTemp(idx)">
                  <i class="fw" [class]="printerAction.extruderIcon(printer.slug, idx)"></i>
                  <div class="multiline">
                    <span [class.strong]="idx===printer.activeExtruderId">{{ printerAction.extruderName(printer.slug, idx) }}</span><br>
                    {{ p | tempFull:1 }}
                  </div>
                </rs-button>
              </div>
            </ng-container>

            <div class="button-icon sep-icon" *ngFor="let p of printer.heatedBeds;let idx = index">
              <rs-button class="secondary outline full" (hit)="ui.setBedTemp(idx)">
                <i class="rs rs-heated-bed fw"></i>
                <div class="multiline">{{ printerAction.bedName(printer.slug, idx) }}<br>
                  {{ p | tempFull:1 }}
                </div>
              </rs-button>
            </div>
            <div class="button-icon sep-icon" *ngFor="let p of printer.heatedChambers;let idx = index">
              <rs-button class="secondary outline full" (hit)="ui.setChamberTemp(idx)">
                <i [class]="'fw '+printerAction.chamberIcon(printer.slug, idx)"></i>
                <div class="multiline">{{ printerAction.chamberName(printer.slug, idx) }}<br>
                  {{ p | tempFullConf:printer.config.heatedChambers[idx]:1 }}
                </div>
              </rs-button>
            </div>
            <div class="button-icon sep-icon" *ngFor="let p of printer.fans;let idx = index">
              <rs-button class="secondary outline full" (hit)="ui.setFan(idx)">
                <i class="rs rs-fan fw"></i>
                <div class="multiline">{{ printerAction.fanName(printer.slug, idx) }}<br>
                  <div *ngIf="p.on">{{ p.percent | fixed:0 }}%</div>
                  <div *ngIf="!p.on"><?php _('Off') ?></div>
                </div>
              </rs-button>
            </div>
            <div class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="ui.setSpeed()">
                <i class="rs rs-speed fw"></i>
                <div class="multiline"><?php _('Speed') ?><br>
                  {{ printer.speedMultiply }}%
                </div>
              </rs-button>
            </div>
            <div class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="ui.setFlow()">
                <i class="rs rs-flow fw"></i>
                <div class="multiline"><?php _('Flow') ?><br>
                  {{ printer.flowMultiply }}%
                </div>
              </rs-button>
            </div>
            <div class="button-icon sep-icon" *ngIf="printer?.config?.general?.softwareLight">
              <rs-button class="secondary outline full" (hit)="printerAction.toggleCaseLights(printer.slug)">
                <i class="rs rs-light fw"></i>
                <h3><?php _('Light On/Off') ?></h3>
              </rs-button>
            </div>
            <div class="button-icon sep-icon" *ngIf="printer?.config?.general?.softwarePower">
              <rs-button class="secondary outline full" (hit)="printerAction.power(printer.slug)">
                <i class="fa fa-power-off fw"></i>
                <h3><?php _('Power On/Off') ?></h3>
              </rs-button>
            </div>
            <div *rsIfPrinting="false" class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="printerAction.preheat(printer.slug, 2)">
                <i class="rs rs-temperature fw"></i>
                <h3><?php _('Preheat Active') ?></h3>
              </rs-button>
            </div>
            <div *rsIfPrinting="false" class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="printerAction.preheat(printer.slug, 1)">
                <i class="rs rs-temperature fw"></i>
                <h3><?php _('Preheat All') ?></h3>
              </rs-button>
            </div>
            <div *rsIfPrinting="false" class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="printerAction.cooldown(printer.slug)">
                <i class="rs rs-temperature fw"></i>
                <h3><?php _('Cooldown') ?></h3>
              </rs-button>
            </div>
            <div class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="goBabystep()">
                <i class="rs rs-move-z fw"></i>
                <h3><?php _('Babystep') ?></h3>
              </rs-button>
            </div>
            <div class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="goConsole()">
                <i class="rs rs-printer-commands fw"></i>
                <h3><?php _('Console') ?></h3>
              </rs-button>
            </div>
            <div class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="goMove()">
                <i class="rs rs-move-xy fw"></i>
                <h3><?php _('Move') ?></h3>
              </rs-button>
            </div>
            <div class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="goExtruder()">
                <i [class]="printerAction.extruderIcon(printer.slug,printer.activeExtruderId)+' fw'"></i>
                <h3><?php _('Extrude') ?></h3>
              </rs-button>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})

export class ParamsComponent implements OnDestroy {
  printer: PrinterState;
  printers$: Observable<PrintersState>;
  tPrinterStatus = "<?php _('Printer Status') ?>";

  private printersSubscription: SubscriptionLike;

  constructor(public ui: RSUIHelper,
              public printerAction: PrinterAction,
              public router: Router,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.printer = p.printers[p.active];
      } else {
        this.printer = undefined;
      }
    });
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
  }

  goBabystep(): void {
    this.router.navigateByUrl("/babystep");
  }

  goConsole(): void {
    this.router.navigateByUrl("/console");
  }

  goMove(): void {
    this.router.navigateByUrl("/move-tool");
  }

  goExtruder(): void {
    this.router.navigateByUrl("/move-e-tool");
  }

}
