/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, ChangeDetectorRef, OnDestroy} from "@angular/core";
import {PrintersState, PrinterState, PrintersAction, PrinterAction, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-printer-list",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="active?.printing" title="<?php _('Print') ?>" icon="rs rs-print" link="/print-status"></rs-sideicon>
        <rs-sideicon *ngIf="!active?.printing && active?.online===1" icon="rs rs-move-xy" title="<?php _('Move') ?>" link="/move-tool"></rs-sideicon>
        <rs-empty-sideicon *ngIf="!active?.printing && active?.online!==1"></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfConnected="true" icon="rs rs-params" title="<?php _('Status') ?>" link="/params"></rs-sideicon>
      </rs-side>
      <main>
        <rs-headline><i class="rs rs-printer"></i> <?php _('Printers') ?></rs-headline>
        <div class="content-h1 " rsDrag *ngIf="printers && active">
          <div>
            <div *ngIf="active" class="margin-leftright">
              {{ active.name }}
              <rs-button class="secondary" *ngIf="active.active" (hit)="printerAction.deactivate(active.slug)">
                <h3><?php _('Deactivate') ?></h3></rs-button>
              <rs-button class="secondary" *ngIf="!active.active" (hit)="printerAction.activate(active.slug)">
                <h3><?php _('Activate') ?></h3>
              </rs-button>
            </div>
            <div class="iconlist">
              <div class="button-icon sep-icon-always full" *ngFor="let printer of plist">
                <rs-button class="secondary outline full leftalign" (hit)="printersAction.setActiveSlug(printer.slug)">
                  <i>
                    <rs-radio [onValue]="printer.slug" [value]="printers.active" class="primary"></rs-radio>
                  </i>
                  <div class="multiline"><h3>{{ printer.name }}</h3>
                    <p *ngIf="printer.printing">{{ printer.printDone|fixed:1 }}% {{ printer.printJob }}</p>
                    <p *ngIf="!printer.printing"><span *ngIf="printer.online"><?php _('Idle') ?></span>
                      <span *ngIf="printer.online===0"><span *ngIf="printer.active"><?php _('Offline') ?></span><span *ngIf="!printer.active"><?php _('Disabled') ?></span></span>
                    </p></div>
                </rs-button>
              </div>
            </div>
            <div *ngIf="printers.sortedPrinters?.length===0" class="alert info margin">
              <?php _('No printers defined!') ?>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})

export class PrinterListComponent implements OnDestroy {
  printers$: Observable<PrintersState>;
  printers: PrintersState = undefined;
  active: PrinterState = undefined;
  plist: PrinterState[];
  private printersSubscription: SubscriptionLike;

  constructor(public printersAction: PrintersAction,
              private ref: ChangeDetectorRef,
              public printerAction: PrinterAction,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((x) => {
      this.printers = x;
      this.plist = [];
      for (const s of x.sortedPrinters) {
        this.plist.push(x.printers[s]);
      }
      if (x.active) {
        this.active = x.printers[x.active];
      } else {
        this.active = undefined;
      }
      this.ref.markForCheck();
    });
  }

  ngOnDestroy() {
    this.printersSubscription.unsubscribe();
  }
}
