/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from '@angular/core';
import {RepetierServer, PrinterState, PrintersState, GroupState, GCodeFileState, RepetierState, ConnectionHandler, PrinterAction } from "binding";
import {Observable, SubscriptionLike} from 'rxjs';
import {Store} from "@ngrx/store";
import {RSIntKeyboard} from "front-ui";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
    selector: 'rs-print-queue',
    template: `
      <rs-page>
        <rs-side>
          <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
        </rs-side>
        <rs-side>
            <rs-sideicon *rsIfPrinting="true" title="<?php _('Print') ?>" icon="rs rs-print" link="/print-status"></rs-sideicon>
            <rs-sideicon *rsIfPrinting="false" icon="rs rs-file-gcode" title="<?php _('G-Codes') ?>" link="/gcodes"></rs-sideicon>
        </rs-side>
        <rs-side>
          <rs-sideicon title="<?php _('Back') ?>" icon="fa fa-reply" (rsClick)="page=0" *ngIf="page>0 && !splitscreen"></rs-sideicon>
          <rs-empty-sideicon *ngIf="!(page>0 && !splitscreen)"></rs-empty-sideicon>
          <rs-sideicon *ngIf="splitscreen && printer?.lastPrints.length>0" icon="rs rs-last-prints-g" title="{{tLastPrints}}" link="/last-prints"></rs-sideicon>
        </rs-side>
        <main>
          <div *ngIf="page===0 && !splitscreen">
            <rs-headline><i class="rs rs-queue"></i> {{tPrintQueue}}</rs-headline>
            <div class="content-h1" rsDrag><div>
              <div class="entry-list">
                <div class="entry left" *ngFor="let f of printer?.queue" (rsClick)="selectGCode(f)">
                  <div class="left">
                    <img [src]="f.imgT" alt="G-Code Image">
                  </div>
                  <h3>{{f.name}}</h3>
                  <p>{{f.printTimeBest | dhms}}</p>
                </div>
              </div>

              <div *ngIf="printer?.queue.length===0" class="alert info margin">
                {{tPrintQueueIsEmpty}}
              </div>
            </div>
            </div>
          </div>
          <div *ngIf="splitscreen" style="display: flex;">
            <div style="flex: 1;max-width:50%">
              <h1><i class="rs rs-queue"></i> {{tPrintQueue}}</h1>
              <div class="content-h1" rsDrag>
                <div>
                <div class="entry-list">
                  <div *ngFor="let f of printer?.queue" class="entry left">
                    <div (rsClick)="selectGCode(f)">
                      <div class="left">
                        <img [src]="f.imgT" alt="G-Code Image">
                      </div>
                      <h3>{{f.name}}</h3>
                      <p>{{f.printTimeBest | dhms}}</p>
                    </div>
                    <div class="right" style="display: flex; align-items: center;">
                      <i class="fa fa-spinner fa-spin margin-leftright" style="width: 100%; height: 100%" *ngIf="f.id === printer.jobid"></i>
                      <rs-button (rsClick)="editQueueRepeat(f)" class="button primary">{{f.repeat}}x</rs-button>
                    </div>
                  </div>
                </div>

                <div *ngIf="printer?.queue.length===0" class="alert info margin">
                  {{tPrintQueueIsEmpty}}
                </div>
                </div>
              </div>
            </div>
            <div style="flex: 1;max-width:50%">
              <rs-gcode-info *ngIf="activeGCode" [file]="activeGCode" (close)="activeGCode=undefined"></rs-gcode-info>
            </div>
          </div>
          <rs-gcode-info *ngIf="page===1 && !splitscreen" (close)="page=0" [file]="activeGCode"></rs-gcode-info>
        </main>
      </rs-page>
    `
})
export class PrintQueueComponent implements OnDestroy {
    group: GroupState;
    activeGCode: GCodeFileState;
    printer: PrinterState;
    active: PrinterState = undefined;
    splitscreen = false;
    printers$: Observable<PrintersState>;
    private printersSubscription: SubscriptionLike;
    page = 0;
    tPrintQueueIsEmpty = "<?php _('Print queue is empty!') ?>";
    tPrintQueue = "<?php _('Print Queue') ?>";
    tLastPrints = "<?php _('Last Prints') ?>";
    constructor(public server: RepetierServer,
                private store: Store<RepetierState>,
                private intKbd: RSIntKeyboard,
                private connectionHandler: ConnectionHandler,
                private printerAction: PrinterAction) {
      this.printers$ = store.select('printers');
        this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
            if (p.active) {
                this.printer = p.printers[p.active];
                this.group = this.printer.groups[this.printer.activeGroupId];
            } else {
                this.printer = undefined;
            }
        });
        window.addEventListener("resize", this.updateSize);
        this.updateSize();
    }

    ngOnDestroy(): void {
        window.removeEventListener("resize", this.updateSize);
        this.printersSubscription.unsubscribe();
    }

    updateSize = (): void => {
        this.splitscreen = document.body.clientWidth >= 800;
    }

    selectGCode(gc: GCodeFileState) {
        this.activeGCode = gc;
        this.page = 1;
    }

    editQueueRepeat(queue: GCodeFileState) {
        this.intKbd.getInt(queue.repeat, "<?php _('Repetitions planned') ?>", {
          min: 1,
        }).then((value) => {
          this.connectionHandler.send("setRepeatJob", {id: queue.id, repeat: value}).then((response) => {
            if (response.ok) {
              this.printerAction.updateData(this.printer.slug);
            }
          });
      });
    }

  protected readonly undefined = undefined;
}
