/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy, AfterContentInit} from "@angular/core";
import {PrinterState, PrintersState, PrinterAction, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {RSUIHelper} from "front-ui";
import {Router} from "@angular/router";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-print-status",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfConnected="true" title="<?php _('Temp.') ?>" icon="rs rs-chart" link="/temperatures"></rs-sideicon>
        <rs-empty-sideicon *rsIfConnected="false"></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfConnected="true" title="<?php _('Status') ?>" icon="rs rs-params" link="/params"></rs-sideicon>
        <rs-empty-sideicon *rsIfConnected="false"></rs-empty-sideicon>
      </rs-side>
      <main>
        <div *rsIfPrinting="true">
          <rs-headline>
            <span *ngIf="printer.paused"><i class="fa fa-pause"></i> <?php _('Paused') ?></span><span *ngIf="!printer.paused"><i class="fa fa-play"></i> <?php _('Printing') ?></span>
            <span *ngIf="!wideScreen">: {{ printer?.printJob }}</span></rs-headline>
          <div class="content-h1" rsDrag>
            <div>
              <div class="centered-grid stacked" id="innerDiv" [style]="innerDivStyle">
                <div class="regular-grid" style="width: 100%;justify-content: space-around;padding-bottom:10px">
                  <rs-gauge [width]="height2" [height]="height2" [value]="printer.printDone" [rangeWidth]="rangeWidth" [valueWidth]="rangeWidth" divider="true" type="primary">
                    <span class="text-primary-bg" style="font-size:0.6rem"><?php _('ETE') ?></span><br>
                    {{ printer.printedTime - printer.printedTimeComp | hmsShort }}
                    <div style="height:5px"></div>
                    {{ (printers?.stateTime + 1000 * (printer.printedTime - printer.printedTimeComp)) | timeShort }}
                    <br>
                    <span class="text-primary-bg" style="font-size:0.6rem"><?php _('ETA') ?></span>
                  </rs-gauge>
                  <!-- <img [src]="printerAction.printingImage(printer.slug,'m')" style="border:1px solid #000;" [style.height]="height1+'px'"> -->
                  <img [src]="printer.imageUrls[printer.slug + '_jobs' + printer.jobid + 'm']" style="border:1px solid #000;" [style.height]="height1+'px'">
                </div>
                <div style="width: 100%;">
                  <div class="regular-grid" [style]="bottomExtraStyle">
                    <div [style]="bottomStyle1">
                      <div class="value-list">
                        <div><i class="fa fa-arrow-up" style="width:2em"></i> <span class="hide-max-320"><?php _('Z Position') ?></span>
                        </div>
                        <div class="text-primary-bg">{{ printer?.posZ | fixed:2 }} mm</div>
                      </div>
                      <div class="value-list">
                        <div><i class="fa fa-bars" style="width:2em"></i> <span class="hide-max-320"><?php _('Layer') ?></span></div>
                        <div class="text-primary-bg">{{ printer?.layer }} / {{ printer?.ofLayer }}</div>
                      </div>
                      <div class="value-list" *ngIf="printer?.config?.extruders?.length>0 && printerAction.toolSupportsTemperatures(printer.slug, printer.activeExtruderId)">
                        <div><i [class]="printerAction.extruderIcon(printer.slug, printer.activeExtruderId)" style="width:2em"></i>
                          <span class="hide-max-320">{{ printerAction.extruderName(printer.slug, printer.activeExtruderId) }}</span>&nbsp;
                        </div>
                        <div class="text-primary-bg">{{ printer?.extruders[printer.activeExtruderId] | tempFull:0 }}
                        </div>
                      </div>
                      <div class="value-list" *ngFor="let p of printer.heatedBeds;let idx = index">
                        <div><i class="rs rs-heated-bed" style="width:2em"></i>
                          <span class="hide-max-320">{{ printerAction.bedName(printer.slug, idx) }}</span></div>
                        <div class="text-primary-bg">{{ p | tempFull:0 }}
                        </div>
                      </div>
                      <div class="value-list" *ngFor="let p of printer.heatedChambers;let idx = index">
                        <div><i [class]="printerAction.chamberIcon(printer.slug, idx)" style="width:2em"></i>
                          <span class="hide-max-320">{{ printerAction.chamberName(printer.slug, idx) }}</span></div>
                        <div class="text-primary-bg">{{ p | tempFullConf:printer.config.heatedChambers[idx]:0 }}
                        </div>
                      </div>
                      <div *ngIf="showExtraParams">
                        <div class="value-list" *ngFor="let p of printer.fans;let idx = index">
                          <div><i class="rs rs-fan" style="width:2em"></i>
                            <span class="hide-max-320">{{ printerAction.fanName(printer.slug, idx) }}</span></div>
                          <div class="text-primary-bg">{{ p.percent | fixed:0 }}%</div>
                        </div>
                        <div class="value-list" *ngIf="showExtraParams">
                          <div><i class="rs rs-speed" style="width:2em"></i> <span class="hide-max-320"><?php _('Speed') ?></span></div>
                          <div class="text-primary-bg">{{ printer.speedMultiply | fixed:0 }}%</div>
                        </div>
                        <div class="value-list" *ngIf="showExtraParams">
                          <div><i class="rs rs-flow" style="width:2em"></i> <span class="hide-max-320"><?php _('Flow') ?></span></div>
                          <div class="text-primary-bg">{{ printer.flowMultiply | fixed:0 }}%</div>
                        </div>
                      </div>
                    </div>
                    <div [style]="bottomStyle2">
                      <div *ngIf="wideScreen" style="text-align: center; margin-bottom:10px">{{ printer?.printJob }}</div>
                      <div class="regular-grid stacked" style="width:auto; flex-direction: row; flex-wrap: wrap;justify-content: space-around;margin:0 -5px">
                        <div style="min-width: 120px;padding: 0 5px;">
                          <rs-button class="primary full" (hit)="printerAction.pausePrint(printer.slug)" *ngIf="!printer?.paused">
                            <i class="fa fa-pause"></i>
                            <h3><?php _('Pause') ?></h3>
                          </rs-button>
                          <rs-button class="primary full" (hit)="printerAction.unpausePrint(printer.slug)" [disabled]="printer.pauseState===3" *ngIf="printer?.paused">
                            <i class="fa fa-play"></i>
                            <h3>{{ pauseNames[printer.pauseState] }}</h3>
                          </rs-button>
                        </div>
                        <div style="min-width: 120px;">
                          <rs-button class="error full" (hit)="ui.stopPrint()">
                            <i class="fa fa-stop"></i>
                            <h3><?php _('Stop') ?></h3>
                          </rs-button>
                        </div>
                      </div>
                      <div style="margin-top: 15px;text-align: center;">
                        <rs-switch (checkedChange)="printerAction.toggleShutdownAfterPrint(printer.slug)" [checked]="shutdownAfterPrint" style="margin:auto;white-space: nowrap;overflow: hidden"></rs-switch>
                        <?php _('Shutdown after Print') ?></div>
                    </div>
                  </div>
                </div>
              </div>
            </div>
          </div>
          <div *rsIfPrinting="false">
            <h1><?php _('Idle') ?></h1>
            <div class="content-h1 margin-leftright15">
              <div class="alert info margin">
                {{ tPrinterIsIdle }}
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>`
})
export class PrintStatusComponent implements OnDestroy, AfterContentInit {
  printers: PrintersState = undefined;
  printer: PrinterState;
  active: PrinterState = undefined;
  printers$: Observable<PrintersState>;
  private printersSubscription: SubscriptionLike;
  height1 = 100;
  height2 = 100;
  rangeWidth = 12;
  showExtraParams = false;
  wideScreen = false;
  tPrinterIsIdle = "<?php _('Printer is idle') ?>";
  pauseNames: any = ["<?php _('Pause') ?>", "<?php _('Continue') ?>", "<?php _('Preheat') ?>", "<?php _('Heating...') ?>", "<?php _('Continue') ?>", "<?php _('Paused') ?>", "<?php _('Restarting...') ?>"];
  innerDivStyle = "flex-wrap: nowrap;";
  isDestroyed = false;
  bottomExtraStyle = "margin: -5px 0;width:100%;";
  bottomStyle1 = "flex-grow:1;flex-shrink:1;flex-basis:230px;min-width:140px;margin:0 auto;padding:5px 10px;";
  bottomStyle2 = "flex-grow:1;flex-shrink:1;flex-basis:260px;min-width:120px;margin:0 auto;padding:5px 10px;";

  constructor(public ui: RSUIHelper,
              public router: Router,
              public printerAction: PrinterAction,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      this.printers = p;
      if (p.active) {
        this.printer = p.printers[p.active];
        if (this.printer.printing === false) {
          this.router.navigateByUrl("/");
        }
      } else {
        this.printer = undefined;
      }
    });
    // this.updateSize();
    window.addEventListener("resize", this.updateSize);
  }

  ngAfterContentInit() {
    this.updateSize();
  }

  get shutdownAfterPrint() {
    return this.printer.shutdownAfterPrint;
  }

  set shutdownAfterPrint(value: boolean) {
    this.printerAction.toggleShutdownAfterPrint(this.printer.slug);
  }

  updateSize = (): void => {
    const mainArr = document.getElementsByTagName("main");
    if (!mainArr || mainArr.length === 0) {
      return;
    }
    const mainWidth = mainArr[0].clientWidth;
    let maxWidth = (mainWidth - 20) / 2.34;
    if (document.body.clientHeight > 300) {
      this.height1 = Math.min(maxWidth, 0.45 * document.body.clientHeight);
    } else {
      this.height1 = Math.min(maxWidth, 100);
    }
    const fullElemArr = document.getElementsByClassName("content-h1");
    if (fullElemArr.length === 0) {
      if (!this.isDestroyed) {
        setTimeout(this.updateSize, 50);
      }
      return;
    }
    const fullHeight = fullElemArr[0].clientHeight;
    if (this.height1 > 200) {
      this.height2 = 200;
      maxWidth = (mainWidth - 220) / 1.34;
      if (document.body.clientHeight > 300) {
        this.height1 = Math.min(maxWidth, 0.45 * document.body.clientHeight);
      } else {
        this.height1 = Math.min(maxWidth, 100);
      }
    } else {
      this.height2 = this.height1;
    }
    this.showExtraParams = document.body.clientHeight > 400;
    this.rangeWidth = Math.min(18, this.height1 * 0.06);
    if (this.height1 < 110) {
      this.rangeWidth = 6;
    }
    this.wideScreen = window.innerWidth >= 800;
    const innderDivElem = document.getElementById("innerDiv");
    if (!innderDivElem) {
      if (!this.isDestroyed) {
        setTimeout(this.updateSize, 50);
      }
      return;
    }
    const innerHeight = innderDivElem.clientHeight;
    // console.log('sizes', mainWidth, fullHeight, innerHeight, this.height1);
    if (innerHeight < fullHeight) {
      this.innerDivStyle = `flex-wrap: nowrap;height:${fullHeight}px;justify-content: space-around`;
    } else {
      this.innerDivStyle = "flex-wrap: nowrap;";
    }
    if (mainWidth < 531) { // bottom parts collapse
      this.bottomExtraStyle = "margin: -5px 0;width:100%;flex-direction: column-reverse;";
      this.bottomStyle1 = "flex-grow:1;flex-shrink:1;min-width:140px;margin:0 auto;padding:5px 10px;";
      this.bottomStyle2 = "flex-grow:1;flex-shrink:1;min-width:120px;margin:0 auto;padding:5px 10px;";
    } else {
      this.bottomExtraStyle = "margin: -5px 0;width:100%;";
      this.bottomStyle1 = "flex-grow:1;flex-shrink:1;flex-basis:230px;min-width:140px;margin:0 auto;padding:5px 10px;";
      this.bottomStyle2 = "flex-grow:1;flex-shrink:1;flex-basis:260px;min-width:120px;margin:0 auto;padding:5px 10px;";
    }
  }

  ngOnDestroy(): void {
    this.isDestroyed = true;
    window.removeEventListener("resize", this.updateSize);
    this.printersSubscription.unsubscribe();
  }

  protected readonly undefined = undefined;
}
