/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from "@angular/core";
import {PrinterState, PrintersState, PrinterAction, ConnectionHandler, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {Router} from "@angular/router";
import {RSModal} from "front-ui";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-recover",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-empty-sideicon *rsIfConnected="false"></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-empty-sideicon *rsIfConnected="false"></rs-empty-sideicon>
      </rs-side>
      <main>
        <rs-headline><i class="rs rs-print"></i> <?php _('Recover') ?></rs-headline>
        <div *rsIfConnected="true" class="content-h1" rsDrag>
          <div>
            <div class="margin-leftright">
              <div *ngIf="printer.recover.state===0">
                <div class="alert info margin-top">
                  <?php _('Printer not in rescue mode.') ?>
                </div>
              </div>
              <div *ngIf="printer.recover.state">
                <table class="">
                  <tr>
                    <th style="text-align: left"><?php _('Job') ?></th>
                    <td colspan="2">{{ printer.recover.name }}</td>
                  </tr>
                  <tr style="text-align: left">
                    <th><?php _('Fail Reason') ?></th>
                    <td colspan="2">{{ reason }}</td>
                  </tr>
                </table>
                <div class="valuelist">
                  <div class="button-icon  sep-icon" style="min-width:12em" *ngIf="printer.recover.lastExecKnown">
                    <rs-button class="secondary full" (hit)="startRecover(printer.recover.lastExecPos)">
                      <i class="fa fa-play"></i>
                      <h3 class="text" style="padding:8px"><?php _('Continue Print') ?></h3></rs-button>
                  </div>
                  <div class="button-icon  sep-icon" style="min-width:12em" *ngIf="!printer.recover.lastExecKnown">
                    <rs-button class="secondary full" (hit)="startRecover(printer.recover.lastSendPos)">
                      <i class="fa fa-play"></i>
                      <h3 class="text" style="padding:8px;white-space: normal;"><?php _('Continue from last send command') ?> (z
                        = {{ printer.recover.lastSendZ|fixed:2 }})</h3></rs-button>
                  </div>
                  <div class="button-icon  sep-icon" style="min-width:12em" *ngIf="!printer.recover.lastExecKnown">
                    <rs-button class="secondary full" (hit)="startRecover(printer.recover.lastACKPos)">
                      <i class="fa fa-play"></i>
                      <h3 class="text" style="padding:8px;white-space: normal;"><?php _('Continue from last ack command') ?> (z
                        = {{ printer.recover.lastExecZ|fixed:2 }})</h3></rs-button>
                  </div>
                  <div class="button-icon  sep-icon" style="min-width:12em">
                    <rs-button class="error full" (hit)="stopRecover()">
                      <i class="fa fa-stop"></i>
                      <h3 class="text" style="padding:8px;white-space: normal;"><?php _('Stop Print') ?></h3></rs-button>
                  </div>
                  <div class="button-icon  sep-icon" style="min-width:12em">
                    <rs-button class="secondary outline full" (hit)="sendRecoverConnect()">
                      <i class="fa fa-play"></i>
                      <h3 class="text" style="padding:8px;white-space: normal;"><?php _('Resend Connection Script') ?></h3></rs-button>
                  </div>
                </div>
                <h4><?php _('Checklist') ?></h4>
                <ul>
                  <li><?php _('Is the print ok or has it fat blobs that will prevent a successful print?') ?></li>
                  <li><?php _('Is the object still on the bed?') ?></li>
                  <li><?php _('Is the bed height unchanged or home to z max in rescue procedure?') ?></li>
                  <li><?php _('Marked the correct line to continue the print? If the selection is hidden the exact position is known.') ?></li>
                  <li><?php _('You can preheat extruder to prefill active extruder.') ?></li>
                  <li><?php _('Clean ooze before continuing.') ?></li>
                </ul>
                <div class="alert info margin-top margin-bottom">
                  <?php _('The regular web interface offers more options!') ?>
                </div>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})
export class RecoverComponent implements OnDestroy {
  printer: PrinterState;
  active: PrinterState = undefined;
  printers$: Observable<PrintersState>;
  reason = "";

  private printersSubscription: SubscriptionLike;

  constructor(public printerAction: PrinterAction,
              private con: ConnectionHandler,
              private modal: RSModal,
              public router: Router,
              private store: Store<RepetierState>) {
    this.printers$ = store.select("printers");
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.printer = p.printers[p.active];
        if (this.printer.recover.state === 3) {
          this.reason = "<?php _('Lost Connection to Printer') ?>";
        } else if (this.printer.recover.state === 4) {
          this.reason = "<?php _('Printer lost Power') ?>";
        } else if (this.printer.recover.state === 2) {
          this.reason = "<?php _('Unknown Reason') ?>";
        }
      } else {
        this.printer = undefined;
      }
    });
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
  }

  startRecover(startPos): void {
    this.con.send("startRecover", {pos: startPos}).then(() => {
      this.router.navigateByUrl("/print-status");
    });
  }

  sendRecoverConnect(): void {
    this.con.send("sendRecoverConnect", {});
  }

  stopRecover(): void {
    this.modal.confirm("<?php _('Confirmation required') ?>", "<?php _('Really cancel interrupted print?') ?>").then(() => {
      this.con.send("cancelRecover", {}).then(() => {
        this.router.navigateByUrl("/");
      });
    }, () => {
    });
  }
}
