/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, ElementRef, OnDestroy} from "@angular/core";
import {PrinterAction, PrintersState, PrinterState, RepetierServer, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {RSSelect, RSFloatKeyboard, RSIntKeyboard} from "front-ui";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-timelapse-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side *ngIf="active && edit && edit.length > 0">
        <rs-sideicon icon="rs rs-webcam" title="<?php _('Webcam') ?>" link="/webcam"></rs-sideicon>
      </rs-side>
      <rs-side *ngIf="!(active && edit && edit.length > 0)">
        <rs-empty-sideicon></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *rsIfConnected="true" icon="rs rs-params" title="<?php _('Status') ?>" link="/params"></rs-sideicon>
        <rs-empty-sideicon *rsIfConnected="false"></rs-empty-sideicon>
      </rs-side>
      <main>
        <rs-headline><i class="rs rs-timelapse"></i> <?php _('Timelapse') ?></rs-headline>
        <div class="content-h1" rsDrag>
          <div>
            <div style="display: table;width:100%;height:99%;margin:0;padding:0;" *ngIf="active && edit && edit.length > 0">
              <div style="display: table-row;height:1px;">
                <div class="margin-leftright margin-bottom" *ngIf="edit.length > 1">
                  <rs-button (hit)="selectWebcam()" class="secondary full wrappable"><i class="rs rs-webcam"></i>
                    <h3><?php _('Webcam') ?> {{ webcamId + 1 }} <span class="caret"></span></h3></rs-button>
                </div>
                <div class="margin-leftright margin-bottom">
                  <rs-button (hit)="selectRecord()" class="secondary full wrappable"><h3><?php _('Timelapse Recording') ?>
                    : {{ record[edit[webcamId].timelapseSelected].name }} <span class="caret"></span></h3></rs-button>
                </div>
                <div class="margin-leftright margin-bottom">
                  <rs-button (hit)="selectSnapshotTime()" class="secondary full wrappable"><h3><?php _('Shoot Images') ?>
                    : {{ snapshotTime[edit[webcamId].timelapseMethod].name }} <span class="caret"></span></h3></rs-button>
                </div>
                <div class="margin-leftright margin-bottom" *ngIf="edit[webcamId].timelapseMethod===0">
                  <rs-button class="secondary outline full" (hit)="setSecondInterval()">
                    <h3><?php _('Every') ?> {{ edit[webcamId].timelapseInterval | fixed: 1 }} <?php _('seconds') ?></h3>
                  </rs-button>
                </div>
                <div class="margin-leftright margin-bottom" *ngIf="edit[webcamId].timelapseMethod===2">
                  <rs-button class="secondary outline full" (hit)="setMMInterval()">
                    <h3><?php _('Every') ?> {{ edit[webcamId].timelapseHeight | fixed:2 }} <?php _('mm in height') ?></h3>
                  </rs-button>
                </div>
                <div class="margin-leftright margin-bottom" *ngIf="edit[webcamId].timelapseMethod===1">
                  <rs-button class="secondary outline full" (hit)="setLayerInterval()">
                    <h3><?php _('Every') ?> {{ edit[webcamId].timelapseLayer }} <?php _('layer') ?></h3>
                  </rs-button>
                </div>
                <div class="margin-leftright margin-bottom">
                  <rs-button (hit)="selectFramerate()" class="secondary full wrappable"><h3><?php _('Framerate') ?>
                    : {{ edit[webcamId].timelapseFramerate }} <span class="caret"></span></h3></rs-button>
                </div>
                <div class="margin-leftright margin-bottom">
                  <rs-button class="secondary outline full wrappable" (hit)="setBitrate()">
                    <h3><?php _('Bitrate') ?> {{ edit[webcamId].timelapseBitrate }} kbps</h3>
                  </rs-button>
                </div>
                <div class="margin-leftright margin-bottom">
                  <rs-switch [checked]="edit[webcamId].forceSnapshotPosition" (checkedChange)="toggleSnapshotPos($event)"></rs-switch>
                  <?php _('Enforce position for snapshots') ?>
                </div>
                <div class="margin-leftright margin-bottom" *ngIf="edit[webcamId].forceSnapshotPosition">
                  <rs-button class="secondary outline full wrappable" (hit)="setSnapshotDelay()">
                    <h3><?php _('Pause Duration for Snapshot') ?> {{ edit[webcamId].snapshotDelay }} ms</h3>
                  </rs-button>
                </div>
                <div class="margin-leftright margin-bottom" *ngIf="edit[webcamId].forceSnapshotPosition">
                  <rs-button class="secondary outline full wrappable" (hit)="setSnapshotX()">
                    <h3><?php _('X Position for Snapshot') ?> {{ edit[webcamId].snapshotX | fixed:2 }} mm</h3>
                  </rs-button>
                </div>
                <div class="margin-leftright margin-bottom" *ngIf="edit[webcamId].forceSnapshotPosition">
                  <rs-button class="secondary outline full wrappable" (hit)="setSnapshotY()">
                    <h3><?php _('Y Position for Snapshot') ?> {{ edit[webcamId].snapshotY | fixed:2 }} mm</h3>
                  </rs-button>
                </div>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})
export class TimelapseComponent implements OnDestroy {
  printers$: Observable<PrintersState>;
  active: PrinterState = undefined;
  webcamId = 0;
  edit: any = null;
  editString = null;
  record = [
    {id: 0, name: "<?php _('Off') ?>", icon: "fa fa-times"},
    {id: 1, name: "<?php _('Next Print') ?>", icon: "fa fa-arrow-right"},
    {id: 2, name: "<?php _('Always') ?>", icon: "fa fa-retweet"}
  ];
  saveImages = [
    {id: 0, name: "<?php _('Off') ?>"},
  ];
  snapshotTime = [
    {id: 0, name: "<?php _('Every x seconds') ?>"},
    {id: 1, name: "<?php _('Every x layers') ?>"},
    {id: 2, name: "<?php _('Every x mm height') ?>"},
    {id: 3, name: "<?php _('At snapshot command') ?>"},
  ];
  framerates = [
    {id: 10, name: "10"},
    {id: 12, name: "12"},
    {id: 15, name: "15"},
    {id: 24, name: "24"},
    {id: 25, name: "25"},
    {id: 30, name: "30"},
    {id: 50, name: "50"},
    {id: 60, name: "60"}
  ];

  private subscription: SubscriptionLike;

  constructor(public server: RepetierServer,
              private store: Store<RepetierState>,
              private elem: ElementRef,
              private floatKbd: RSFloatKeyboard,
              private intKbd: RSIntKeyboard,
              private printerAction: PrinterAction,
              private rsselect: RSSelect) {
    this.printers$ = store.select("printers");
    this.subscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.active = p.printers[p.active];
        if (this.active.config && this.active.config.webcams) {
          const newEdit = JSON.stringify(this.active.config.webcams);
          if (JSON.stringify(this.edit) !== newEdit) {
            this.edit = JSON.parse(newEdit);
          }
        }
      }
    });
  }

  ngOnDestroy() {
    this.subscription.unsubscribe();
  }

  async selectWebcam() {
    try {
      const list = [];
      for (let i = 0; i < this.edit.length; i++) {
        list.push({id: i, name: "<?php _('Webcam') ?> " + (i + 1), icon: "rs rs-webcam"});
      }
      const data = await this.rsselect.select(list, "name", "<?php _('Webcam') ?>");
      this.webcamId = data.id;
    } catch {
    }
  }

  selectRecord() {
    this.rsselect.select(this.record, "name", "<?php _('Timelapse Recording') ?>").then((data) => {
      this.edit[this.webcamId].timelapseSelected = data.id;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    }, () => {
    });
  }

  selectFramerate() {
    this.rsselect.select(this.framerates, "name", "<?php _('Framerate') ?>").then((data) => {
      this.edit[this.webcamId].timelapseFramerate = data.id;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    }, () => {
    });
  }

  selectSnapshotTime() {
    this.rsselect.select(this.snapshotTime, "name", "<?php _('Shoot Images') ?>").then((data) => {
      this.edit[this.webcamId].timelapseMethod = data.id;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    }, () => {
    });
  }

  setSecondInterval() {
    this.floatKbd.getFloat(
      this.edit[this.webcamId].timelapseInterval,
      "<?php _('Every x seconds') ?>",
      {unit: "s", min: 1}).then((data) => {
      this.edit[this.webcamId].timelapseInterval = data;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    });
  }

  setMMInterval() {
    this.floatKbd.getFloat(
      this.edit[this.webcamId].timelapseHeight,
      "<?php _('Every x mm height') ?>",
      {unit: "mm", min: 0.01}).then((data) => {
      this.edit[this.webcamId].timelapseHeight = data;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    });
  }

  setLayerInterval() {
    this.intKbd.getInt(
      this.edit[this.webcamId].timelapseLayer,
      "<?php _('Every x layers') ?>",
      {unit: "mm", min: 1}).then((data) => {
      this.edit[this.webcamId].timelapseLayer = data;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    });
  }

  setBitrate() {
    this.intKbd.getInt(
      this.edit[this.webcamId].timelapseBitrate,
      "<?php _('Bitrate') ?>",
      {unit: "kbps", min: 1}).then((data) => {
      this.edit[this.webcamId].timelapseBitrate = data;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    });
  }

  toggleSnapshotPos(data) {
    this.edit[this.webcamId].forceSnapshotPosition = data;
    this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
  }

  setSnapshotDelay() {
    this.intKbd.getInt(
      this.edit[this.webcamId].snapshotDelay,
      "<?php _('Pause Duration for Snapshot') ?>",
      {unit: "ms", min: 0}).then((data) => {
      this.edit[this.webcamId].snapshotDelay = data;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    });
  }

  setSnapshotX() {
    this.floatKbd.getFloat(
      this.edit[this.webcamId].snapshotX,
      "<?php _('X Position for Snapshot') ?>",
      {unit: "mm"}).then((data) => {
      this.edit[this.webcamId].snapshotX = data;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    });
  }

  setSnapshotY() {
    this.floatKbd.getFloat(
      this.edit[this.webcamId].snapshotY,
      "<?php _('Y Position for Snapshot') ?>",
      {unit: "mm"}).then((data) => {
      this.edit[this.webcamId].snapshotY = data;
      this.printerAction.saveTimelapseConfig(this.active.slug, this.edit);
    });
  }

}

