/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from "@angular/core";
import {ServerState, ServerAction, ConnectionHandler, WifiState, WifiAction, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import moment from "moment";
import {RSSelect} from "front-ui";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-timesettings-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon icon="fa fa-gear" title="<?php _('Settings') ?>" link="/settings"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-empty-sideicon></rs-empty-sideicon>
      </rs-side>
      <main>
        <rs-headline><i class="fa fa-clock-o"></i> <?php _('Time & Format') ?></rs-headline>
        <div class="content-h1 " rsDrag>
          <div>
            <div class="margin-leftright">
              <table>
                <tr>
                  <td><?php _('Time Locale') ?>:</td>
                  <td>{{ localeText }}</td>
                </tr>
                <tr *ngIf="(wifi$ | async).timezone">
                  <td><?php _('Time Zone') ?></td>
                  <td>{{ (wifi$ | async).timezone }}</td>
                </tr>
                <tr>
                  <td><?php _('Time') ?>:</td>
                  <td>{{ now | timeShort }}</td>
                </tr>
                <tr>
                  <td><?php _('Date') ?>:</td>
                  <td>{{ now | dateShort }}</td>
                </tr>
              </table>
              <div class="margin-top">
                <rs-button class="secondary outline full" (hit)="selectLocale()"><h3><?php _('Change Locale') ?></h3></rs-button>
              </div>
              <div class="margin-top" *ngIf="(wifi$ | async).manageable">
                <rs-button class="secondary outline full margin-top" (hit)="selectTimezone()"><h3><?php _('Change Time Zone') ?></h3>
                </rs-button>
              </div>
              <div class="alert info margin-top margin-leftright">
                <?php _('The selected locale will be used in this interface to show times and dates. Time in the example is the current time - change time zone if it does not match your local time.') ?>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})

export class TimesettingsComponent implements OnDestroy {
  server$: Observable<ServerState>;
  wifi$: Observable<WifiState>;
  server: ServerState = undefined;
  localesMoment = [
    {code: "af", name: "Afrikaans"},
    {code: "sq", name: "Albanian"},
    {code: "ar", name: "Arabic"},
    {code: "ar-dz", name: "Arabic (Algeria)"},
    {code: "ar-kw", name: "Arabic (Kuwait)"},
    {code: "ar-ly", name: "Arabic (Lybia)"},
    {code: "ar-ma", name: "Arabic (Morocco)"},
    {code: "ar-sa", name: "Arabic (Saudi Arabia)"},
    {code: "ar-tn", name: "Arabic (Tunisia)"},
    {code: "hy-am", name: "Armenian"},
    {code: "az", name: "Azerbaijani"},
    {code: "bm", name: "Bambara"},
    {code: "eu", name: "Basque"},
    {code: "be", name: "Belarusian"},
    {code: "bn", name: "Bengali"},
    {code: "bs", name: "Bosnian"},
    {code: "br", name: "Breton"},
    {code: "bg", name: "Bulgarian"},
    {code: "my", name: "Burmese"},
    {code: "km", name: "Cambodian"},
    {code: "ca", name: "Catalan"},
    {code: "tzm", name: "Central Atlas Tamazight"},
    {code: "tzm-latn", name: "Central Atlas Tamazight Latin"},
    {code: "zh-cn", name: "Chinese (China)"},
    {code: "zh-hk", name: "Chinese (Hong Kong)"},
    {code: "zh-tw", name: "Chinese (Taiwan)"},
    {code: "cv", name: "Chuvash"},
    {code: "hr", name: "Croatian"},
    {code: "cs", name: "Czech"},
    {code: "da", name: "Danish"},
    {code: "nl", name: "Dutch"},
    {code: "nl-be", name: "Dutch (Belgium)"},
    {code: "en-au", name: "English (Australia)"},
    {code: "en-ca", name: "English (Canada)"},
    {code: "en-ie", name: "English (Ireland)"},
    {code: "en-il", name: "English (Israel)"},
    {code: "en-nz", name: "English (New Zealand)"},
    {code: "en-SG", name: "English (Singapore)"},
    {code: "en-gb", name: "English (United Kingdom)"},
    {code: "en", name: "English (United States)"},
    {code: "eo", name: "Esperanto"},
    {code: "et", name: "Estonian"},
    {code: "fo", name: "Faroese"},
    {code: "fi", name: "Finnish"},
    {code: "fr", name: "French"},
    {code: "fr-ca", name: "French (Canada)"},
    {code: "fr-ch", name: "French (Switzerland)"},
    {code: "fy", name: "Frisian"},
    {code: "gl", name: "Galician"},
    {code: "ka", name: "Georgian"},
    {code: "de", name: "German"},
    {code: "de-at", name: "German (Austria)"},
    {code: "de-ch", name: "German (Switzerland)"},
    {code: "el", name: "Greek"},
    {code: "gu", name: "Gujarati"},
    {code: "he", name: "Hebrew"},
    {code: "hi", name: "Hindi"},
    {code: "hu", name: "Hungarian"},
    {code: "is", name: "Icelandic"},
    {code: "id", name: "Indonesian"},
    {code: "ga", name: "Irish or Irish Gaelic"},
    {code: "it", name: "Italian"},
    {code: "it-ch", name: "Italian (Switzerland)"},
    {code: "ja", name: "Japanese"},
    {code: "jv", name: "Javanese"},
    {code: "kn", name: "Kannada"},
    {code: "kk", name: "Kazakh"},
    {code: "tlh", name: "Klingon"},
    {code: "gom-latn", name: "Konkani Latin script"},
    {code: "ko", name: "Korean"},
    {code: "ku", name: "Kurdish"},
    {code: "ky", name: "Kyrgyz"},
    {code: "lo", name: "Lao"},
    {code: "lv", name: "Latvian"},
    {code: "lt", name: "Lithuanian"},
    {code: "lb", name: "Luxembourgish"},
    {code: "mk", name: "Macedonian"},
    {code: "ms-my", name: "Malay"},
    {code: "ms", name: "Malay"},
    {code: "ml", name: "Malayalam"},
    {code: "dv", name: "Maldivian"},
    {code: "mt", name: "Maltese (Malta)"},
    {code: "mi", name: "Maori"},
    {code: "mr", name: "Marathi"},
    {code: "mn", name: "Mongolian"},
    {code: "me", name: "Montenegrin"},
    {code: "ne", name: "Nepalese"},
    {code: "se", name: "Northern Sami"},
    {code: "nb", name: "Norwegian Bokmål"},
    {code: "nn", name: "Nynorsk"},
    {code: "fa", name: "Persian"},
    {code: "pl", name: "Polish"},
    {code: "pt", name: "Portuguese"},
    {code: "pt-br", name: "Portuguese (Brazil)"},
    {code: "x-pseudo", name: "Pseudo"},
    {code: "pa-in", name: "Punjabi (India)"},
    {code: "ro", name: "Romanian"},
    {code: "ru", name: "Russian"},
    {code: "gd", name: "Scottish Gaelic"},
    {code: "sr", name: "Serbian"},
    {code: "sr-cyrl", name: "Serbian Cyrillic"},
    {code: "sd", name: "Sindhi"},
    {code: "si", name: "Sinhalese"},
    {code: "sk", name: "Slovak"},
    {code: "sl", name: "Slovenian"},
    {code: "es", name: "Spanish"},
    {code: "es-do", name: "Spanish (Dominican Republic)"},
    {code: "es-us", name: "Spanish (United States)"},
    {code: "sw", name: "Swahili"},
    {code: "sv", name: "Swedish"},
    {code: "tl-ph", name: "Tagalog (Philippines)"},
    {code: "tg", name: "Tajik"},
    {code: "tzl", name: "Talossan"},
    {code: "ta", name: "Tamil"},
    {code: "te", name: "Telugu"},
    {code: "tet", name: "Tetun Dili (East Timor)"},
    {code: "th", name: "Thai"},
    {code: "bo", name: "Tibetan"},
    {code: "tr", name: "Turkish"},
    {code: "uk", name: "Ukrainian"},
    {code: "ur", name: "Urdu"},
    {code: "ug-cn", name: "Uyghur (China)"},
    {code: "uz", name: "Uzbek"},
    {code: "uz-latn", name: "Uzbek Latin"},
    {code: "vi", name: "Vietnamese"},
    {code: "cy", name: "Welsh"},
    {code: "yo", name: "Yoruba Nigeria"},
    {code: "ss", name: "siSwati"}
  ];
  localeText: string;
  private serverSubscription: SubscriptionLike;
  energySend: boolean = undefined;
  intervalStop: any;
  now: number;
  tzList: any;
  tzList1: any;
  tzList2: any;
  tzRegion: string;

  constructor(private serverAction: ServerAction,
              private con: ConnectionHandler,
              public wifiAction: WifiAction,
              public rsselect: RSSelect,
              private store: Store<RepetierState>) {
    this.server$ = store.select("server");
    this.wifi$ = store.select("wifi");
    this.serverSubscription = this.server$.subscribe((p: ServerState) => {
      const last = this.server;
      this.server = p;
      if (!last || last.settings.touchTime !== this.server.settings.touchTime) {
        this.updateLocaleText();
      }
    });
    this.intervalStop = setInterval(() => {
      this.now = this.now = new Date().getTime();
    }, 1000);
  }

  ngOnDestroy(): void {
    this.serverSubscription.unsubscribe();
    clearInterval(this.intervalStop);
  }

  updateLocaleText(): void {
    this.localesMoment.forEach(x => {
      if (x.code === this.server.settings.touchTime) {
        this.localeText = x.name;
      }
    });
    moment.locale(this.server.settings.touchTime);
  }

  selectLocale(): void {
    this.rsselect.select(this.localesMoment, "name", "<?php _('Select Format') ?>").then((data) => {
      this.serverAction.updateSetting("touchTime", data.code);
      this.updateLocaleText();
    }, () => {
    });
  }

  selectTimezone(): void {
    this.con.send("timezoneList").then(
      (data) => {
        this.tzList = data.list;
        this.tzList1 = [];
        const added = {};
        data.list.forEach(x => {
          const part = x.split("/")[0];
          if (!added[part]) {
            added[part] = true;
            this.tzList1.push({name: part});
          }
        });
        this.rsselect.select(this.tzList1, "name", "<?php _('Time Zone Region') ?>").then((reg) => {
          this.selectRegion(reg.name);
        }, () => {
        });

      }, () => {
      }
    );
  }

  selectRegion(region:string): void {
    this.tzRegion = region;
    this.tzList2 = [];
    const search = region + "/";
    this.tzList.forEach(x => {
      if (x.indexOf(search) === 0) {
        this.tzList2.push({name: x.substring(search.length)});
      }
    });
    if (this.tzList2.length) {
      this.rsselect.select(this.tzList2, "name", "<?php _('Select Time Zone') ?>").then((reg) => {
        this.setTimezoneName(reg.name);
      }, () => {
      });
    } else {
      this.setTimezoneName("");
    }
  }

  setTimezoneName(tz): void {
    if (tz) {
      this.tzRegion += "/" + tz;
    }
    this.con.send("timezoneSet", {timezone: this.tzRegion}).then((data) => {
      if (data.ok) {
        this.wifiAction.updateRouterList();
      }
    });
  }
}
