/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
import {Component, OnDestroy} from "@angular/core";
import {RSModal, RSStringKeyboard, RSIntKeyboard, RSNetworkKeyboard} from "front-ui";
import {RepetierServer, WifiAction, ConnectionHandler, WifiState, VisualAids, RouterState, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-wlan-component",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon title="<?php _('Refresh') ?>" icon="fa fa-refresh" (rsClick)="wifiAction.refreshRouterList()"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-sideicon *ngIf="page!==1" title="{{tAP}}" icon="rs rs-accesspoint" (rsClick)="confAP()"></rs-sideicon>
        <rs-sideicon *ngIf="page===1" title="<?php _('WLAN') ?>" icon="fa fa-wifi" (rsClick)="confCon()"></rs-sideicon>
      </rs-side>
      <main>
        <div *ngIf="page===0">
          <h1><i class="fa fa-wifi"></i> <?php _('WLAN') ?></h1>
          <div class="content-h1" rsDrag style="align-items: flex-start;">
            <div>
              <div *ngIf="wifi.country==='00'" class="alert warning margin" style="margin-bottom:15px">
                <?php _('You need to select the correct region first to ensure the wifi adapter uses the correct frequencies for your country!') ?>
              </div>
              <div *ngIf="wifi?.mode===0" class="alert info margin">
                <?php _('No WLAN adapter installed') ?>
              </div>
              <div *ngIf="wifi?.mode===1" class="alert info margin">
                <?php _('Connected') ?>: {{ wifi.activeSSID }}
              </div>
              <div *ngIf="wifi?.mode===2" class="alert info margin">
                {{ tAccessPoint }}: {{ wifi.apSSID }}
              </div>
              <div *ngIf="wifi?.mode===3" class="alert info margin">
                <?php _('Unconnected') ?>
              </div>
              <div *ngIf="wifi?.manualWifi" class="alert info margin">
                <?php _('The WIFI handling is manually configured and can not be changed here!') ?>
              </div>
              <div class="iconlist center-icons" style="padding: 0 10px;" *ngIf="wifi?.manualWifi===false">
                <div class="button-icon full" *ngFor="let r of filteredRouters()">
                  <rs-button class="secondary outline full leftright" (hit)="editConnection(r)">
                    <div style="display:flex;justify-content: center">
                      <rs-dyn-icon class="hide-max-320" [icon]="r.secure ? 'fa fa-lock' : 'fa fa-unlock'"></rs-dyn-icon>
                      <div class="multiline">
                        <h3 [class.text]="!r.active">{{ r.SSID }}</h3>
                        <p>
                          {{ r.rate }} <?php _('Channel') ?>: {{ r.channel }}</p>
                      </div>
                    </div>
                    <div class="">
                      <div class="fa-stack" style="font-size:26px;height:26px">
                        <i class="rs rs-bars-4 fa-stack-1x" style="color:#606060"></i>
                        <i class="rs rs-bars-{{r.bars}} fa-stack-1x"></i>
                      </div>
                    </div>
                  </rs-button>
                </div>
              </div>
              <div class="margin-leftright" style="margin-top:20px;margin-bottom: 30px" *ngIf="wifi?.manualWifi===false">
                <rs-switch [checked]="ignoreOn" (checkedChange)="toggleIgnore()"></rs-switch>
                {{ tHideIgnoredAccessPoint }}
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===1 && wifi">
          <h1><i class="fa fa-wifi"></i> <?php _('Network settings') ?></h1>

          <div class="content-h1 margin-leftright" *ngIf="wifi.manageable" rsDrag>
            <div>
              <div *ngIf="wifi.country==='00' && wifi?.manualWifi===false" class="alert warning margin" style="margin-bottom:15px">
                <?php _('You need to select the correct region first to ensure the wifi adapter uses the correct frequencies for your country!') ?>
              </div>
              <h2><?php _('General') ?></h2>

              <div class="iconlist">
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeHostname()">
                    <div class="multiline"><strong>{{ tHostname }}</strong><br>
                      {{ wifi.hostname }}
                    </div>
                  </rs-button>
                </div>
                <div class="button-icon" *ngIf="wifi?.manualWifi===false">
                  <rs-button class="secondary outline full" (hit)="changeRegion()">
                    <div class="multiline"><strong><?php _('Region') ?></strong><br>
                      {{ wifi.country }}
                    </div>
                  </rs-button>
                </div>

              </div>

              <h2>{{"<?php _('Access Point') ?>"}}</h2>
              <div class="iconlist">
                <div class="button-icon" *ngIf="wifi.supportAP && wifi?.manualWifi===false">
                  <rs-button class="secondary outline full" (hit)="changeAPSSID()">
                    <div class="multiline"><strong><?php _('SSID') ?></strong><br>
                      {{ wifi.apSSID }}
                    </div>
                  </rs-button>
                </div>
                <div class="button-icon" *ngIf="wifi.supportAP && wifi?.manualWifi===false">
                  <rs-button class="secondary outline full" (hit)="changeAPPasswd()">
                    <div class="multiline"><strong><?php _('Password') ?></strong><br>*****</div>
                  </rs-button>
                </div>
                <div class="button-icon" *ngIf="wifi.supportAP && wifi?.manualWifi===false">
                  <rs-button class="secondary outline full" (hit)="changeAPChannel()">
                    <div class="multiline"><strong><?php _('Channel') ?></strong><br>
                      {{ wifi.channel }}
                    </div>
                  </rs-button>
                </div>
                <div class="button-icon" *ngIf="wifi.supportAP && wifi?.manualWifi===false">
                  <rs-button class="secondary outline full" (hit)="changeAPMode()">
                    <div class="multiline"><strong><?php _('Mode') ?></strong><br>
                      {{ wifi.apModes[wifi.apMode].n }}
                    </div>
                  </rs-button>
                </div>
              </div>

              <h2><?php _('Ethernet') ?></h2>

              <div>
                <rs-switch (checkedChange)="toggleIPV4Mode($event)" [checked]="wifi.ethernet.ipv4_method" trueValue="auto" falseValue="manual"></rs-switch>
                <?php _('IP V4 Automatic') ?>
              </div>
              <div class="iconlist" *ngIf="wifi.ethernet.ipv4_method==='manual'">
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEthV4Address()">
                    <div class="multiline"><strong><?php _('IP') ?></strong><br>
                      {{ wifi.ethernet.ipv4_addresses }}
                    </div>
                  </rs-button>
                </div>
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEthV4Nameserver()">
                    <div class="multiline"><strong><?php _('Nameserver') ?></strong><br>
                      {{ wifi.ethernet.ipv4_dns }}
                    </div>
                  </rs-button>
                </div>
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEthV4Gateway()">
                    <div class="multiline"><strong><?php _('Gateway') ?></strong><br>
                      {{ wifi.ethernet.ipv4_gateway }}
                    </div>
                  </rs-button>
                </div>

              </div>

              <div>
                <rs-switch (checkedChange)="toggleIPV6Mode($event)" [checked]="wifi.ethernet.ipv6_method" trueValue="auto" falseValue="manual"></rs-switch>
                <?php _('IP V6 Automatic') ?>
              </div>
              <div class="iconlist" *ngIf="wifi.ethernet.ipv6_method==='manual'">
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEthV6Address()">
                    <div class="multiline"><strong><?php _('IP') ?></strong><br>
                      {{ wifi.ethernet.ipv6_addresses }}
                    </div>
                  </rs-button>
                </div>
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEthV6Nameserver()">
                    <div class="multiline"><strong><?php _('Nameserver') ?></strong><br>
                      {{ wifi.ethernet.ipv6_dns }}
                    </div>
                  </rs-button>
                </div>
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEthV6Gateway()">
                    <div class="multiline"><strong><?php _('Gateway') ?></strong><br>
                      {{ wifi.ethernet.ipv6_gateway }}
                    </div>
                  </rs-button>
                </div>

              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===2"><!-- select channel -->
          <h1><i class="fa fa-wifi"></i> <?php _('Select Channel') ?></h1>
          <div class="content-h1" rsDrag>
            <div>
              <div class="iconlist">
                <div class="button-icon" *ngFor="let r of wifi?.channels" style="min-width:4em;flex-basis: initial;">
                  <rs-button class="secondary outline full" (hit)="setChannel(r)">
                    <h3 [class.text]="r===wifi.channel" style="text-align: center"><?php _('Channel') ?> {{ r }}</h3>
                  </rs-button>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===3"><!-- select region -->
          <h1><?php _('Select Region') ?></h1>
          <div class="content-h1" rsDrag>
            <div>
              <div class="margin-leftright">
                <p>{{ tRebootRequired }}</p></div>
              <div class="iconlist">
                <div class="button-icon" *ngFor="let r of wifi?.regions">
                  <rs-button class="secondary outline full" (hit)="setRegion(r)">
                    <h3>{{ r.country }}</h3>
                  </rs-button>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===4"><!-- select ap mode -->
          <h1>{{ tAPHandling }}</h1>
          <div class="content-h1 iconlist" rsDrag>
            <div class="button-icon" style="min-width: 100%;" *ngFor="let r of wifi?.apModes">
              <rs-button class="secondary outline full" (hit)="setApMode(r)">&nbsp;
                <h3>&nbsp;<span [style.strong]="r.id===wifi.apMode">{{ r.n }}</span>&nbsp;
                </h3>&nbsp;
              </rs-button>
            </div>
          </div>
        </div>
        <div *ngIf="page===5"><!-- edit connection -->
          <h1><i class="fa fa-wifi"></i> {{ edit.data.ssid }}</h1>
          <div class="content-h1" rsDrag style="align-items: flex-start;">
            <div>
              <div class="margin-leftright iconlist">
                <div class="button-icon sep-icon">
                  <rs-button class="secondary full" (hit)="cancelEdit()">
                    <i class="fa fa-chevron-left"></i>
                    <h3><?php _('Back') ?></h3>
                  </rs-button>
                </div>
                <div class="button-icon sep-icon">
                  <rs-button *ngIf="canConnect()" (hit)="connectEdit()" class="success full">
                    <i class="fa fa-plug"></i>
                    <h3><?php _('Connect') ?></h3>
                  </rs-button>
                </div>
                <div class="button-icon sep-icon">
                  <rs-button *ngIf="!edit.data.manualManaged" (hit)="saveEdit()" class="secondary full">
                    <i class="fa fa-save"></i>
                    <div class="multiline"><strong><?php _('Save') ?></strong></div>
                  </rs-button>
                </div>
                <div class="button-icon sep-icon">
                  <rs-button *ngIf="edit.data.active && !edit.data.manualManaged" (hit)="disconnectEdit()" class="error full">
                    <i class="fa fa-times"></i>
                    <h3>{{ tDeactiveUntilConnection }}</h3></rs-button>
                </div>
              </div>
              <div *ngIf="edit.data.manualManaged" class="alert info margin">
                {{ tDefinedInBootfile }}
              </div>
              <div class="iconlist" *ngIf="!edit.data.manualManaged">
                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="page=6">
                    <div class="multiline"><strong>{{ tAuthMethod }}</strong><br>
                      {{ passwordMethod() }}
                    </div>
                  </rs-button>
                </div>

                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEditPassword()">
                    <div class="multiline"><strong><?php _('Password') ?></strong><span *ngIf="edit.data.password.length>0"><br>*****</span>
                    </div>
                  </rs-button>
                </div>

                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEditV4Address()">
                    <div class="multiline"><strong><?php _('Additional Address') ?></strong><br>
                      {{ edit.data.ipv4Address }}
                    </div>
                  </rs-button>
                </div>

                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEditV4Nameserver()">
                    <div class="multiline"><strong><?php _('Additional Nameserver') ?></strong><br>
                      {{ edit.data.ipv4Nameserver }}
                    </div>
                  </rs-button>
                </div>

                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEditV4Gateway()">
                    <div class="multiline"><strong>{{ tGateway }}</strong><br>
                      {{ edit.data.ipv4Gateway }}
                    </div>
                  </rs-button>
                </div>

                <div class="button-icon">
                  <rs-button class="secondary outline full" (hit)="changeEditV4Mask()">
                    <div class="multiline"><strong><?php _('Network Mask Bits') ?></strong><br>
                      {{ edit.data.ipv4MaskBits }}
                    </div>
                  </rs-button>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div *ngIf="page===6"><!-- Select encryption method -->
          <h1><i class="fa fa-wifi"></i> {{ tAPHandling }}</h1>
          <div class="content-h1 iconlist" rsDrag>
            <div>
              <div class="button-icon" style="min-width: 100%;" *ngFor="let r of encryptionModes">
                <rs-button class="secondary outline full" (hit)="setEditEncryption(r.v)">&nbsp;
                  <h3>&nbsp;<span [style.strong]="r.v===edit.data.passwordMethod">{{ r.n }}</span>&nbsp;
                  </h3>&nbsp;
                </rs-button>
              </div>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})
export class WLANComponent implements OnDestroy {
  wifi$: Observable<WifiState>;
  wifi: WifiState;
  page = 0;
  ignoreOn = true;
  edit: RouterState = undefined;
  encryptionModes = [{v: "none", n: "<?php _('No Encryption') ?>"},
    {v: "psk2", n: "<?php _('PSK2') ?>"},
    {v: "psk", n: "<?php _('PSK') ?>"}];
  tAP = "<?php _('More') ?>"; // "<?php _('AP') ?>";
  tAccessPoint = "<?php _('Access Point') ?>";
  tHideIgnoredAccessPoint = "<?php _('Hide ignored Access Points') ?>";
  tHostname = "<?php _('Hostname') ?>";
  tAuthMethod = "<?php _('Authentication Method') ?>";
  tDeactiveUntilConnection = "<?php _('Deactivate until explicit connection') ?>";
  tRebootRequired = "<?php _('Changes to active region require a reboot to take effect!') ?>";
  tAPHandling = "<?php _('AP Handling') ?>";
  tDefinedInBootfile = "<?php _('This connection is explicitly defined in /boot/repetier-network.xml and can for that reason not be modified here. Please edit the file manually instead and restart the server.') ?>";
  tGateway = "<?php _('Gateway') ?>";

  private wifiSubscription: SubscriptionLike;

  constructor(public server: RepetierServer,
              public modal: RSModal,
              private stringKbd: RSStringKeyboard,
              private intKbd: RSIntKeyboard,
              private networkKbd: RSNetworkKeyboard,
              public wifiAction: WifiAction,
              private visualAid: VisualAids,
              private con: ConnectionHandler,
              private store: Store<RepetierState>) {
    this.wifi$ = store.select("wifi");
    this.wifiSubscription = this.wifi$.subscribe(w => this.wifi = w);
    this.wifiAction.refreshRouterList();
  }

  ngOnDestroy() {
    this.wifiSubscription.unsubscribe();
  }

  confAP() {
    this.page = 1;
  }

  confCon() {
    this.page = 0;
  }

  toggleIgnore() {
    this.ignoreOn = !this.ignoreOn;
  }

  toggleIPV4Mode(mode) {
    this.wifiAction.setEthernet({ipv4_method: mode});
  }

  changeEthV4Address() {
    this.networkKbd.getNetwork(this.wifi.ethernet.ipv4_addresses, "<?php _('Additional Address') ?>", {validate: "v4AddressNetmask"}).then((ipv4Address) => {
      this.wifiAction.setEthernet({ipv4_addresses: ipv4Address});
    }, () => {});
  }

  async changeEthV4Nameserver() {
    try {
      const ipv4Address = await this.networkKbd.getNetwork(this.wifi.ethernet.ipv4_dns, "<?php _('Nameserver IP') ?>", {validate: "v4Address"})
      this.wifiAction.setEthernet({ipv4_dns: ipv4Address});
    } catch (e) {}
  }

  changeEthV4Gateway() {
    this.networkKbd.getNetwork(this.wifi.ethernet.ipv4_gateway, "<?php _('Gateway IP') ?>", {validate: "v4Address"}).then((ipv4Address) => {
      this.wifiAction.setEthernet({ipv4_gateway: ipv4Address});
    }, () => {});
  }

  toggleIPV6Mode(mode) {
    this.wifiAction.setEthernet({ipv6_method: mode});
  }

  changeEthV6Address() {
    this.networkKbd.getNetwork(this.wifi.ethernet.ipv6_addresses, "<?php _('Additional Address') ?>", {validate: "v6AddressNetmask"}).then((ipv6Address) => {
      this.wifiAction.setEthernet({ipv6_addresses: ipv6Address});
    }, () => {});
  }

  changeEthV6Nameserver() {
    this.networkKbd.getNetwork(this.wifi.ethernet.ipv6_dns, "<?php _('Nameserver IP') ?>", {validate: "v6Address"}).then((ipv6Address) => {
      this.wifiAction.setEthernet({ipv6_dns: ipv6Address});
    }, () => {});
  }

  changeEthV6Gateway() {
    this.networkKbd.getNetwork(this.wifi.ethernet.ipv6_gateway, "<?php _('Gateway IP') ?>", {validate: "v6Address"}).then((ipv6Address) => {
      this.wifiAction.setEthernet({ipv6_gateway: ipv6Address});
    }, () => {});
  }

  filteredRouters():Array<RouterState> {
    const a = [];
    for (const i of this.wifi.routerList) {
      if (!i.data.ignore || !this.ignoreOn) {
        a.push(i);
      }
    }
    return a;
  }

  changeHostname() {
    this.stringKbd.getString(this.wifi.hostname, "<?php _('New Hostname') ?>", {
      minLength: 1,
      maxLength: 63,
      regex: "^[a-zA-Z0-9][a-zA-Z0-9-]*$"
    }).then((newhost) => {
      if (this.wifi.hostname !== newhost) {
        this.wifiAction.setHostname(newhost);
      }
    }, () => {
    });
  }

  changeAPSSID() {
    this.stringKbd.getString(this.wifi.apSSID, "<?php _('New SSID') ?>", {
      minLength: 2,
      regex: "^[a-zA-Z0-9:_-]{0,32}$"
    }).then((newssid) => {
      if (this.wifi.apSSID !== newssid) {
        this.wifiAction.setApSSID(newssid);
      }
    }, () => {
    });
  }

  changeAPPasswd() {
    this.stringKbd.getString("", "<?php _('New Password') ?>", {minLength: 8}).then((newpasswd) => {
      this.wifiAction.setApPassword(newpasswd);
    }, () => {
    });
  }

  changeAPChannel() {
    this.page = 2;
  }

  setChannel(c: number) {
    this.page = 1;
    this.wifiAction.setApChannel(c);
  }

  changeRegion() {
    this.page = 3;
  }

  setRegion(r) {
    this.page = 1;
    this.wifiAction.setRegion(r.iso);
  }

  changeAPMode() {
    this.page = 4;
  }

  setApMode(m) {
    this.page = 1;
    this.wifiAction.setApMode(m.id);
  }

  editConnection(c: RouterState) {
    this.page = 5;
    // this.edit = Object.assign({}, c);
    this.edit = JSON.parse((JSON.stringify(c)));
  }

  cancelEdit() {
    this.page = 0;
    this.edit = undefined;
  }

  connectEdit() {
    const msgId: string = this.visualAid.showBusy("<?php _('Connecting ...') ?>");
    setTimeout(() => { // fallback in case we miss message
      this.visualAid.hideBusy(msgId);
    }, 30000);
    this.con.send("wifiSetSSIDData", this.edit.data).then(() => {
      this.wifiAction.connect(this.edit.data.ssid); /* .then((data) => {
                this.visualAid.hideBusy(msgId);
                if (data.connected) {
                    this.page = 0;
                    this.edit = undefined;
                } else {
                    this.modal.message("<?php _('Connection') ?>", "<?php _('Connecting to WLAN failed.') ?>");
                }
            }, () => {
                this.visualAid.hideBusy(msgId);
                this.modal.message("<?php _('Connection') ?>", "<?php _('Connecting to WLAN failed.') ?>");
            }); */
      this.visualAid.hideBusy(msgId);
      this.page = 0;
    }, () => {
      this.visualAid.hideBusy(msgId);
      this.modal.message("<?php _('Connection') ?>", "<?php _('Connecting to WLAN failed.') ?>");
    });
  }

  disconnectEdit() {
    this.edit.data.active = false;
    this.con.send("wifiSetSSIDData", this.edit.data);
    this.page = 0;
    this.edit = undefined;
  }

  saveEdit() {
    this.con.send("wifiSetSSIDData", this.edit.data);
  }

  connect(w: any) {
    const msgId: string = this.visualAid.showBusy("<?php _('Connecting ...') ?>");
    setTimeout(() => {
      this.visualAid.hideBusy(msgId);
    }, 30000);
    this.wifiAction.connect(w.SSID).then((data) => {
      this.visualAid.hideBusy(msgId);
      if (!data.connected) { // password required
        this.stringKbd.getString("", "<?php _('Password') ?>", {}).then((password) => {
          const msgId2 = this.visualAid.showBusy("<?php _('Connecting ...') ?>");
          setTimeout(() => {
            this.visualAid.hideBusy(msgId2);
          }, 30000);
          this.wifiAction.connectPassword(w.SSID, password).then((data2) => {
            this.visualAid.hideBusy(msgId2);
            if (!data2.connected) {
              this.modal.message("<?php _('Connection') ?>", "<?php _('Connecting to WLAN failed.') ?>");
            }
          }, () => {
            this.visualAid.hideBusy(msgId2);
          });
        }, () => { // success
        });
      }
    }, () => {
      this.visualAid.hideBusy(msgId);
    });
  }

  setEditEncryption(e) {
    this.edit.data.passwordMethod = e;
    this.page = 5;
  }

  passwordMethod() {
    for (const i of this.encryptionModes) {
      if (i.v === this.edit.data.passwordMethod) {
        return i.n;
      }
    }
    return "Unknown";
  }

  canConnect() {
    if (this.edit.data.passwordMethod === "none") {
      return true;
    }
    return this.edit.data.password.length > 0;
  }

  changeEditPassword() {
    this.stringKbd.getString(this.edit.data.password, "<?php _('Password') ?>", {
      password: true,
      minLength: 8,
      maxLength: 63
    }).then((password) => {
      this.edit.data.password = password;
    });
  }

  changeEditV4Address() {
    this.networkKbd.getNetwork(this.edit.data.ipv4Address, "<?php _('Additional Address') ?>", {validate: "v4Address"}).then((ipv4Address) => {
      this.edit.data.ipv4Address = ipv4Address;
    }, () => {});
  }

  changeEditV4Nameserver() {
    this.networkKbd.getNetwork(this.edit.data.ipv4Nameserver, "<?php _('Additional Nameserver') ?>", {validate: "v4Address"}).then((ipv4Nameserver) => {
      this.edit.data.ipv4Nameserver = ipv4Nameserver;
    }, () => {});
  }

  changeEditV4Gateway() {
    this.networkKbd.getNetwork(this.edit.data.ipv4Gateway, "<?php _('Gateway') ?>", {validate: "v4Address"}).then((ipv4Gateway) => {
      this.edit.data.ipv4Gateway = ipv4Gateway;
    }, () => {});
  }

  changeEditV4Mask() {
    this.intKbd.getInt(this.edit.data.ipv4MaskBits, "<?php _('Network Mask Bits') ?>", {min: 1, max: 32}).then((ipv4bits) => {
      this.edit.data.ipv4MaskBits = parseInt(ipv4bits, 10);
    }, () => {});
  }
}
