/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, ViewContainerRef, ChangeDetectionStrategy, OnDestroy, OnInit} from "@angular/core";
import {
  RepetierServer, Language, WifiState, LicenseState, LicenseAction, ConnectionHandler,
  ServerAction, RepetierState, VisualAids, ServerEvent, ServerState, getState
} from "binding";
import {RSModal, RSModalDialog, RSUIVisualAids, RSLicenceKeyboard} from "front-ui";
import {Observable, SubscriptionLike} from "rxjs";
import {ExternalCommand, PrintersState} from "binding";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
    selector: "rs-front-register",
    template: `
<rs-page>
<rs-side>
    <rs-sideicon title="<?php _('Home') ?>" icon="rs rs-key" (rsClick)="page=0"></rs-sideicon>
</rs-side>
<rs-side>
    <rs-sideicon *ngIf="(wifi$ | async).manageable && (server$|async).features['config-network']" icon="fa fa-wifi" title="WLAN" link="/wlan"></rs-sideicon>
</rs-side>
<rs-side>
    <rs-sideicon title="<?php _('Language') ?>" icon="fa fa-language" (rsClick)="page=1"></rs-sideicon>
</rs-side>
<main>
<div *ngIf="page===0">
    <h1 class="margin-bottom"><i class="rs rs-key"></i> <?php _('License Activation') ?></h1>
    <div class="content-h1 margin" rsDrag>
        <div *ngIf="!online" class="alert warning">
            <i class="fa fa-warning"></i> {{tActivationNeedsInternet}}
        </div>
      <div *ngIf="(license$ | async).rentedExpired" class="alert warning">
        <i class="fa fa-warning"></i>
        <?php _('Your license was not verified for a month. Please ensure an internet connection to verify the validity!') ?>
        <?php _('Until verification you can only use the free features.') ?>
      </div>
        <p> <?php _('The requested function requires a active licence.') ?></p>
        <div class="alert error" style="margin-bottom:15px" *ngIf="(license$ | async).licenceError">
            <i class="fa fa-warning"></i> {{(license$ | async).licenceError}}
        </div>
        <rs-button class="secondary" (hit)="enterLicenece()"><?php _('Enter Licence Code') ?></rs-button>

        <div *ngIf="(license$ | async).testperiodMode<2 && (license$ | async).testperiodMode>=0" class="alert info margin-top">
                <p><?php _('You use the free version of this software, which contains all the essential functions for printing. The Pro version offers significantly more features and possibilities. Activate the Pro version now without any obligations for 14 days!') ?><br><br>
                  <?php _('No contact or payment information is required, after the trial period expires it will automatically convert to the free version and you can continue working normally.') ?></p>
        <p>{{tTrialNeedsInternet}}</p>
<rs-button type="button" class="info" (hit)="startPeriod()">
            <i class="fa fa-check"></i> {{tStartFreeTrial}}</rs-button>
        </div>

        <div class="centered-grid" style="margin-top:10px">
            <div  *ngFor="let n of net.ipv4" class="col-2" style="min-width:200px">
                <h2>{{connectionHandler.httpType}}://{{n.ip}}:{{net.port}}</h2>
                <div style="text-align: center;margin:8px">
                    <p>MAC: {{n.mac}}</p>
                    <rs-qrcode [ip]="n.ip" [port]="net.port" margin="2" modSize="5"></rs-qrcode>
                </div>
            </div>
         </div>
    <div class="iconlist">
        <div class="button-icon sep-icon" *ngFor="let f of filtered(commands$ | async)" style="min-width:12em">
            <rs-button class="secondary outline full" (hit)="runCommand(f)">
            <rs-dyn-icon [icon]="(f.icon ? f.icon : 'fa fa-play')"></rs-dyn-icon>
            <h3 class="text" style="padding:8px">{{f.name}}</h3></rs-button>
        </div>
    </div>

    </div>

</div>
<div *ngIf="page===1">
    <h1><?php _('Language') ?></h1>
    <div class="content-h1" rsDrag>
       <div class="iconlist">
           <div class="button-icon sep-icon-always full" *ngFor="let l of languages$ | async">
                <rs-button class="secondary outline full" (hit)="selectLanguage(l)">
                    <i *ngIf="(front$ | async).frontSetting"><rs-radio [onValue]="l.id" [value]="(front$ | async).frontSetting.language" class="primary"></rs-radio></i>
                    <h3 class="text single"><img [src]="l.flag"> {{l.name}} {{(front$ | async).language}}</h3>
                </rs-button>
           </div>
        </div>
    </div>
</div>
</main>
</rs-page>

 `,
    changeDetection: ChangeDetectionStrategy.Default
})
export class FrontRegisterComponent implements OnDestroy {
    languages$: Observable<Language[]>;
    license$: Observable<LicenseState>;
    wifi$: Observable<WifiState>;
    front$: Observable<any>;
    commands$: Observable<ExternalCommand[]>;
    printers$: Observable<PrintersState>;
    server$: Observable<ServerState>;
    private printersSubscription: SubscriptionLike;
    active = "";

    public page = 0;
    online: boolean = window.navigator.onLine;
    net: any = {ipv4: []};
    private connectionSubscription: SubscriptionLike;
    tActivationNeedsInternet = "<?php _('It seems you are not connected with the internet. Activation will work only with active internet connection!') ?>";
    tTrialNeedsInternet = "<?php _('Note: The free trial only works with active internet connection!') ?>";
    tStartFreeTrial = "<?php _('Start Free Trial') ?>";

    constructor(public server: RepetierServer,
                public modal: RSModal,
                public view: ViewContainerRef,
                public busy: RSUIVisualAids,
                private kbd: RSLicenceKeyboard,
                private licenseAction: LicenseAction,
                public connectionHandler: ConnectionHandler,
                private serverAction: ServerAction,
                private visualAid: VisualAids,
                private store: Store<RepetierState>) {
      this.languages$ = store.select(s => s.server.languages);
      this.license$ = store.select('license');
      this.server$ = store.select("server");
      this.wifi$ = store.select('wifi');
      this.front$ = store.select(s => s.server.settings);
      this.commands$ = store.select(s => s.server.externalCommands);
      this.printers$ = store.select('printers');
        // RSModal.defaultView = view;
        this.connectionSubscription = connectionHandler.connectionState.subscribe(on => {
            if (on) {
                this.updateInterfaces();
            }
        });
        this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
            this.active = p.active;
        });
    }

    private localEventHandler = (evt: ServerEvent) => {
        switch (evt.event) {
            case "connected":
                this.updateInterfaces();
                break;
        }
    }

    startPeriod() {
        this.connectionHandler.send("setTestperiodMode", {mode: 2});
    }

    updateInterfaces() {
        this.connectionHandler.networkInterfaces().then((data) => {
            this.net = data;
        }, () => {
        });
    }

    ngOnDestroy() {
        this.connectionSubscription.unsubscribe();
        this.printersSubscription.unsubscribe();
    }

    selectLanguage(l: Language) {
        this.serverAction.updateSetting("frontSetting", Object.assign({}, getState(this.store).server.settings["frontSetting"], {language: l.id})).then(() => {
            this.visualAid.showBusy("<?php _('Switching language ...') ?>");
            window.location.assign(window.location.pathname + "?rand=" + Math.random());
        });
    }

    enterLicenece() {
        this.kbd.getString("", "<i class=\"rs rs-key\"></i> " + "<?php _('License Key') ?>", {}).then(
            (v) => {
                this.licenseAction.activateLicense(v, true);
            }, () => {
            }
        );
    }

    filtered(list) {
        const l = [];
        for (const x of list) {
            if (x.local && (x.slug === "" || x.slug === this.active)) {
                l.push(x);
            }
        }
        return l;
    }

    runCommand(cmd: ExternalCommand) {
        if (cmd.confirm) {
            this.modal.confirm("<?php _('Security Question') ?>", cmd.confirm, {}).then(
                (res) => this.serverAction.runExternalCommand(cmd), () => {
                }
            );
        } else {
            this.serverAction.runExternalCommand(cmd);
        }
    }
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
    selector: "rs-register-app",
    template: `<router-outlet></router-outlet>`
})
export class RegisterAppComponent implements OnInit, OnDestroy {
    license$: Observable<LicenseState>;
    private licenseSubscription: SubscriptionLike;
    wifi: WifiState = undefined;
    serverState: ServerState = undefined;
    screenTimer: any = undefined;
    screenTimeout = -1;
    screenHidden = false;
    screenEnergySavingInitalized = false;
    private serverSubscription: SubscriptionLike;
    private wifiSubscription: SubscriptionLike;
    server$: Observable<ServerState>;
    wifi$: Observable<WifiState>;

    constructor(public server: RepetierServer,
                public modal: RSModal,
                public modalDialog: RSModalDialog,
                public view: ViewContainerRef,
                private connectionHandler: ConnectionHandler,
                private serverAction: ServerAction,
                private store: Store<RepetierState>) {
      this.server$ = store.select('server');
      this.wifi$ = store.select('wifi');
      this.license$ = store.select('license');
        RSModal.defaultView = view;
        RSModalDialog.defaultView = view;
        this.serverAction.initSetting("screensaver", {timeout: 10, energySaving: true});
        this.serverSubscription = this.server$.subscribe((s: ServerState) => {
            this.serverState = s;
            if (s.settings && s.settings["screensaver"] && s.settings["screensaver"].timeout !== this.screenTimeout) {
                this.screenTimeout = s.settings["screensaver"].timeout;
                this.initEnergySaving();
                this.resetScreensaver();
            }
        });
        this.wifiSubscription = this.wifi$.subscribe((w: WifiState) => {
            this.wifi = w;
            this.initEnergySaving();
        });
        window.document.body.addEventListener("mousemove", this.resetScreensaver);
        document.getElementById("screensaver").addEventListener("click", this.disableScreensaver);
        this.checkLicense();
    }

    ngOnInit() {
        this.connectionHandler.connectFrontend();
    }

    ngOnDestroy() {
        document.getElementById("screensaver").removeEventListener("click", this.disableScreensaver);
        window.document.body.removeEventListener("mousemove", this.resetScreensaver);
        this.wifiSubscription.unsubscribe();
        this.serverSubscription.unsubscribe();
        this.licenseSubscription.unsubscribe();
        this.connectionHandler.close();
        this.server.dispose();
    }


    initEnergySaving() {
        if (this.screenEnergySavingInitalized || !this.wifi || this.screenTimeout < 0 || !this.wifi.screensaver) {
            return;
        }
        this.screenEnergySavingInitalized = true;
        if (this.screenTimeout === 0 || !this.serverState.settings["screensaver"].energySaving) {
            this.connectionHandler.send("screensaver", {enable: 0, timeout: 0});
        } else {
            this.connectionHandler.send("screensaver", {enable: 1, timeout: this.screenTimeout * 60 + 10});
        }
    }

    triggerScreensaver = () => {
        this.screenTimer = undefined;
        this.screenHidden = true;
        this.connectionHandler.send("setScreensaver", {on: true});
        document.getElementById("screensaver").style.display = "block";
    };

    disableScreensaver = (event: MouseEvent) => {
        if (this.screenHidden) {
            this.screenHidden = false;
            this.connectionHandler.send("setScreensaver", {on: false});
            document.getElementById("screensaver").style.display = "none";
            this.screenTimer = setTimeout(this.triggerScreensaver, this.screenTimeout * 60000);
        }
        event.stopPropagation();
    };

    resetScreensaver = () => {
        if (this.screenHidden) {
            return;
        }
        if (this.screenTimer) {
            window.clearTimeout(this.screenTimer);
        }
        if (this.screenTimeout <= 0) {
            return;
        } // disabled
        this.screenTimer = setTimeout(this.triggerScreensaver, this.screenTimeout * 60000);
    }

    private checkLicense() {
        this.licenseSubscription = this.license$.subscribe(license => {
            if (!license.dataLoaded) {
                return;
            }
            if (license.licensed || license.testperiodMode === 2) {
                // redirect to frontend when we have a license!
              // console.log("move app to frontend",license)
              if(license.features && !license.rentedExpired) {
                 window.location.href = license.printerFrontendUrl;
              }
            }
        });
    }
}
